/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: InterfaceContext.java,v 1.1 2004/04/14 09:43:14 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.context;

import java.util.List;
import java.util.Vector;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.naming.AbstractContext;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;

/**
 * Basic interface context.
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class InterfaceContext 
     extends AbstractContext 
{
    /** the interface wrapper reference. */
    protected InterfaceWrapper interfaceWrapper ;
    
    /**
     * @return Returns the interfaceWrapper.
     */
    protected InterfaceWrapper getInterfaceWrapper() {
        return interfaceWrapper;
    }
    
    /**
     * @param interfaceWrapper The interfaceWrapper to set.
     */
    protected void setInterfaceWrapper(InterfaceWrapper interfaceWrapper) {
        this.interfaceWrapper = interfaceWrapper;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.browser.api.Context#getEntries()
     */
    public Entry[] getEntries(Interface itf) {
        List l = new Vector();
        Component ci = itf.getFcItfOwner();
        Interface bindInterface = null;
        try{
            BindingController bc = Fractal.getBindingController(ci);
            bindInterface = (Interface)bc.lookupFc(itf.getFcItfName());
        }catch(Exception e) {
            System.err.println("Client interface Error : " + e.getMessage());   
        }
        if(bindInterface!=null)
            l.add(new DefaultEntry(bindInterface, 
                                   new DefaultName(NameProvider.singleton.getPrefixedName(bindInterface)), 
                                   this));
        return (Entry[])l.toArray(new Entry[0]);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.browser.core.naming.AbstractContext#getEntries()
     */
    public Entry[] getEntries() {
        return getEntries(getInterfaceWrapper().getItf());
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.browser.core.naming.AbstractContext#setWrapped(java.lang.Object)
     */
    public void setWrapped(Object object) {
        setInterfaceWrapper((InterfaceWrapper) object);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.browser.core.naming.AbstractContext#getWrapped()
     */
    public Object getWrapped() {
        return getInterfaceWrapper();
    }
}
