/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Philippe Merle, Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: BindInterfaceOnDropAction.java,v 1.2 2004/04/14 09:43:14 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.menu;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.browser.api.DropAction;
import org.objectweb.util.browser.api.DropTreeView;
import org.objectweb.util.browser.plugins.fractal.context.ClientCollectionInterfaceContainer;
import org.objectweb.util.browser.plugins.fractal.context.NameProvider;

/**
 * Action for binding an interface on drop.
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * @version 0.1
 */
public class BindInterfaceOnDropAction 
  implements DropAction 
{
    
    //==================================================================
    //
    // No internal state.
    //
    //==================================================================
    
    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================
    
    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================
    
    protected String getName(Interface itf, int number) {
        String name = NameProvider.singleton().getName(((InterfaceType)itf.getFcItfType()));
        return name + ((number<10)?"0":"") + number; 
    }
    
    //==================================================================
    //
    // No public method.
    //
    //==================================================================
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.DropAction#execute(org.objectweb.util.browser.api.DropTreeView)
     */
    public void execute(DropTreeView dropTreeView) throws Exception {
        try {
            Interface itfToBind = (Interface) dropTreeView.getDragSourceObject();
            ClientCollectionInterfaceContainer icc = (ClientCollectionInterfaceContainer) dropTreeView.getSelectedObject();
            int nbConnectedItf = icc.getEntries().length;
            Interface collectionItf = icc.getItf();
            Component component = collectionItf.getFcItfOwner();
            BindingController bindingController = Fractal.getBindingController(component);
            bindingController.bindFc(getName(collectionItf, nbConnectedItf), itfToBind);          
        } catch(ClassCastException e) {
            throw new Exception("Interface expected!");
        }
    }
    
}


