/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: CreateInterfaceTypeAction.java,v 1.2 2004/04/14 09:43:14 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.menu;

import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.BooleanBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;
import org.objectweb.util.browser.gui.lib.LabelBox;

/**
 * This action allows to create a new interface type.
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class CreateInterfaceTypeAction 
  implements MenuItem, 
             DialogAction 
{
    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    /** The boolean boxes. */
    protected BooleanBox isClientBox_, isOptionalBox_, isCollectionBox_;
    
    /** The label boxes. */
    protected LabelBox nameBox_, signatureBox_;
    
    /** The context container where adding the type. */
    protected ContextContainer cc_;
    
    /** The type factory used to create a type. */
    protected TypeFactory typeFactory_;
    
    //==================================================================
    //
    // No constructor.
    //
    //==================================================================
    
    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================
    
    /**
     * Creates a box containing all the box to specify all the params
     */
    protected void createBox(DialogBox dialogBox) {
        nameBox_ = new LabelBox("Name");
        dialogBox.addElementBox(nameBox_);
        signatureBox_ = new LabelBox("Signature");
        dialogBox.addElementBox(signatureBox_);
        isClientBox_ = new BooleanBox("Type","Client","Server",true);
        dialogBox.addElementBox(isClientBox_);
        isOptionalBox_ = new BooleanBox("Contingency","Optional","Mandatory",true);
        dialogBox.addElementBox(isOptionalBox_);
        isCollectionBox_ = new BooleanBox("Cardinality","Collection","Single",true);
        dialogBox.addElementBox(isCollectionBox_);
    }
    
    //==================================================================
    //
    // Public methods for ExtendedActionListener interface.
    //
    //==================================================================
    
    public void actionPerformed(MenuItemTreeView e) throws Exception {     
        typeFactory_ = Fractal.getTypeFactory(Fractal.getBootstrapComponent());
        cc_ = (ContextContainer)e.getSelectedObject();
        DialogBox dialog = new DefaultDialogBox("Create a new interface type");      
        createBox(dialog);
        dialog.setValidateAction(this);
        dialog.show();
    }
    
    /**
     * @see org.objectweb.util.browser.api.MenuItem#isActive(org.objectweb.util.browser.gui.common.TreeView)
     */
    public int getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;
    }
    
    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================
    
    /**
     * Executes an action.
     */
    public void executeAction() throws Exception {
        String name = nameBox_.getLabel();
        String signature = signatureBox_.getLabel();
        boolean isClient = isClientBox_.getValue();
        boolean isOptional = isOptionalBox_.getValue();
        boolean isCollection = isCollectionBox_.getValue();
        cc_.addEntry(name,typeFactory_.createFcItfType(name,signature,isClient,isOptional,isCollection));
    }
    
}