/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;

/**
 * This action allows you to configure the visibility on a class. 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class GlobalClassVisibilityConfigAction 
    implements MenuItem, DialogAction {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    protected Class class_;

    protected ClassVisibilityBox classVisibilityConfig_;

    //==================================================================
    //
    // Constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for MenuItem interface
    //
    //==================================================================

    public int getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;
    }

    public void actionPerformed(MenuItemTreeView treeView) 
        throws Exception {
    
        class_ = (Class)treeView.getSelectedObject();
        
        ClassVisibilityConfig config = (ClassVisibilityConfig)ClassVisibility.config_.get(ClassVisibility.DEFAULT_CONFIG);
        if(config == null)
            config = ClassVisibility.createNewDefaultConfig();    

        classVisibilityConfig_ = new ClassVisibilityBox(config);
            
        DialogBox dialog = new DefaultDialogBox("Class visibility");  
        dialog.addElementBox(classVisibilityConfig_);
        dialog.setValidateAction(this);
        dialog.show();

    }

    //==================================================================
    //
    // Public method for DialogAction interface.
    //
    //==================================================================
    
    public void executeAction() throws Exception {
        ClassVisibility.config_.put(ClassVisibility.DEFAULT_CONFIG, classVisibilityConfig_.getConfig());
    }

}