/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import org.objectweb.util.browser.api.IconProvider;
import org.objectweb.util.browser.core.common.ClassResolver;

/**
 * Provides an icon to represent <code>java.lang.reflect.Method</code> object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class MethodIconProvider 
    implements IconProvider {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The icons to use for public method. */
    protected static Icon publicMethod_, publicAbstractMethod_, publicStaticMethod_, publicFinalMethod_, publicStaticFinalMethod_; 
    
    /** The icons to use for protected method. */
    protected static Icon protectedMethod_, protectedAbstractMethod_, protectedStaticMethod_, protectedFinalMethod_, protectedStaticFinalMethod_;

    /** The icons to use for package class. */
    protected static Icon packageMethod_, packageAbstractMethod_, packageStaticMethod_, packageFinalMethod_, packageStaticFinalMethod_; 
    
    /** The icons to use for private method. */
    protected static Icon privateMethod_, privateFinalMethod_;

    //==================================================================
    //
    // Constructor.
    // 
    //==================================================================

    /**
     * Default constructor
     */
    public MethodIconProvider(){
        publicMethod_ = new ImageIcon(ClassResolver.getResource("publicMethod.png"));
        publicAbstractMethod_ = new ImageIcon(ClassResolver.getResource("publicAbstractMethod.png"));
        publicStaticMethod_ = new ImageIcon(ClassResolver.getResource("publicStaticMethod.png"));
        publicFinalMethod_ = new ImageIcon(ClassResolver.getResource("publicFinalMethod.png"));
        publicStaticFinalMethod_ = new ImageIcon(ClassResolver.getResource("publicStaticFinalMethod.png"));
        protectedMethod_ =  new ImageIcon(ClassResolver.getResource("protectedMethod.png"));
        protectedAbstractMethod_ = new ImageIcon(ClassResolver.getResource("protectedAbstractMethod.png"));
        protectedStaticMethod_ = new ImageIcon(ClassResolver.getResource("protectedStaticMethod.png"));
        protectedFinalMethod_ = new ImageIcon(ClassResolver.getResource("protectedFinalMethod.png"));
        protectedStaticFinalMethod_ = new ImageIcon(ClassResolver.getResource("protectedStaticFinalMethod.png"));
        privateMethod_ =  new ImageIcon(ClassResolver.getResource("privateMethod.png"));
        privateFinalMethod_ =  new ImageIcon(ClassResolver.getResource("privateFinalMethod.png"));
        packageMethod_ = new ImageIcon(ClassResolver.getResource("packageMethod.png"));
        packageAbstractMethod_ = new ImageIcon(ClassResolver.getResource("packageAbstractMethod.png")); 
        packageStaticMethod_ = new ImageIcon(ClassResolver.getResource("packageStaticMethod.png"));
        packageFinalMethod_ = new ImageIcon(ClassResolver.getResource("packageFinalMethod.png"));
        packageStaticFinalMethod_ = new ImageIcon(ClassResolver.getResource("packageStaticFinalMethod.png"));
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for IconProvider interface.
    //
    //==================================================================

    /**
     * Return an <code>javax.swing.Icon</code> according to the given object.
     *
     * @param object The object to compute the associated icon.
     * 
     * @return The icon according to the given object.
     */
    public Icon 
    newIcon(Object object){
        Method method_ = (Method)object;
        int modifier = method_.getModifiers();
        if (Modifier.isPrivate(modifier)){
            if (Modifier.isFinal(modifier))
                return privateFinalMethod_;
            else
                return privateMethod_;                
        } else if (Modifier.isProtected(modifier)){
            if (Modifier.isAbstract(modifier)) 
                return protectedAbstractMethod_;
            else if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return protectedStaticFinalMethod_;
            else if (Modifier.isStatic(modifier))
                return protectedStaticMethod_;
            else if (Modifier.isFinal(modifier))
                return protectedFinalMethod_;
            else
                return protectedMethod_;                
        } else if (Modifier.isPublic(modifier)){
            if (Modifier.isAbstract(modifier)) 
                return publicAbstractMethod_;
            else if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return publicStaticFinalMethod_;
            else if (Modifier.isStatic(modifier))
                return publicStaticMethod_;
            else if (Modifier.isFinal(modifier))
                return publicFinalMethod_;
            else
                return publicMethod_;                
        } else if (!Modifier.isPrivate(modifier) && !Modifier.isProtected(modifier) && !Modifier.isPublic(modifier)){
            if (Modifier.isAbstract(modifier)) 
                return packageAbstractMethod_;
            else if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return packageStaticFinalMethod_;
            else if (Modifier.isStatic(modifier))
                return packageStaticMethod_;
            else if (Modifier.isFinal(modifier))
                return packageFinalMethod_;
            else
                return packageMethod_;                
        } else {
            return null;   
        }
    }
    
}