/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;

/**
 * This class allows to browse a <code>TreeInheritance</code> object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class TreeInheritanceContext 
    implements Wrapper, Context {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The class to browse */
    protected TreeInheritance treeInheritance_;

    /** Contains the configuration for each class */
    protected static Map config_;

    /** The key of the default configuration entry */
    public static final String DEFAULT_CONFIG = "_default_config_";

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for Context interface.
    //
    //==================================================================

    /**
     * Returns an array containing the entries contained by the context.
     * @return The entries of the context.
     */
    public Entry[] getEntries() {
        List entryList = new Vector();
        Class class_ = treeInheritance_._getClass();
        Class superClass = class_.getSuperclass();
        if(superClass!=null){
            entryList.add(new DefaultEntry(superClass,new DefaultName(superClass.getName()),this));
        }
        Class[] interfaces = class_.getInterfaces();
        for (int i = 0; i < interfaces.length; i++) {
            entryList.add(new DefaultEntry(interfaces[i],new DefaultName(interfaces[i].getName()),this));
		}
        return (Entry[])entryList.toArray(new Entry[0]);        
    }

    //==================================================================
    //
    // Public methods for Wrapper interface.
    //
    //==================================================================

    /**
     * Sets the wrapped object.
     * 
     * @param object The object to wrap.
     */
    public void 
    setWrapped(Object object) {
        treeInheritance_ = (TreeInheritance)object;
    }

    /**
     * Gets the wrapped object.
     * 
     * @return The wrapped object.
     */
    public Object 
    getWrapped() {
        return treeInheritance_;
    }

}