/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.browser.core.common;

import org.objectweb.util.browser.core.api.Role;

/**
 * Default implementation of a role.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 *
 */ 
public class DefaultRole 
  implements Role {

    /** The id of the role. */
    protected String role_id_;
    
    /** Indicates if this role must be visible in the "role" menu */
    protected boolean isConcrete_ = true;
    
    /**
     * Default constructor.
     * @param role_id The id of the role.
     */
    public DefaultRole(String role_id){
        role_id_ = role_id;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.browser.core.api.Role#getId()
     */
    public String getId() {
        return role_id_;
    }

    /**
     * Indicates the kind of the role (concrete or abstract). The default value is true.
     */
     public void setConcrete(boolean isConcrete){
        isConcrete_ = isConcrete;    
    }
    
    /**
     * Indicates if this role is concrete or not. A concrete role is visible in the "role" menu. The default value is true.
     */
    public boolean isConcrete(){
        return isConcrete_;
    }
    
    public boolean equals(Object obj){
        if(obj!=null){
            return ((DefaultRole)obj).getId().trim().equals(role_id_.trim());
        }
        return false;
    }
    
    public String toString(){
        return "DefaultRole[id=" + this.getId() + "]";   
    }
}
