/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.common;

/** The console's imports */
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultContextContainerFactory;
import org.objectweb.util.browser.core.naming.InitialEntry;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.DropAction;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Table;
import org.objectweb.util.browser.api.Tree;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.api.Panel;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.api.Info;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.api.ContextContainerFactory;
import org.objectweb.util.browser.core.api.ContextProperty;
import org.objectweb.util.browser.core.api.ExtendedEntry;
import org.objectweb.util.browser.core.api.Role;
import org.objectweb.util.browser.core.api.RoleManagement;
import org.objectweb.util.browser.core.api.StatusBar;
import org.objectweb.util.browser.core.api.ViewPanel;
import org.objectweb.util.browser.core.api.TreeConfiguration;
import org.objectweb.util.browser.core.dnd.DefaultDropTreeView;
import org.objectweb.util.browser.core.dnd.EntryViewTransferable;
import org.objectweb.util.browser.core.panel.WhitePanel;
import org.objectweb.util.browser.core.xmlparser.ContextXMLParser;
import org.objectweb.util.browser.core.xmlparser.XMLParser;
import org.objectweb.util.browser.core.xmlparser.BrowserXMLParser;

import org.objectweb.util.browser.core.popup.DefaultAction;

/** The java API's imports */
import java.util.Arrays;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.io.IOException;

/** The Drag and Drop API's imports */
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.Autoscroll;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceContext;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.dnd.InvalidDnDOperationException;

/** The java swing API's imports */
import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.JPopupMenu;
import javax.swing.Icon;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/** The java swing tree API's imports */
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * Implementation of a Entry tree which is dynamic. 
 * You can customize icons, panels and popup menus 
 * for every objects you display.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DynamicTree 
    extends JTree 
    implements TreeConfiguration, Tree, Autoscroll, RoleManagement {

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** To access to the customization */
    protected AdminCustomization custom;

    /** Parent panel, use to refresh right panel */
    protected ViewPanel viewPanel_;

    /** Root node */
    protected DefaultMutableTreeNode rootNode;

    /** The graphical initial context (The entries might be replaced by their associated context) */
    protected ContextContainer graphicalInitialContext_; 
    
    /** The inital context. */
    protected Vector initialContext_;

    /** Model to manipulate node */
    protected DefaultTreeModel treeModel;

    /** Model to define what to do when a node is selected */
    protected DefaultTreeSelectionModel selectionModel;

    /** Model to display node */
    protected DefaultTreeCellRenderer treeCellRenderer;

    /** A map which informs about the node state (expanded or collapsed) */
    protected Map nodeState_;

    /** To know the selected node */
    protected String selectedNode_;

    /** To know the selected treepath*/
    protected TreePath selectedTreePath_;

    /** Panel display when any panel was found */
    protected Panel blankPanel_;

    /** The associated contextProperty */
    protected ContextProperty contextProperty_;

    /** The associated browserProperty */
    protected XMLParser browserProperty_;
    
    /** The current TreeView*/
    protected TreeView currentTreeView_;
    
    /** To know if the popup menus have to be enabled or not */
    protected boolean popupEnabled_;
    
    /** To know if the Drag and Drop have to be enabled or not */
    protected boolean dndEnabled_;
    
    /** To know if the application allows the selection of several roles at the same time. */
    protected boolean multipleRolesEnabled_;
    
    /** The status bar */
    protected StatusBar statusBar_;
    
    /** The MenuBar to use. */
    protected JMenuBar menuBar_;

    /** The ToolBar to use. */
    protected JToolBar toolBar_ = null;
    
    /** The number of element of the toolBar which has to be removed. */
    protected int nbElementsToRemove_ = 0;

    /** The action menu. */
    protected JMenu actionMenu_, roleMenu_;
    
    /** The PopupMenu corresponding to the selected node. */
    protected JPopupMenu currentJPopupMenu_ = null;

    /** List of elements which have already been added into the initial context. */
    protected Vector alreadyInInitialContext_ = null;
                
    // Drag and Drop attributes

    /** The DragSource to use */
    protected DragSource dragSource_;
    
    /** The DragGestureListener to use */
    protected DragGestureListener dragGestureListener_;
    
    /** The DragSourceListener to use */
    protected DragSourceListener dragSourceListener_;
    
    /** The dragAction */
    protected int dragAction_ = DnDConstants.ACTION_COPY_OR_MOVE;
    
    /** The  allowed drop action */
    protected int acceptableActions_ = DnDConstants.ACTION_COPY_OR_MOVE;

    /** The DropTarget */
    protected DropTarget dropTarget_;
        
    /** The DropTargetListener to use */
    protected DropTargetListener dropTargetListener_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Calls by the constructor
     */
    private void buildDnD(){
        // Drag
        dragSource_ = DragSource.getDefaultDragSource();
        dragSourceListener_ = new TreeDragSourceListener();
        dragGestureListener_ = new TreeDragGestureListener();
        dragSource_.createDefaultDragGestureRecognizer(this, dragAction_, dragGestureListener_);
        
        // Drop
        dropTargetListener_ = new TreeDropTargetListener();
        dropTarget_ = new DropTarget(this, acceptableActions_, dropTargetListener_, true); 
    }

    /**
     * Builds a new DynamicTree
     * @param initialContext The context to be displayed
     * @param initAll Uses to know if the constructor may initialize all the attribute
     */
    private void build(ContextContainer initialContext, boolean initAll) {
        graphicalInitialContext_ = initialContext;

        rootNode = new DefaultMutableTreeNode(new DefaultEntry(graphicalInitialContext_,new DefaultName("Root"),null));
        treeModel = new MyTreeModel(rootNode);
        setModel(treeModel);

        if (initAll) {
            nodeState_ = new HashMap();
            initialContext_ = new Vector();
            alreadyInInitialContext_ = new Vector();
            selectedNode_ = "";
            blankPanel_ = new WhitePanel();
            selectionModel = new DefaultTreeSelectionModel();
            selectionModel.setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);
            setSelectionModel(selectionModel);
            addMouseListener(new MyMouseAdapter());
            addTreeWillExpandListener(new MyTreeWillExpandListener());
            addTreeExpansionListener(new MyTreeExpansionListener());
            addTreeSelectionListener(new MyTreeSelectionListener());
            setCellRenderer(new MyTreeCellRenderer());
            popupEnabled_ = true;
            dndEnabled_ = true;
            multipleRolesEnabled_ = false;
            buildDnD();
            setToolTipText("");
        }

        setShowsRootHandles(true);
        collapseRow(0);
        expandRow(0);

        setRootVisible(false);
    }

    /**
     * Constructs a new DynamicTree.
     */
    public DynamicTree() {
        ContextContainerFactory ccf = new DefaultContextContainerFactory(null);
        build(ccf.create(), true);
    }

    /**
     * Constructs a new DynamicTree initialized with the specified context.
     * @param initialContext The context to be displayed
     */
    public DynamicTree(ContextContainer initialContext) {
        build(initialContext, true);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Adds an entry to a node.
     * @param parent Node which is the parent of the new entry
     * @param child Entry to add
     * @return The created node
     */
    protected DefaultMutableTreeNode addObject(
        DefaultMutableTreeNode parent,
        ExtendedEntry child) {
        if (custom != null) {
            Context context = custom.getContext(child.getValue());
            if (context != null) {
                child.setOWValue(context);
                child.setWrappedObject(((Wrapper) context).getWrapped());
            }
        }
        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
        if (parent == null) {
            parent = rootNode;
        }
        treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

        return childNode;
    }

    /**
     * Gets a string representation of the path
     * 
     * @param node The node
     * @return The path from th root to the node
     * 
     */
    protected String pathToString(DefaultMutableTreeNode node) {
        StringBuffer sf = new StringBuffer();
        Object[] o = node.getUserObjectPath();
        for (int i = 0; i < o.length; i++) {
            sf.append(((Entry) o[i]).getName().toString());
            if (i != o.length - 1)
                sf.append("/");
        }
        return sf.toString();
    }

    /**
     * Returns the entry associates to a node
     * @param o The node
     * @return The associated entry
     */
    protected Entry objectToEntry(Object o) {
        if (o != null) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) o;
            return (Entry) node.getUserObject();
        }
        return null;
    }

    /**
     * Gives the path of the object pointing by the cursor
     * @param p The position of the cursor
     */
    protected TreePath getPath(Point p){
        if(p!=null){
            return getPathForLocation((int) p.getX(), (int) p.getY());
        }
        return null;
    }

    /**
     * Shows the associated PopupMenu, if exists and if displaying is enabled
     * @param p Point which indicates where is the mouse
     */
    protected void showMenu(Point p) {
        TreePath path = getPath(p);
        if (path != null) {
            selectionModel.setSelectionPath(path);
            Entry entry = objectToEntry(getLastSelectedPathComponent());
            if (custom != null) {
                JPopupMenu menu = custom.getMenu((ExtendedEntry) entry);
                if (menu != null) {
                    menu.show((Component) this,(int) (p.getX()),(int) (p.getY()));                                        
                }
            }
        }
    }

    /**
     * Gives the TreePath correspoding to the given id
     * @param id The id of the node of the tree
     * @return The associated TreePath or null
     */
    protected TreePath getPathFromInitialContext(String id) {
        if (id != null) {
            Enumeration enum = rootNode.children();
            while (enum.hasMoreElements()) {
                DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enum.nextElement();
                Entry entry = (Entry) childNode.getUserObject();
                if (entry.getName().toString().equals(id))
                    return new TreePath(childNode.getPath());
            }
        }
        return null;
    }

    /**
     * Copies the content of the initialContext into the graphicalIntialContext
     */
    protected void createGraphicalInitialContext() {
        // Clears the existing initial context.
        graphicalInitialContext_.clear();
        if(custom!=null){
            // Finds and copies the root context
            Entry[] entries = custom.getRoot();
            for (int i = 0 ; i < entries.length ; i++){
                InitialEntry initialEntry = null;
                try{
                    initialEntry = (InitialEntry)entries[i];
                } catch (ClassCastException e) {
                    initialEntry = new InitialEntry(entries[i].getName().toString(), entries[i].getValue(), 0);
                }
                addToInitialContext(initialEntry.getName().toString(), initialEntry.getValue(), initialEntry.getLevel());
            }
        }
        // Copies the content of the initialContext
        InitialEntry entryToAdd = null;
        for(int i=0 ; i<initialContext_.size() ; i++) {
            entryToAdd = (InitialEntry)initialContext_.get(i);
            addToInitialContext(entryToAdd.getName().toString(), entryToAdd.getValue(), entryToAdd.getLevel());
        }        
        
    }

    /**
     * Open the tree nodes until a specific level
     *
     * @param root The root TreePath to develop
     * @param level The level (>0 for a specific level)
     */
    protected void openTree(TreePath root, int level) {
        if (level > 0) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) root.getLastPathComponent();
            Enumeration enum = node.children();
            while (enum.hasMoreElements()) {
                DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enum.nextElement();
                TreePath p = new TreePath(childNode.getPath());
                expandPath(p);
                openTree(p, level - 1);
            }
            //  	}else if (level <0){
            //    	    for(int i=0;i<getRowCount();i++)
            //    		expandRow(i);
        }
    }

    /**
     * Refreshes the view panel
     */
    protected void refreshPanel(){
        if(viewPanel_!=null){
            Entry entry = null;
            Object object = getLastSelectedPathComponent();
            String newSelectedNode = null;
            if (object != null) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) object;
                if (!node.equals(rootNode)) {
                    newSelectedNode = pathToString(node);
                }
                entry = (Entry) node.getUserObject();
            }
            if (newSelectedNode==null || newSelectedNode.equals("")) {
                viewPanel_.setViewPanel(blankPanel_);            
            } else if (selectedNode_ != null && newSelectedNode != null && !selectedNode_.equals(newSelectedNode)) {
                // New selected node
                selectedNode_ = newSelectedNode;
                // Obtaining the current panel
                Panel currentPanel = viewPanel_.getViewPanel();
                if(currentPanel!=null && currentTreeView_!=null)
                    currentPanel.unselected(currentTreeView_);
                // Obtaining the new panel
                Panel panel = null;
                Table table = null;
                if (entry != null && custom != null) {
                    
                    panel = custom.getPanel((ExtendedEntry) entry);
                    if (panel != null) {
                        currentTreeView_ = new DefaultTreeView(this);
                        panel.selected(currentTreeView_);
                        viewPanel_.setViewPanel(panel);
                    }
                    
               /*     // Look for a panel for the wrapped object by the entry
                    panel = custom.getPanel(((ExtendedEntry)entry).getWrappedObject(), true);
                    if(panel==null) {
                        // Look for a table for the wrapped object by the entry
                        table = custom.getTable(((ExtendedEntry)entry).getWrappedObject(), true);
                    }
                    if(panel==null && table==null) {
                        // Look for a panel for the value of the entry
                        panel = custom.getPanel(entry.getValue());
                        if(panel==null) {
                            // Look for a table for the value of the entry
                            table = custom.getTable(entry.getValue());
                        }
                    }
                    // A table is defined
                    if(panel == null && table != null){
                        if(contextProperty_!=null)
                            panel = new DefaultTablePanel(table, custom, contextProperty_.getDecoder());
                        else
                            panel = new DefaultTablePanel(table, custom);
                    }
                    if (panel != null) {
                        currentTreeView_ = new DefaultTreeView(this);
                        panel.selected(currentTreeView_);
                        viewPanel_.setViewPanel(panel);
                    }
               */ 
                }
                if (entry == null || custom == null || panel == null) {
                    viewPanel_.setViewPanel(blankPanel_);
                }
            }   
        }        
    }

    /**
     * Refreshes the status bar depending on the selected object
     */
    protected void refreshStatusBar(){
        if(statusBar_!=null){
            Entry entry = null;
            Object object = getLastSelectedPathComponent();
            if (object != null) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) object;
                entry = (Entry) node.getUserObject();
            }
            if (entry != null && custom != null) {
                Info info = custom.getInfo((ExtendedEntry) entry);
                if (info != null) {
                    statusBar_.setText(info.getInfo(new DefaultTreeView(this)));
                } else {
                    statusBar_.setText("");
                }
            } else {
                statusBar_.setText("");
            }
        }
    }

    /**
     * Refreshes the action menu depending on the selected object.
     */
    protected void refreshActionMenu(){
        if(menuBar_!=null && actionMenu_ != null){
            Entry entry = null;
            Object object = getLastSelectedPathComponent();
            if (object != null) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) object;
                entry = (Entry) node.getUserObject();
            }
            if (entry != null && custom != null) {
                currentJPopupMenu_ = custom.getMenu((ExtendedEntry) entry);
                actionMenu_.removeAll();
                Component[] componentList = currentJPopupMenu_.getComponents();
                if(componentList.length>0){
                    actionMenu_.setEnabled(true);
                    for(int i=0; i<componentList.length;i++){
                        if(JPopupMenu.Separator.class.isAssignableFrom(componentList[i].getClass()))
                            actionMenu_.addSeparator();
                        else
                            actionMenu_.add((JMenuItem)componentList[i]);
                    }
                } else {
                    actionMenu_.setEnabled(false);
                }
                actionMenu_.revalidate();
            }    
        }
    }

    protected void refreshToolBar(){
        if(toolBar_!=null){
            // Removes the old elements
            if(nbElementsToRemove_>0){
                int nbComp = toolBar_.getComponentCount();
                for(int i = nbComp - 1 ; i >= nbComp - nbElementsToRemove_ ; i--)
                    toolBar_.remove(i);
                nbElementsToRemove_ = 0;
            }
            
            // Adds the new elements
            Entry entry = null;
            Object object = getLastSelectedPathComponent();
            if (object != null) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) object;
                entry = (Entry) node.getUserObject();
            }
            if (entry != null && custom != null) {
                currentJPopupMenu_ = custom.getMenu((ExtendedEntry) entry);
                Component[] componentList = currentJPopupMenu_.getComponents();
                if(componentList.length>0){
                    boolean isFirst = true;
                    boolean iconDisplayed = false;
                    for(int i=0; i<componentList.length;i++){
                        if(!JPopupMenu.Separator.class.isAssignableFrom(componentList[i].getClass())){
                            if(((DefaultAction)((JMenuItem)componentList[i]).getAction()).isUserIcon()){
                                if(isFirst){
                                    isFirst = false;
                                    if(toolBar_.getComponentCount()>0){
                                        toolBar_.addSeparator();
                                        nbElementsToRemove_++;
                                    }
                                }
                                toolBar_.add(((JMenuItem)componentList[i]).getAction());
                                iconDisplayed = true;
                                nbElementsToRemove_++;   
                            }
                        } else if(iconDisplayed){
                            toolBar_.addSeparator();
                            iconDisplayed = false;
                            nbElementsToRemove_++;
                        }
                    }
                }
            }
            if(toolBar_.getComponentCount()==0){
                toolBar_.setVisible(false);
            } else{
                toolBar_.setVisible(false);
                toolBar_.setVisible(true);
            }        
        }
    }

    /**
     * Returns true if the context has at least one child. 
     * @param context The context to check
     * @return true if the context has children
     */
    protected boolean hasChild(Context context) {
        if(context!=null){
            Entry[] entries = context.getEntries();
            return (entries.length > 0);
        }
        return false;
    }

    /**
     * Add an object into the initial context and expands the ndoe until a specific level if this one is a context
     *  
     * @param id Name of the object
     * @param object The object to add
     * @param level The level (>0 for a specific level)
     */
    protected void addToInitialContext(String id, Object object, int level) {
        if (id != null && object != null) {
            graphicalInitialContext_.addEntry(id, object);
            refreshPath(new TreePath(rootNode.getPath()));
            //TreePath path = getPathFromInitialContext(id);
            //refreshPath(path);
            if(!alreadyInInitialContext_.contains(id)){
                if (level > 0) {
                    TreePath path = getPathFromInitialContext(id);
                    if (path != null) {
                        expandPath(path);
                        openTree(path, level - 1);
                    }
                }
                alreadyInInitialContext_.add(id);
            }
        }
    }

    /**
     * Refreshes the liste of role in the menu item
     * @param roles
     */
    protected void refreshRoleMenu(Role[] roles){
        if(menuBar_!=null){
            Arrays.sort(roles, new RoleComparator());
            Role[] currentRoles = getCurrentRoles();
            if(roleMenu_==null){
                roleMenu_ = new JMenu("Roles");
                menuBar_.add(roleMenu_);
            }
            roleMenu_.removeAll();
            if(roles.length>0){
                roleMenu_.setVisible(true);
                if(multipleRolesEnabled_){
                    List theRoles = Arrays.asList(currentRoles);
                    for(int i=0; i<roles.length;i++){
                        if(roles[i].isConcrete()){
                            JCheckBoxMenuItem cbMenuItem = new JCheckBoxMenuItem(new RoleSelected(roles[i]));
                            cbMenuItem.setSelected(theRoles.contains(roles[i]));
                            roleMenu_.add(cbMenuItem);
                        }
                    }
                } else {
                    ButtonGroup group = new ButtonGroup();
                    JRadioButtonMenuItem rbMenuItem = null; 
                    for(int i=0; i<roles.length;i++){
                        if(roles[i].isConcrete()){
                            rbMenuItem = new JRadioButtonMenuItem(new RoleSelected(roles[i]));
                            if(currentRoles!=null)
                                rbMenuItem.setSelected(currentRoles[0].getId().equals(roles[i].getId()));
                            group.add(rbMenuItem);
                            roleMenu_.add(rbMenuItem);
                        }
                    }               
                }
            } else {
                roleMenu_.setVisible(false);
            }
        }
    }

    // ==================================================================
    //
    // Public methods overriden JComponent methods.
    //
    // ==================================================================
    
    public String getToolTipText(MouseEvent evt) {
        String label = "";
        TreePath currentPath = getPathForLocation(evt.getX(), evt.getY());
        if(currentPath!=null){
            Entry entry = objectToEntry(currentPath.getLastPathComponent());
            if(custom != null && entry!=null){
                label = custom.getDropLabel((ExtendedEntry) entry);
                if(label != null && label.length()>0){
                    label = "Drop action: " + label;
                }
            }
        }
        return label;    
    }

    // ==================================================================
    //
    // Public methods for TreeConfiguration interface.
    //
    // ==================================================================

    /** 
     * Set a new property context.
     * @param files An array of XML files to parse and to apply (It can be a single file or a directory)
     */
    public void setNewBrowserProperty(String[] files) {
        if (files != null) {
            XMLParser config = new BrowserXMLParser(this);
            config.setPropertyFile(files);
            setNewBrowserProperty((BrowserProperty) config);
            browserProperty_ = config;
        }
    }

    /** 
     * Set a new property context
     */
    public void setNewBrowserProperty(BrowserProperty properties) {
        if (properties != null) {
            properties.setContextProperty(contextProperty_);
            custom = new AdminCustomization(properties, this);
        }
        refreshRoleMenu(((RoleManagement)properties).getRoleList());
        refreshAll();
    }

    /**
     * Adds the new properties
     * @param files The properties files to add
     */
    public void addBrowserProperty(String[] files){
        if (files != null) {
            if(browserProperty_!=null){
                browserProperty_.setPropertyFile(files);
                setNewBrowserProperty((BrowserProperty) browserProperty_);
                refreshRoleMenu(((RoleManagement)browserProperty_).getRoleList());
            } else {
                setNewBrowserProperty(files);
            }
        }
        refreshAll();
    }

    /**
     * Gives the current BrowserProperty. 
     * @return The current BrowserProperty
     */
    public BrowserProperty getBrowserProperty() {
        return custom.getBrowserProperty();
    }

    /**
     * Creates a new context property with the given file
     * @param file The properties file to parse and to apply
     */
    public void setNewContextProperty(String file) {
        if (file != null && !file.equals("")) {
            XMLParser config = new ContextXMLParser();
            config.setPropertyFile(file);
            setNewContextProperty((ContextProperty) config);
        }
    }

    /**
     * Sets a new context property
     * @param properties The properties to apply 
     */
    public void setNewContextProperty(ContextProperty properties) {
        if (properties != null) {
            contextProperty_ = properties;
            // Updates the graphicalInitialContext_
            ContextContainerFactory ccf = new DefaultContextContainerFactory(properties.getDecoder());
            ContextContainer newContext = ccf.create();
            if (graphicalInitialContext_ != null) {
                Entry[] entries =  graphicalInitialContext_.getEntries();
                for (int i = 0; i < entries.length; i++) {
                    Entry entry = entries[i];
                    newContext.addEntry(entry.getName().toString(),entry.getValue());
                }
            }
            build(newContext, false);
        }
    }

    /**
     * Gives the current ContextProperty.
     * @return The current ContextProperty
     */
    public ContextProperty getContextProperty() {
        return contextProperty_;
    }

    /**
     * Returns the selected entry
     */
    public Entry getSelectedEntry() {
        Object object = getLastSelectedPathComponent();
        if (object != null)
            return objectToEntry(object);
        return null;
    }

    /**
     * Returns the parent of the selected entry
     */
    public Entry getSelectedEntryParent() {
        Object object = getLastSelectedPathComponent();
        if (object != null)
            return objectToEntry(((DefaultMutableTreeNode) object).getParent());
        return null;
    }

    /**
     * Refreshes the given path.
     */
    protected void refreshPath(TreePath treePath){
        collapsePath(treePath);
        expandPath(treePath);        
    }

    /**
     * Refresh all the tree
     */
    public void refreshAll() {
        createGraphicalInitialContext();
        TreePath p = new TreePath(rootNode.getPath());
        refreshPath(p);
        selectedNode_ = "";
        refreshPanel();
        refreshActionMenu();
        refreshToolBar();
        
    }

    /**
     * Refresh the selected node
     */
    public void refreshSelectedNode() {
        TreePath path = getSelectionPath();
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
        if (isExpanded(path)) {
            collapsePath(path);
            expandPath(path);
        } else if (node.isLeaf()) {
            fireTreeExpanded(path);
        }
        selectedNode_ = "";
        refreshPanel();
        refreshActionMenu();
        refreshToolBar();
    }

    /**
     * Gives the initial context 
     * @return The initial context
     */
    public Context getInitialContext() {
        return graphicalInitialContext_;
    }

    /**
     * Returns true if this initial context contains an entry for the specified id.
     * @param id Key whose presence in this initial context is to be tested.
     * @return True if this initial context contains an entry for the specified id.
     */
    public boolean containsEntry(String id) {
        Entry entry = graphicalInitialContext_.getLocalEntry(id);
        return entry != null;
    }

    /**
     * Fixes the view panel for the tree (Where the associated panels will be displayed).
     * @param viewPanel The panel to use to display associated panels
     */
    public void setTargetPanel(ViewPanel viewPanel) {
        viewPanel_ = viewPanel;
    }

    /**
     * Fixes the status bar for the tree (Where info will be displayed)
     * @param statusField
     */
    public void setStatusBar(StatusBar statusBar){
        statusBar_ = statusBar; 
    }

    /**
     * Indicates if the popup are enabled or not. By default, the popup menus are enabled.
     * @param popupEnabled True if the popup menu have to be enabled, false either.
     */
    public void setPopupEnabled(boolean popupEnabled){
        popupEnabled_ = popupEnabled;
    }
    
    /**
     * Returns true if the popup menus are enabled.
     * @return true if the popup menus are enabled.
     */
    public boolean isPopupEnabled(){
        return popupEnabled_;
    }

    /**
     * Indicates if the Drag and Drop is enabled or not. By default, the Drag and Drop is available.
     * @param dndEnabled True if the popup menu have to be enabled, false either.
     */
    public void setDragAndDropEnabled(boolean dndEnabled){
        dndEnabled_ = dndEnabled;
    }
        
    /**
     * Returns true if the Drag and Drop is enabled.
     * @return true if the Drag and Drop is enabled.
     */
    public boolean isDragAndDropEnabled(){
        return dndEnabled_;
    }
    
    /**
     * Indicates if the application allows the selection of several roles at the same time.
     * @param multipleRolesEnabled True if the application allows the selection of several roles at the same time, false either.
     */
    public void setMultipleRolesEnabled(boolean multipleRolesEnabled){
        multipleRolesEnabled_ = multipleRolesEnabled;
    }
    
    /**
     * Returns true if the application allows the selection of several roles at the same time.
     * @return true if the application allows the selection of several roles at the same time, false either.
     */
    public boolean isMultipleRolesEnabled(){
        return multipleRolesEnabled_;
    }
    
    /**
     * Provides the customizations to use for the dynamic tree.
     * @return The AdminCustomization to use.
     */
    public AdminCustomization getAdminCustomization(){
        return custom;
    }
    
    /**
     * Fixes the menu bar to use
     * @param jMenuBar
     */
    public void setJMenuBar(JMenuBar jMenuBar) {
        menuBar_ = jMenuBar;
        actionMenu_ = new JMenu("Actions");
        actionMenu_.setEnabled(false);
        menuBar_.add(actionMenu_);
        if(browserProperty_!=null)
            refreshRoleMenu(((RoleManagement)browserProperty_).getRoleList());
    }

    /**
     * Fixes the toolBar to use
     * @param jToolBar The toolBar to use
     */
    public void setJToolBar(JToolBar jToolBar){
        toolBar_ = jToolBar;
        if(toolBar_!=null && toolBar_.getComponents().length==0)
            toolBar_.setVisible(false);
    }
    
    // ==================================================================
    //
    // Public methods for Tree interface.
    //
    // ==================================================================

    /**
     * Refresh the selected node.
     */
    public void 
    refresh(){
        refreshSelectedNode();
    }

    /**
     * Adds an object into the initial context.
     *  
     * @param name Name of the object
     * @param value The object to add
     */
    public void 
    addEntry(Object name, Object value){
        addEntry(name,value,0);
    }

    /**
     * Adds an object into the initial context and expands the node until a specific level if this one is a context.
     *  
     * @param name Name of the object
     * @param value The object to add
     * @param level The level (>0 for a specific level or <0 for all levels)
     */
    public void 
    addEntry(Object name, Object value, int level){
        initialContext_.add(new InitialEntry(name.toString(),value,level));
        addToInitialContext(name.toString(),value,level);
    }

    /**
     * Refreshes the selected node name.
     * 
     * @param oldName The old name.
     * @param newName The new name.
     */
    public void 
    renameSelectedNode(String oldName, String newName){
        Object object = getLastSelectedPathComponent();
        if (object != null && oldName!=null && newName!=null) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) object;
            String selectedNode = pathToString(node);
            String prefixe = selectedNode.substring(0,selectedNode.lastIndexOf(oldName));
            if(!selectedNode.equals(prefixe)){
                // The oldName is contained by the selected node
                Object[] keys = nodeState_.keySet().toArray();
                for(int i=0 ; i<keys.length ; i++) {
                    if(keys[i].toString().startsWith(prefixe+oldName)){
                        boolean valid = true;
                        String oldSuffixe = ((String)keys[i]).substring(prefixe.length() + oldName.length());
                        String newSuffixe = oldSuffixe;
                        if(oldSuffixe!=null && !oldSuffixe.equals("")){
                            int index = oldSuffixe.indexOf("/");
                            if(index==-1 || index>0) {
                                valid = false;
                            }
                        } 
                        if(valid){
                            String oldEntry = prefixe + oldName + oldSuffixe;
                            String newEntry = prefixe + newName + newSuffixe;
                            nodeState_.put(newEntry,nodeState_.get(oldEntry));
                            nodeState_.remove(oldEntry);
                        }
                    }
                }
            }
        }
    }
    
    // ==================================================================
    //
    // Public methods for Autoscroll interface.
    //
    // ==================================================================

    public void autoscroll(Point cursorLocn){
        Rectangle rect=getVisibleRect();
        Dimension dim=getSize();
        
        // Vertival autoscroll
        if(cursorLocn.y>rect.y+rect.height-10) {
            rect.y=Math.min(rect.y+10,dim.height-rect.height);
        } else if(cursorLocn.y<rect.y+10) {
            int i = rect.y+rect.height-10;
            rect.y=Math.max(rect.y-10,0);
        }
        
        // Horizontal autoscroll
        if(cursorLocn.x>rect.x+rect.width-10) {
            rect.x=Math.min(rect.x+10,dim.width-rect.width);
        } else if(cursorLocn.x<rect.x+10) {
            rect.x=Math.max(rect.x-10,0);
        }
        
        scrollRectToVisible(rect);
    }
              
    public Insets getAutoscrollInsets(){
        Rectangle rect=getVisibleRect();
        Dimension dim=getSize();
        Insets inset=new Insets(rect.y+10,rect.x+10,dim.height-(rect.y+rect.height)+10,dim.width-(rect.x+rect.width)+10);
        return inset;
    }

    // ==================================================================
    //
    // Internal classes for JTree management.
    //
    // ==================================================================

    /**
     * Mouse Adapter
     * Use to draw Popup Menu
     */
    final class MyMouseAdapter extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            popupLayout(e);
        }
        public void mouseReleased(MouseEvent e) {
            popupLayout(e);
        }
        private void popupLayout(MouseEvent e) {
            if (e.isPopupTrigger()&& popupEnabled_) {
                showMenu(e.getPoint());
            }
        }
    }

    /**
     * Tree Will Expand Listener
     * Use to complete node : it adds child to current node
     */
    final class MyTreeWillExpandListener implements TreeWillExpandListener {
        public void treeWillCollapse(TreeExpansionEvent event) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
            Entry e = (Entry) currentNode.getUserObject();
            nodeState_.put(pathToString(currentNode), new GraphicState(false));
        }

        public void treeWillExpand(TreeExpansionEvent event) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
            Entry e = (Entry) currentNode.getUserObject();
            Object userObject = e.getValue();
            nodeState_.put(pathToString(currentNode), new GraphicState(true));
            currentNode.removeAllChildren();
            Entry[] entries = ((Context) userObject).getEntries();
            for (int i = 0; i < entries.length; i++) {
                Entry entry = entries[i];
                DefaultMutableTreeNode childNode = addObject(currentNode, (ExtendedEntry) entry);
            }
            treeModel.reload(currentNode);
        }
    }

    /**
     * Tree Expansion Listener
     * Use to refresh node
     */
    final class MyTreeExpansionListener implements TreeExpansionListener {
        public void treeCollapsed(TreeExpansionEvent event) {
        }

        public void treeExpanded(TreeExpansionEvent event) {
            DefaultMutableTreeNode node =
                (DefaultMutableTreeNode) (event.getPath()).getLastPathComponent();
            if (!node.isLeaf()) {
                Enumeration enum = node.children();
                while (enum.hasMoreElements()) {
                    DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enum.nextElement();
                    Entry entry = (Entry) childNode.getUserObject();
                    if (nodeState_.containsKey(pathToString(childNode))) {
                        GraphicState gs = (GraphicState) nodeState_.get(pathToString(childNode));
                        if (gs.isExpanded()) {
                            expandPath(new TreePath(childNode.getPath()));
                        }
                    }

                    if (selectedNode_.equals(pathToString(childNode))) {
                        setSelectionPath(
                            new TreePath(treeModel.getPathToRoot(childNode)));
                    }
                }
            }
        }
    }

    /** 
     * Tree Selection Listener 
     * Use to put the panel corresponding to the selected node
     */
    final class MyTreeSelectionListener implements TreeSelectionListener {

        public void valueChanged(TreeSelectionEvent e) {
            refreshPanel();
            refreshActionMenu();
            refreshToolBar();
            refreshStatusBar();
        }
    }

    /**
     * Tree Cell Renderer
     * Use to put the icon corresponding to the class 
     * of the Value object of the Entry
     */
    final class MyTreeCellRenderer extends DefaultTreeCellRenderer {

        public Component getTreeCellRendererComponent(JTree tree,Object value,boolean selected,boolean expanded,boolean leaf,int row,boolean hasFocus) {
        	super.getTreeCellRendererComponent(tree,value,selected,expanded,leaf,row,hasFocus);

            // Put the appropriate icon if one is defined.
            Entry entry = objectToEntry(value);
            if (custom != null) {
                Icon icon = custom.getIcon((ExtendedEntry)entry);
                if (icon != null)
                    setIcon(icon);
            }

            // Put the name of the entry as the label of the node
            setText(entry.getName().toString());

            //setToolTipText("Essai");
            return this;
        }

    }

    /**
     * Tree Model
     * Uses to know if a node is leaf or not
     */
    final class MyTreeModel extends DefaultTreeModel {

        public MyTreeModel(TreeNode root) {
            super(root);
        }

        public boolean isLeaf(Object node) {
            Entry entry = objectToEntry(node);
            Object o = entry.getValue();
            // Optimzed version : An entry is a leaf if it is not a Context instance 
            return !(o instanceof Context);
            // An entry is a leaf if it is not a Context instance or if it's a Context instance without child.
            //return (!(o instanceof Context) || ((o instanceof Context) && !hasChild((Context) o)));
        }

    }

    /**
     * Uses to know if a node is expanded
     */
    protected class GraphicState {

        protected boolean isExpanded_;

        public GraphicState(boolean isExpanded) {
            isExpanded_ = isExpanded;
        }

        public boolean isExpanded() {
            return isExpanded_;
        }

        public void isEspanded(boolean value) {
            isExpanded_ = value;
        }

    }

    // ==================================================================
    //
    // Public methods of RoleManagement interface.
    //
    // ==================================================================
    
    /** 
     * Provides the current role.
     * 
     * @return The current role. 
     */
    public Role[] getCurrentRoles(){
        if(browserProperty_!=null){
            return ((RoleManagement)browserProperty_).getCurrentRoles();
        }
        return null;
    }
    
    /** 
     * Fixes the current role.
     * 
     * @param role_id The id of the role.
     */
    public void setCurrentRole(String[] role_ids){
        if(browserProperty_!=null){
            ((RoleManagement)browserProperty_).setCurrentRole(role_ids);
            custom.clear();
        }
        if(browserProperty_!=null)
            refreshRoleMenu(((RoleManagement)browserProperty_).getRoleList());
        refreshAll();
    }

    /**
     * Fixes the current role.
     * 
     * @param role The role to define as current role.
     */
    public void setCurrentRole(Role[] roles){
        if(browserProperty_!=null){
            ((RoleManagement)browserProperty_).setCurrentRole(roles);
            custom.clear();
        }
        if(browserProperty_!=null)
            refreshRoleMenu(((RoleManagement)browserProperty_).getRoleList());
        refreshAll();
    }

    /**
     * Provides the list of all define roles.
     * 
     * @return The list of roles.
     */
    public Role[] getRoleList(){
        if(browserProperty_!=null){
            ((RoleManagement)browserProperty_).getRoleList();
        }
        return null;
    }
        
    // ==================================================================
    //
    // Internal classes for Drag & Drop.
    //
    // ==================================================================
   
    /**
     * Implementation of DragGestureListener interface.
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
     * 
     * @version 0.1
     */
    protected class TreeDragGestureListener implements DragGestureListener {

        /* 
         * @see java.awt.dnd.DragGestureListener#dragGestureRecognized(java.awt.dnd.DragGestureEvent)
         */
        public void dragGestureRecognized(DragGestureEvent dge) {
            if(dndEnabled_){
                int dragAction = dge.getDragAction();
                if( (dragAction & dragAction_) != 0) { 
                    Transferable transferable = new EntryViewTransferable(new EntryView(DynamicTree.this.getSelectedEntry()));
                    try {
                        //dge.startDrag(DragSource.DefaultLinkDrop, transferable, dragSourceListener_);
                        dragSource_.startDrag(dge,DragSource.DefaultLinkDrop, transferable, dragSourceListener_);
                    }catch(InvalidDnDOperationException idoe) {
                        System.out.println( idoe );
                    }
                }
            } 
        }
    }
    
    /**
     * Implementation of DragSourceListener interface
     * 
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
     * 
     * @version 0.1
     */
    public class TreeDragSourceListener implements DragSourceListener {

        /* 
         * @see java.awt.dnd.DragSourceListener#dragEnter(java.awt.dnd.DragSourceDragEvent)
         */
        public void dragEnter(DragSourceDragEvent dsde) {
            DragSourceContext context = dsde.getDragSourceContext();
            if( (dsde.getDropAction() & dragAction_) != 0) {    
                context.setCursor(DragSource.DefaultLinkDrop);    
            } else {
                context.setCursor(DragSource.DefaultLinkNoDrop);        
            }       
        }

        /* 
         * @see java.awt.dnd.DragSourceListener#dragOver(java.awt.dnd.DragSourceDragEvent)
         */
        public void dragOver(DragSourceDragEvent dsde) {
        }

        /* 
         * @see java.awt.dnd.DragSourceListener#dropActionChanged(java.awt.dnd.DragSourceDragEvent)
         */
        public void dropActionChanged(DragSourceDragEvent dsde) {
        }

        /* 
         * @see java.awt.dnd.DragSourceListener#dragExit(java.awt.dnd.DragSourceEvent)
         */
        public void dragExit(DragSourceEvent dse) {
            DragSourceContext context = dse.getDragSourceContext();
            context.setCursor(DragSource.DefaultLinkNoDrop); 
        }

        /* 
         * @see java.awt.dnd.DragSourceListener#dragDropEnd(java.awt.dnd.DragSourceDropEvent)
         */
        public void dragDropEnd(DragSourceDropEvent dsde) {
        }

    }

    /**
     * Implementation of DropTargetListener interface. 
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
     * 
     * @version 0.1
     */
    protected class TreeDropTargetListener implements DropTargetListener {

        /** The old position of the cursor */
        protected Point oldPosition_ = new Point(0,0);

        /** The old Path*/
        protected TreePath oldPath_ = null;

        /* 
         * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragEnter(DropTargetDragEvent dtde) {
            dtde.acceptDrag(dtde.getDropAction());
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragOver(DropTargetDragEvent dtde) {
            dtde.acceptDrag(dtde.getDropAction());
            String label = "";
            TreePath path = getPath(dtde.getLocation());
            if(path==null){
                if(statusBar_!=null)
                    statusBar_.setText(label);
                oldPath_ = null;                
            } else if(oldPath_ == null || (path != null && !path.equals(oldPath_))){
                oldPath_ = path;
                Entry entry = objectToEntry(path.getLastPathComponent());
                if(custom != null){ 
                    label = custom.getDropLabel((ExtendedEntry) entry);
                    if(label == null)
                        label = "";
                    if(statusBar_ != null)
                        statusBar_.setText(label);
                }
            }
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
         */
        public void dropActionChanged(DropTargetDragEvent dtde) {
            dtde.acceptDrag(dtde.getDropAction());
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
         */
        public void dragExit(DropTargetEvent dte) {
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
         */
        public void drop(DropTargetDropEvent dtde) {
            DataFlavor[] df = EntryViewTransferable.flavors_;
            DataFlavor chosen = null;
            for (int i = 0; i < df.length; i++) {
                if(dtde.isDataFlavorSupported(df[i])){
                    chosen = df[i];
                    break;
                }
            }
            
            EntryView entryView = null;
            // This method is responsible of a bug (it is waiting for ???)
//            dtde.acceptDrop(acceptableActions_);
            try {
                entryView = (EntryView)dtde.getTransferable().getTransferData(chosen);
            } catch (UnsupportedFlavorException e1) {
                e1.printStackTrace();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            
            TreePath path = getPath(dtde.getLocation());
            if (path != null) {
                selectionModel.setSelectionPath(path);
                Entry entry = objectToEntry(getLastSelectedPathComponent());
                if(custom != null) {
                    DropAction dropAction = custom.getDropAction((ExtendedEntry) entry);
                    if(dropAction!=null && entryView!=null){
                        try{
                            dropAction.execute(new DefaultDropTreeView(DynamicTree.this, entryView.getCurrentEntry()));
                        }catch(Exception e){
                            JOptionPane.showMessageDialog(null, e.getMessage(), "Drag & Drop error ! (" + e.getClass().getName() + ")", JOptionPane.ERROR_MESSAGE);
                        }
                        DynamicTree.this.refreshAll();
                    }
                }     
            }
        }
    }

    private final class RoleComparator implements Comparator{

        protected int compare(Role r1, Role r2) {
            return r1.getId().compareTo(r2.getId());
        }

        public int compare(Object o1, Object o2) {
            return compare((Role)o1,(Role)o2);
        }

    }

    private final class RoleSelected extends AbstractAction{

        protected Role role_;
        
        public RoleSelected(Role role){
            super(role.getId());
            role_ = role;
        }
        
        public void actionPerformed(ActionEvent e) {
            if(DynamicTree.this.multipleRolesEnabled_){
                Role[] currentRoles = DynamicTree.this.getCurrentRoles();
                Vector theRoles = new Vector(Arrays.asList(currentRoles));
                if(theRoles.contains(role_))
                    theRoles.remove(role_);
                else
                    theRoles.add(role_);
                DynamicTree.this.setCurrentRole((Role[])theRoles.toArray(new Role[0]));
            } else {
                Role currentRole = (DynamicTree.this.getCurrentRoles())[0];
                if(!role_.getId().equals(currentRole.getId())) {
                    DynamicTree.this.setCurrentRole(new Role[]{role_});
                }
            }
            refreshAll();
        }

    }

}
