/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.dnd;

import org.objectweb.util.browser.api.DropAction;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.api.AssociatedDropAction;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.DropActionFactory;
import org.objectweb.util.browser.core.common.AssociatedElementImpl;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class AssociatedDropActionImpl
    extends AssociatedElementImpl 
    implements AssociatedDropAction {

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    /**
     * Constructs a AssociatedPanelImpl
     * @param loader The propertyLoader to use to load new configuration
     */
    public AssociatedDropActionImpl(BrowserProperty loader) {
        super();
        context_ = new DropActionPropertyContainer(loader);
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for AssociatedDropAction.
    //
    //==================================================================

    /* 
     * @see org.objectweb.util.browser.api.AssociatedDropAction#getDropAction(java.lang.Object)
     */
    public DropAction getDropAction(Object node) {
        return getDropAction(node,false);
    }

    /* 
     * @see org.objectweb.util.browser.api.AssociatedDropAction#getDropAction(java.lang.Object, boolean)
     */
    public DropAction getDropAction(Object node, boolean disabledJavaLangObject) {
        if (context_ != null) {
            Entry entry = context_.getRecursiveProperty(node);
            if (entry != null && !(entry.getName().toString().equals("java.lang.Object") && disabledJavaLangObject)) {
                DropActionFactory dropActionFactory = (DropActionFactory) entry.getValue();
                return dropActionFactory.newDropAction(node);
            }
        }
        return null;
    }

    /*
     *  @see org.objectweb.util.browser.api.AssociatedDropAction#getDropLabel(java.lang.Object)
     */
    public String getDropLabel(Object node) {
        return getDropLabel(node,false);
    }

	/* 
	 * @see org.objectweb.util.browser.api.AssociatedDropAction#getDropLabel(java.lang.Object, boolean)
	 */
	public String getDropLabel(Object node, boolean disabledJavaLangObject) {
        if (context_ != null) {
            Entry entry = context_.getRecursiveProperty(node);
            if (entry != null && !(entry.getName().toString().equals("java.lang.Object") && disabledJavaLangObject)) {
                DropActionFactory dropActionFactory = (DropActionFactory) entry.getValue();
                return dropActionFactory.getLabel();
            }
        }
		return null;
	}
}