/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.panel;

import java.awt.Color;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JPanel;

import org.objectweb.util.browser.api.Panel;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.core.api.Composite;

/**
 * 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class DefaultCompositePanel 
     extends JPanel 
  implements Composite, Panel {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The list containing the inner panels */
    protected Vector panelList_;
    
//    protected Box boxPanel_;
    
    //==================================================================
    //
    // Constructor.
    // 
    //==================================================================

    /**
     * Default Constructor
     */
    public DefaultCompositePanel(){
        panelList_ = new Vector();
        setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

//        boxPanel_ = Box.createVerticalBox();
//        add(boxPanel_);    
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for Composite interface.
    //
    //==================================================================

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.core.api.Composite#add(org.objectweb.util.browser.api.Panel)
     */
    public void add(Panel panel) {
        if(panel!=null){
            panelList_.add(panel);
        }
    }

    //==================================================================
    //
    // Public methods for Panel interface.
    //
    //==================================================================

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Panel#selected(org.objectweb.util.browser.api.TreeView)
     */
    public void selected(TreeView treeView) {
        Iterator it = panelList_.iterator();
        while (it.hasNext()) {
            Panel element = (Panel) it.next();
            element.selected(treeView);
        }
    }

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Panel#getPanel()
     */
    public JPanel getPanel() {
        Iterator it = panelList_.iterator();
//        boxPanel_.add(Box.createVerticalGlue());
        while (it.hasNext()) {
            Panel element = (Panel) it.next();
//            boxPanel_.add(element.getPanel());
            add(element.getPanel());
        }
//        boxPanel_.add(Box.createVerticalGlue());
        return this;
    }

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Panel#unselected(org.objectweb.util.browser.api.TreeView)
     */
    public void unselected(TreeView treeView) {
        Iterator it = panelList_.iterator();
        while (it.hasNext()) {
            Panel element = (Panel) it.next();
            element.unselected(treeView);
        }        
    }

}

