/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.popup;

/** The Java API's imports  */
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

/** The Console's imports */
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.core.api.MenuItemTreeViewConfiguration;
import org.objectweb.util.browser.core.common.DynamicTree;

/**
 *  Defines the default action which wraps the ExtendedActionListener.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DefaultAction extends AbstractAction {

    protected DynamicTree tree_ = null;
    protected MenuItem delegate_ = null;
    protected MenuItemTreeView treeView_ = null;
    protected boolean isUserIcon_ = false;

    protected void init(DynamicTree tree, MenuItem actionListener, MenuItemTreeView treeView){
        tree_ = tree;
        delegate_ = actionListener;
        treeView_ = treeView;        
    }

    public DefaultAction(DynamicTree tree, MenuItem actionListener, MenuItemTreeView treeView, String name, Icon icon, KeyStroke keyStroke, Character mnemonic, boolean isUserIcon, int itemState) {
        super(name, icon);
        init(tree,actionListener,treeView);
        // Fixes the short description of the action.
        putValue(SHORT_DESCRIPTION, name);
        // Fixes the mnemonic value.
        if(mnemonic!=null){
            putValue(MNEMONIC_KEY, new Integer(Character.toUpperCase(mnemonic.charValue())));
        }
        // Fixes the accelerator key.
        if(keyStroke!=null){
            putValue(ACCELERATOR_KEY, keyStroke);
        }
        isUserIcon_ = isUserIcon;
        if (itemState == MenuItem.DISABLED_STATUS)
            setEnabled(false);
    }

    public boolean isUserIcon(){
        return isUserIcon_;
    }

    public void actionPerformed(ActionEvent e) {
        try {
            ((MenuItemTreeViewConfiguration)treeView_).setActionEvent(e);
            delegate_.actionPerformed(treeView_);
        } catch (Exception e1) {
            //e1.printStackTrace();
            JOptionPane.showMessageDialog(null, e1.getClass().getName() + ":\n" + e1.getMessage(), "Exception (" + e.getActionCommand() + ")", JOptionPane.ERROR_MESSAGE);
        }
        tree_.refreshAll();
    }

}
