/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.corba.idl2java;

// Package dependencies.
import org.objectweb.util.cmdline.lib.DefaultOptionBase;


/**
 * This is a base class for Preprocessor Options.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public abstract class PreprocessorOptionBase
              extends DefaultOptionBase
           implements PreprocessorOption
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public PreprocessorOptionBase()
    {
        // Calls the DefaultOptionBase constructor.
        super();

        setArguments(new String[0]);
        setMandatory(false);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Checks the current command line argument.
     *
     * @param current The current command line argument.
     *
     * @return true if the option accepts this argument.
     */
    public boolean check(String current) {
      String baseOptionLabel = getBaseOptionLabel();

        if (!current.startsWith(baseOptionLabel)) {
            return false;
        }

        consumeOption(current.substring(baseOptionLabel.length()));
        return true;
    }

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator) {
        // Nothing to do.
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorOption
    //
    // ==================================================================

    /**
     * Gets the base option label.
     *
     * @return The base option label.
     */
    public abstract String getBaseOptionLabel();

    /**
     * Consumes the option.
     *
     * @param current The option value.
     */
    public abstract void consumeOption(String current);

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

}
