/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.task;

// Package dependencies.
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/**
 * This task run an idl2java compiler.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class Idl2JavaTask
     extends Task
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The idl2java application. */
    private Idl2JavaApplication app_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /**
     * The default constructor.
     */
    public 
    Idl2JavaTask()
    {
        // Init internal states
        app_ = new Idl2JavaApplication();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Set the file to parse.
     *
     * @param file - The file name.
     */
    public void
    setFile(String file)
    {
        app_.setFile(file);
    }

    /**
     * Set the destination directory.
     *
     * @param dir - The directory name.
     */
    public void
    setDestdir(String dir)
    {
       app_.setDestdir(dir);
    }

    /**
     * Set a directory list to add to the search path.
     *
     * @param dirs - A directory list.
     */
    public void
    setIncludes(String dirs)
    {
        app_.setIncludes(dirs);
    }

    /**
     * Create an inner element for Package Mapping.
     */
    public PackageMapping
    createMapping()
    {
        return app_.createMapping();
    }
    
    /**
     * The method executing the task.
     *
     * @throws BuildException if an error occurs.
     */
    public void
    execute()
    throws BuildException
    {
        app_.setProjectProperties( getProject().getProperties() );
        app_.run();
    }
}
