/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): offroy ________________________.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 10 juin 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */
 
package org.omg.lifl.eclipse.plugin.project.utils.SWT;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;

/**
 * This class is used to make a basic SWT area with a Radio (On/Off) Button SWT widget 
 * which allow configuration of some preferences store in a SetterGroupPreference
 *      this basic SWT use a check button to set its visibility
 *      this class implements update of the Radio Button SWT widget's value 
 *      and synchonisation with its corresponding preferences 
 *
 * @author offroy@lifl.fr
 *
 * @version 0.1
 */
public class DirectoryChooserCheckRadioButtonSetter
    extends DirectoryChooserCheckTextSetter {

    private Button _radios[];
    private String[] value = { "On", "Off" };

    /**
     * Constructor
     * @param parent is the SWT parent group
     * @param preference the preferences to manage 
     */
    public DirectoryChooserCheckRadioButtonSetter(
        Group parent,
        SetterGroupPreference preference) {
        super(parent, preference);
        _text.dispose();
        _btnChooser.dispose();
        _radios = new Button[value.length];
        for (int i = 0; i < _radios.length; i++) {
            _radios[i] = new Button(getSubGroup(), SWT.RADIO);
            if (i < value.length && value[i] != null) {

                _radios[i].setText(value[i]);
                _radios[i].addListener(SWT.Selection, new Listener() {

                    public void handleEvent(Event arg0) {
                        _preference.set_Name(getSelected().trim());
                    }

                });
            } else
                _radios[i].setText("");

        }
        _radios[0].setSelection(true);
        _preference.set_Name(getSelected().trim());

    }
    
    /**
     * @return the value of the selected radio button
     */
    protected String getSelected() {
        for (int i = 0; i < _radios.length; i++) {
            if (_radios[i].getSelection())
                return _radios[i].getText();
        }
        return null;
    }

    /* (non-Javadoc)
     * @see org.omg.lifl.plugin.project.utils.SWT.DirectoryChooser#setCheckButton()
     */
    protected void setCheckButton() {
        _checkButton = new Button(subGroup, SWT.CHECK);
        _checkButton.setSelection(isSubGroupVisible());
        _checkButton.addListener(SWT.Selection, new Listener() {

            public void handleEvent(Event event) {
                setSubGroupVisible(_checkButton.getSelection());
                if (subGroup != null) {
                    //_label.setVisible(isSubGroupVisible());
                    for (int i = 0; i < _radios.length; i++)
                        _radios[i].setVisible(isSubGroupVisible());
                    get_preference().setEnable(isSubGroupVisible());
                } else //TODO do something when there is no subGroup
                    ;
            }

        });

    }

    /* (non-Javadoc)
     * @see org.omg.lifl.plugin.project.utils.SWT.DirectoryChooser#config(java.lang.String)
     */
    protected void config(String buffer) {
    }

}
