/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/
package org.omg.lifl.eclipse.plugin.shell.views;

import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

/**
 * This class plug-in a new workbench view for OpenCCM use.
 * <p>
 */

public class OpenCCMConsole extends ViewPart {
	private TextViewer viewer;
	private Document document;
    public static final String CONSOLE_ID = "org.omg.lifl.eclipse.plugin.shell.views.OpenCCMConsole";

	/**
	 * The constructor.
	 */
	public OpenCCMConsole() {
		viewer = null;
		document = null;
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		viewer =
			new TextViewer(parent, 832); //TODO find code 832 SWT.
		GridData viewerData = new GridData(1808);
		viewer.getControl().setLayoutData(viewerData);
        viewer.setEditable(false);

    }

    private void showMessage(String message) {
        MessageDialog.openInformation(
            viewer.getControl().getShell(),
            "OpenCCM Console View",
            message);
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {}

    public void clear()
    {
        viewer.setDocument(null);
    }

    public void refresh(String string, Vector errorHighlight)
    {
        document = new Document(string);
        viewer.setDocument(document);
        try
        {
            for(int i = 0; i < errorHighlight.size(); i++)
            {
                Point highlightRange = (Point)errorHighlight.elementAt(i);
                viewer.setTextColor(new Color(null, 255, 25, 20), highlightRange.x, highlightRange.y, false);
            }

        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        int numLines = document.getNumberOfLines();
        viewer.setTopIndex(numLines);
    }

    
    
    /**
     * @return
     */
    public TextViewer getViewer() {
        return viewer;
    }

    /**
     * @param viewer
     */
    public void setViewer(TextViewer viewer) {
        this.viewer = viewer;
    }

}