/*====================================================================

This file was produced by the OpenCCM CIF_JIMPL generator.

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package ist.coach.coachEmfServicesComponents.SnmpAdapterProviderComposition;

import org.opennms.protocols.snmp.*;

import ist.coach.coachEmfCommon.Utils;
import ist.coach.coachEmfCommon.ExceptionMessages;
import ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError;
import ist.coach.coachEmfCommon.DataTypeImpl;
import ist.coach.coachEmfServices.SnmpAdapter.DataType;
import ist.coach.coachEmfServicesComponents.SnmpAdapterProviderPackage.trap_targetConnection;
//import ist.coach.coachEmfServicesComponents.SnmpObjectData;
//import ist.coach.coachEmfServicesComponents.Configuration;

import org.omg.CORBA.Any;

import java.util.LinkedList;
import java.util.ListIterator;

import org.omg.CosNaming.NameComponent;

/**
 *  This is the CIDL-based implementation of the
 *  OMG IDL3 IDL:coach.ist/coachEmfServicesComponents/SnmpAdapterProvider:1.0 component type.
 *
 *  @author OpenCCM CIF_Jimpl Compiler.
 */
public class SnmpAdapterProviderImpl
     extends ist.coach.coachEmfServicesComponents.SnmpAdapterProviderComposition.SnmpAdapterProvider_impl
	 implements Runnable,
			    SnmpTrapHandler
	 {
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================
	protected static LinkedList linkedList;
	protected static Configuration config;
	trap_targetConnection[] trap_target_facades = null;
	/**
     ** Private ORB reference (used to create Anys)
     **/
    private static org.omg.CORBA.ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();
    /**
	/**
     ** Declaration of the identifier used to identify SNMP trap types
     **/
    private static final SnmpObjectId SNMPv2_MIB_snmpTrapOid =
	new SnmpObjectId(".1.3.6.1.6.3.1.1.4.1.0");

	public static final int SNMP_TRAP_PORT = 2010;
    /**
     ** SnmpTrapDaemon Listener Thread
     **/
    private Thread listenerThread;
    /**
     ** Declaration of Unknown Trap
     **/
    private static final int UNKNOWN_TRAP_TYPE = 0;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public SnmpAdapterProviderImpl()
    {
	  this.linkedList = new LinkedList();
    }
    // ==================================================================
    //
    // Methods for the OMG IDL Components::EnterpriseComponent local interface.
    //
    // ==================================================================
    //
    //  IDL:omg.org/Components/EnterpriseComponent/configuration_complete:1.0
    //
    /**
     **  Completes the component configuration.
     **
     **  @throws org.omg.Components.InvalidConfiguration
     **          Thrown if the configuration is invalid.
     **/
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        //
        //  DONE : implement !!!
        //

		config = Configuration.getInstance();

		if (config == null) {
			System.err.println("Could not create a valid configuration. Exiting...");
			throw new org.omg.Components.InvalidConfiguration();
		}

		listener_init();

		if ((trap_target_facades =  get_context().get_connections_trap_target()) == null) {
			System.err.println("SnmpAdapter is not configured with Facades to serve....");
		}

		System.err.println("SnmpAdapterProvider configuration completed...");
   	 }
	// added by FSET
	public void ccm_remove() 
	throws org.omg.Components.CCMException {
		listener_stop();
		super.ccm_remove();
		System.err.println("SnmpAdapter has been destroyed");
	}
    // ==================================================================
    //
    // Private Methods.
    //
    // ==================================================================

    protected static Any convertSnmpSyntaxToAny(SnmpSyntax data) {

		Any value =orb.create_any();


		switch (data.typeId()) {
			case SnmpSMI.SMI_OBJECTID:
				value.insert_string( new String(((SnmpObjectId)data).toString()));
				break;
			case SnmpSMI.SMI_COUNTER32:
				value.insert_longlong( ((SnmpCounter32)data).getValue());
				break;
			case SnmpSMI.SMI_COUNTER64: //Same as SnmpSMI.SMI_UNSIGNED32
				value.insert_longlong( ((SnmpCounter64)data).getValue().longValue());
				break;
			case SnmpSMI.SMI_GAUGE32:
				value.insert_longlong( ((SnmpGauge32)data).getValue());
				break;
			case SnmpSMI.SMI_INTEGER:
				value.insert_long( ((SnmpInt32)data).getValue());
				break;
			case SnmpSMI.SMI_TIMETICKS:
				value.insert_longlong( ((SnmpTimeTicks)data).getValue());
				break;
			case SnmpSMI.SMI_IPADDRESS:
			    value.insert_string( new String(((SnmpIPAddress)data).toString()));
				break;
			case SnmpSMI.SMI_STRING:
			    //fset HACK
				// It seems that orbacus has problems handling anys as strings if these contain
				// non printable characters. NOT RESOLVED BUT BYPASSED....
			    //System.err.println("SnmpSMI.SMI_STRING length= " + ((SnmpOctetString)data).getLength());
				//System.err.print("As string = " + ((SnmpOctetString)data).toString() +
				//    " as bytes = " + new String(((SnmpOctetString)data).getString()));
				/*********
				if (octetData.toString().indexOf("RAW") < 0) {
				  //System.err.println(" Type is HEX");
				  value.insert_string("Hello");
				 }
				 else {
				  //System.err.println(" Type is RAW");
				  value.insert_string( new String(((SnmpOctetString)data).getString()));
			  	}
				******/
				SnmpOctetString octetData = (SnmpOctetString) data;
				byte[] octetDataBytes = octetData.getString();
				org.omg.CORBA.OctetSeqHelper.insert(value, octetDataBytes);

				//initially by digi
				//value.insert_string( new String(((SnmpOctetString)data).getString()));
				break;
			case SnmpSMI.SMI_OPAQUE:
			    value.insert_string( new String(((SnmpOpaque)data).getString()));
				break;
			case SnmpSMI.SMI_NOSUCHOBJECT:
				//value = null;
				// Is this good...???
				value.insert_string("No Such Object");
				break;
			case SnmpSMI.SMI_NOSUCHINSTANCE:
				//value = null;
				// Is this good...???
				value.insert_string( "No Such Instance");
				break;
			default:
				System.err.println("Unknown Variable Binding.");
				value = null;
				break;

		}

		return value;

    }

    protected static SnmpSyntax convertAnyToSnmpSyntax(Any data, byte code) {

		SnmpSyntax value = null;

		switch (code) {

			case SnmpSMI.SMI_COUNTER32:
				value = ((SnmpSyntax)new SnmpCounter32(data.extract_longlong()));
				break;
			case SnmpSMI.SMI_COUNTER64:
				value = ((SnmpSyntax)new SnmpCounter64(data.extract_longlong()));
				break;
			case SnmpSMI.SMI_UNSIGNED32: // same as SnmpSMI.SMI_GAUGE32
				value = ((SnmpSyntax)new SnmpGauge32(data.extract_longlong()));
				break;
			case SnmpSMI.SMI_INTEGER:
				value = ((SnmpSyntax)new SnmpInt32(data.extract_long()));
				break;
			case SnmpSMI.SMI_IPADDRESS:
				value = ((SnmpSyntax)new SnmpIPAddress(data.extract_string().getBytes()));
				break;
			case SnmpSMI.SMI_OBJECTID:
				value = ((SnmpSyntax)new SnmpObjectId(data.extract_string()));
				break;
			case SnmpSMI.SMI_OPAQUE:
				value = ((SnmpSyntax)new SnmpOpaque(data.extract_string().getBytes()));
				break;
			case SnmpSMI.SMI_STRING:
			    value = (SnmpSyntax)new SnmpOctetString(org.omg.CORBA.OctetSeqHelper.extract(data));
				//initially by digi
				//value = ((SnmpSyntax)new SnmpOctetString(data.extract_string().getBytes()));
				break;
			case SnmpSMI.SMI_TIMETICKS:
				value = ((SnmpSyntax)new SnmpTimeTicks(data.extract_longlong()));
				break;
			default:
				System.err.println("SnmpAdapter:convertAnyToSnmpSyntax> " +
				    " Unknown (or un-supported) SMI Type");
				break;
		}

		return value;
    }

    private void processTrapData(
		java.net.InetAddress agent,
		SnmpOctetString community,
		SnmpPduPacket pdu)
    throws SnmpApplicationError{

		int snmpTrapType = 0;
		String trapObjectIdentifier = null;
		int k = pdu.getLength();
		DataTypeImpl[] dataSetType = new DataTypeImpl[k];

		for (int i = 0; i < k ; i++ ) {

			SnmpVarBind vb = pdu.getVarBindAt(i);

			String oid = vb.getName().toString();
			Any value = orb.create_any();

			switch (vb.getValue().typeId()) {
				case SnmpSMI.SMI_OBJECTID:
					value.insert_string( ((SnmpObjectId)vb.getValue()).toString());
					break;
				case SnmpSMI.SMI_COUNTER32:
					value.insert_longlong( ((SnmpCounter32)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_COUNTER64:
					value.insert_longlong( ((SnmpCounter64)vb.getValue()).getValue().longValue());
					break;
				case SnmpSMI.SMI_GAUGE32:
					value.insert_longlong( ((SnmpGauge32)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_INTEGER:
					value.insert_long( ((SnmpInt32)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_TIMETICKS:
					value.insert_longlong( ((SnmpTimeTicks)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_IPADDRESS:
					value.insert_string( ((SnmpIPAddress)vb.getValue()).toString());
					break;
				case SnmpSMI.SMI_STRING:
					SnmpOctetString octetData = (SnmpOctetString) vb.getValue();
				    byte[] octetDataBytes = octetData.getString();
				    org.omg.CORBA.OctetSeqHelper.insert(value, octetDataBytes);
					//by digi
					//value.insert_string( new String(((SnmpOctetString)vb.getValue()).getString()));
					break;
				default:
					System.err.println("SnmpAdapter: processTrapData> Unknown Variable Binding.");
					break;

			}

			if (vb.getName().compare(SNMPv2_MIB_snmpTrapOid) == 0)
				trapObjectIdentifier = ((SnmpObjectId)vb.getValue()).toString();

			dataSetType[i] = new DataTypeImpl(oid, value);

		}

		snmpTrapType = determineTrapType(trapObjectIdentifier);
		if (snmpTrapType == UNKNOWN_TRAP_TYPE)
			throw new SnmpApplicationError(ExceptionMessages.uknown_trap_error);

		String objectClass = determineObjectClass(snmpTrapType);
		if (objectClass == null) {
			System.err.println("SnmpAdapter: processTrapData> Trap could not be mapped to an object class");
			throw new SnmpApplicationError("Trap could not be mapped to an object class");
		}

		System.err.println("SnmpAdapter: processTrapData> Trap is " + trapTypeToString(snmpTrapType) +
			" (trap type " + snmpTrapType +
			" ), thrown by objects of objectclass " + objectClass);

		NameComponent [] name = null;

		if (isRootObject(objectClass)) {
			name = findRegisteredRootObject(objectClass, agent.getHostAddress());
			if (name == null) {
				System.err.println("SnmpAdapter: processTrapData> Trap could not be mapped to a registered (root) object.");
				throw new SnmpApplicationError(ExceptionMessages.uknown_trap_origin_error);
			}
		}
		else {

			boolean valueIsNeeded = determineIfValueIsNeeded(objectClass);

			Any value = null;

			if (valueIsNeeded == true) {
				value = determineValue(objectClass, dataSetType);

				if (value == null) {
					System.err.println("SnmpAdapter: processTrapData> Could not determine value for trap!");
					throw new SnmpApplicationError(ExceptionMessages.trap_misconfig_error);
				}
			}

			name = findRegisteredLeafObject(objectClass, agent.getHostAddress(), value);
			if (name == null) {
				System.err.println("SnmpAdapter: processTrapData> Trap could not be mapped to a registered (leaf) object.");
				throw new SnmpApplicationError(ExceptionMessages.uknown_trap_origin_error);
			}
		}

		String objectFacadeKind = Utils.name2facade(Utils.name2string(name));
		String facadeNameKind = null;
		boolean facadeFound = false;

		System.err.println("SnmpAdapter: processTrapData> Object with name " + Utils.name2string(name) +
			" is accessible through a facade with kind " + objectFacadeKind);

		int index = 0;
		String facadeName = new String();
		trap_target_facades =  get_context().get_connections_trap_target();
		for (index = 0; index < trap_target_facades.length; index++) {

			facadeName = trap_target_facades[index].objref.facade_name();
			facadeNameKind = Utils.name2facade(facadeName);

			//System.err.println("Checking with facade with name " +
			//	facadeName + " which has a kind " + facadeNameKind);

			if (objectFacadeKind.equals(facadeNameKind)) {

				System.err.println("SnmpAdapter: processTrapData> Object with name " +
					  Utils.name2string(name) +
					" is accessible through facade with name " + facadeName +
					    " found at position " + index);

				facadeFound = true;

				break;

			}
		}

		if (facadeFound == false) {

			System.err.println("SnmpAdapter: processTrapData> Could not find a facade to send trap to");
			throw new SnmpApplicationError("No Facade trap sink found");
		}

		int varBindNumber = pdu.getLength();

		DataTypeImpl [] trapData = new DataTypeImpl[varBindNumber];

		for (int j = 0; j < varBindNumber; j++) {

			SnmpVarBind vb = pdu.getVarBindAt(j);

			trapData[j] = new DataTypeImpl(vb.getName().toString(),
				convertSnmpSyntaxToAny(vb.getValue()),
				vb.getValue().typeId());

		}

		String community_str = (community != null) ?
			new String(community.getString()) : new String();

		System.err.println("Calling notifyTrap with arguments: ");
		System.err.println("Name: " + name);
		System.err.println("Community: " + community_str);
		System.err.println("RequestId: " + pdu.getRequestId());
		System.err.println("SnmpTrapType: " + snmpTrapType);
		System.err.println("Length of trap data: " + trapData.length);

		try {
			ist.coach.coachEmfServices.SnmpAdapter.TrapCallback facade_callback =
				trap_target_facades[index].objref;

			if (facade_callback != null) {
				facade_callback.notifyTrap( name,
						community_str,
						pdu.getRequestId(),
						snmpTrapType,
						trapData);
				System.err.println("SnmpAdapter has served Trap successfully");
			}
			else
			System.err.println("SnmpAdapter: processTrapData> Could not retrieve reference to facade of " +
					Utils.name2string(name));
		}
		catch(SnmpApplicationError snmp_ex) {
			System.err.println("SnmpAdapterProvider: SnmpApplicationError exception caught " +
				" while trying to forward trap to " + Utils.name2string(name));
		}
		catch(intt.itu.itut_x780.ApplicationError app_exc) {
			System.err.println("SnmpAdapterProvider: ApplicationError exception caught " +
				" while trying to forward trap to " + Utils.name2string(name));
		}

    }

    private int determineTrapType(String trapObjectIdentifier) {


		int snmpTrapType = UNKNOWN_TRAP_TYPE;

		for (int i = 0; i < this.config.availableTraps.length; i++) {
			if (trapObjectIdentifier.equals(
				this.config.availableTraps[i].oid.toString())) {

				snmpTrapType = this.config.availableTraps[i].identifier;
				break;
			}
		}

		return snmpTrapType;

    }

    private String determineObjectClass(int snmpTrapType) {

		for (int i = 0; i < this.config.availableTraps.length; i++) {
			if (snmpTrapType == this.config.availableTraps[i].identifier) {
				return this.config.availableTraps[i].objectclass;
			}
		}

		return null;

    }

   private boolean determineIfValueIsNeeded(String objectClass) {

		boolean valueNeeded = false;

		for (int i = 0; i < this.config.availableTraps.length; i++) {

			if (objectClass.equalsIgnoreCase(this.config.availableTraps[i].objectclass)) {

				if (this.config.availableTraps[i].valueOid == null) {
					System.err.println("SnmpAdapter: determineIfValueIsNeeded> No value needed to fetch " +
						"leaf object of objectclass " + objectClass);
					valueNeeded = false;
				}
				else
					valueNeeded = true;
			}
		}

		return valueNeeded;
    }


    private Any determineValue(String objectClass, DataType [] varBindings) {

		String valueOid = null;

		//System.err.println("SnmpAdapter: determineValue> Determining value for objectClass..." +
		//	objectClass + ", with " + varBindings.length + " variable bindings");


		for (int i = 0; i < this.config.availableTraps.length; i++) {

			if (objectClass.equalsIgnoreCase(this.config.availableTraps[i].objectclass)) {

				if (this.config.availableTraps[i].valueOid != null)
					valueOid = new String(this.config.availableTraps[i].valueOid.toString());
				//else
				//	System.err.println("SnmpAdapter: determineValue> No value need to fetch leaf object of objectclass "
				//	  + objectClass);

				break;
			}

		}

		if (valueOid != null) {
			for (int i = 0; i < varBindings.length; i++) {
				if (varBindings[i].identifier.equals(
					valueOid)) {
					return (varBindings[i].value);
				}
			}
		}
		//System.err.println("SnmpAdapter: determineValue> Returning null");
		return null;

    }

    private String trapTypeToString(int snmpTrapType) {
		for (int i = 0; i < this.config.availableTraps.length; i++) {
			if (this.config.availableTraps[i].identifier == snmpTrapType) {

				return this.config.availableTraps[i].id;
			}

		}

		return null;
    }

	protected static synchronized boolean isRootObject(String objectClass) {

		for (int i = 0; i < config.rootObjects.length; i++)
			if (config.rootObjects[i].equals(objectClass))
				return true;

		return false;
    }
	/**
	 * @param objectClass
	 * @param ipAddress
	 * @return NameComponent[]
	 *
	 * Returns the name of a root-object with the given
	 * objectclass and IP Address.
	 */
    private NameComponent[] findRegisteredRootObject(
		String objectClass, String ipAddress) {

		ListIterator li = this.linkedList.listIterator();

		//System.err.println("SnmpAdapter: findRegisteredRootObject> Trying to find Root object with object class " +
		//	objectClass + " and IP Address " + ipAddress);

		while (li.hasNext()) {

			SnmpObjectData objData = (SnmpObjectData)li.next();

			NameComponent [] name = objData.getName();
			String myObjectClass = name[name.length - 2].kind;
			String myIpAddress = objData.getIPAddress();

			//System.out.println("Comparing to object class " +
				//myObjectClass + " and IP Address " + myIpAddress);

			if (myObjectClass.equals(objectClass) &&
				myIpAddress.equals(ipAddress)) {

				//System.err.println("-----> Matched with object named " +
				//	Utils.name2string(objData.getName()));
				return objData.getName();
			}
		}

		return null;
    }

	/**
	 * @param objectClass
	 * @param ipAddress
	 * @param value
	 * @return NameComponent[]
	 *
	 * Returns the name of a registered leaf object with
	 * the given name, IP Address, registered with the
	 * given value.
	 */

    private NameComponent[] findRegisteredLeafObject(
		String objectClass, String ipAddress, Any value) {

		ListIterator li = this.linkedList.listIterator();

		//System.err.println("SnmpAdapter> Trying to find Leaf object with object class " +
		//		objectClass + " and IP Address " + ipAddress);

		while (li.hasNext()) {

			SnmpObjectData objData = (SnmpObjectData)li.next();

			NameComponent [] name = objData.getName();
			String myObjectClass = getObjectClass(name);
			String myIpAddress = objData.getIPAddress();

			if (myObjectClass.equals(objectClass) &&
				myIpAddress.equals(ipAddress)) {

				if (value != null) {
					if (objData.getValue().equal(value)) {

						//System.err.println("-----> Matched with object named" +
						//	Utils.name2string(objData.getName()));
						return objData.getName();
					}
				}
				else {
					//System.err.println("-----> Matched with object named" +
					//	Utils.name2string(objData.getName()));
					return objData.getName();
				}
			}
		}

		return null;
    }

    protected static String getObjectClass(NameComponent[] name) {
		return name[name.length - 2].kind;
    }

	private void listener_init() {
		if (listenerThread == null) {
			listenerThread = new Thread(this, "Listener");
			listenerThread.start();
		}
    }

    private void listener_stop() {

		Thread dummyThread =listenerThread;
		listenerThread = null;
	    dummyThread.interrupt();

    }

	 // ==================================================================
    //
    // Runnable Interface methods.
    //
    // ===================================================================
    public void run() {

		Thread myThread = Thread.currentThread();
		SnmpTrapSession trapSession = null;
		while (listenerThread == myThread) {

			try {
				trapSession = new SnmpTrapSession(this,
							SNMP_TRAP_PORT);
				System.err.println("SNMP Trap Receiver Started");
				synchronized(trapSession)
				{
					trapSession.wait();
				}
			}
			catch (InterruptedException ie) {
				System.err.println("SNMP Trap Receiver Exiting");
				trapSession.close();
			}
			catch (Exception e) {
				System.err.println("Exception in init(): " + e);
				e.printStackTrace();
			}
		}
   	 }


    // ==================================================================
    //
    // SnmpTrapHandler Interface methods.
    //
    // ===================================================================


    /**
     * Method from SnmpHandler interface, used to parse SNMPv2 Traps
     */
    public void snmpReceivedTrap(
		 SnmpTrapSession 	session,
		 java.net.InetAddress 	agent,
		 int 			port,
		 SnmpOctetString	community,
		 SnmpPduPacket 		pdu) {

		System.err.println("V2 Trap from agent " + agent.toString() + " on port " + port);
		System.err.println("V2 Trap Community........... " + new String(community.getString()));
		System.err.println("V2 Trap PDU command......... " + pdu.getCommand());
		System.err.println("V2 Trap PDU ID.............. " + pdu.getRequestId());
		System.err.println("V2 Trap PDU Length.......... " + pdu.getLength());

		if(pdu instanceof SnmpPduRequest)
		{
			System.err.println("V2 Trap PDU Error Status.... " + ((SnmpPduRequest)pdu).getErrorStatus());
			System.err.println("V2 Trap PDU Error Index..... " + ((SnmpPduRequest)pdu).getErrorIndex());
		}

		int k = pdu.getLength();

		for (int i = 0; i < k ; i++ )
		{
			SnmpVarBind vb = pdu.getVarBindAt(i);
			System.err.print("Varbind[" + i + "] := " + vb.getName().toString());
			System.err.println(" --> " + vb.getValue().toString());
		}

		try {

			processTrapData(agent, community, pdu);
		}
		catch (SnmpApplicationError e) {
			System.err.println("Unable to process trap!");
		}

   	 }

	/**
	 * Method from SnmpHandler interface used to parse SNMPv1 traps.
	 *
	 * @param session	The Trap Session that received the PDU.
	 * @param agent		The address of the remote sender.
	 * @param port		The remote port where the pdu was transmitted from.
	 * @param community	The decoded community string.
	 * @param pdu		The decoded V1 trap pdu.
	 *
	 */
    public void snmpReceivedTrap(
		SnmpTrapSession 	session,
		 java.net.InetAddress 	agent,
		 int 			port,
		 SnmpOctetString 	community,
		 SnmpPduTrap 		pdu)

	{

		System.err.println("V1 Trap from agent " + agent.toString() + " on port " + port);
		System.err.println("Ip Address................. " + pdu.getAgentAddress() );
		System.err.println("Enterprise Id.............. " + pdu.getEnterprise() );
		System.err.println("Generic ................... " + pdu.getGeneric() );
		System.err.println("Specific .................. " + pdu.getSpecific() );
		System.err.println("TimeStamp ................. " + pdu.getTimeStamp() );
		System.err.println("Length..................... " + pdu.getLength() );


		int k = pdu.getLength();
		for (int i = 0; i < k ; i++ )
		{
			SnmpVarBind vb = pdu.getVarBindAt(i);
			System.err.print("Varbind[" + i + "] := " + vb.getName().toString());
			System.err.println(" --> " + vb.getValue().toString());
		}
		System.err.println("");

		//synchronized(session)
		//{
		//session.notify();
		//}
   	 }

	/**
	 * Process session errors.
	 *
	 * @param session	The trap session in error.
	 * @param error		The error condition.
	 * @param ref		The reference object, if any.
	 */
    public void snmpTrapSessionError(
		SnmpTrapSession 	session,
		int 			error,
 		java.lang.Object 	ref)
	{

		System.err.println("SnmpAdapter:snmpTrapSessionError> An error occured in the trap session");
		System.err.println("Session error code = " + error);
		if(ref != null)
		{
			System.err.println("SnmpAdapter:snmpTrapSessionError> Session error reference: " + ref.toString());
		}

		if(error == SnmpTrapSession.ERROR_EXCEPTION)
		{
			synchronized(session)
			{
				session.notify(); // close the session
			}
		}
   	 }


	// ==================================================================
    //
    //  Methods.
    //
    // ==================================================================
}
