/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Areski Flissi.
Contributor(s): 

====================================================================*/

package org.objectweb.ccm.chat.cif;

import org.objectweb.ccm.chat.*;

/**
 * This is the implementation of the OMG IDL3 chat::Client component type.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * This class also implements the java.awt.event.ActionListener to manage
 * users' interactions on the Swing GUI associated to this component.
 *
 * @author <a href="mailto:Areski.Flissi@lifl.fr">Areski Flissi</A>
 */
public class ClientImpl
     extends org.objectweb.ccm.chat.ClientSessionComposition.ComponentImpl
  implements java.awt.event.ActionListener
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The name of the component. */
    private String name_;
    
    /** To refer to the GUI frame. */
    private java.awt.Frame frame_;
    /** To refer to the GUI user input field. */
    private java.awt.TextField text_;
    /** To refer to the GUI user output area. */
    private java.awt.TextArea textArea_;
    
    public static final java.awt.Color white_ = java.awt.Color.white;
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ClientImpl()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL Components::EnterpriseComponent
    //
    // ==================================================================

    /**
     * Complete the component configuration.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
    // Checks if the configuration is completed.
    if(name_ == null)
        throw new org.omg.Components.InvalidConfiguration();

    // Check if the connection to the server is set.
    if(get_context().get_connection_the_service() == null)
        throw new org.omg.Components.InvalidConfiguration();

    //Instantiating the GUI.
	// Creates a AWT Frame.
	frame_ = new java.awt.Frame(name_ + "'s Chat GUI");
		
	// Creates a text area for displaying inputs.
	textArea_ = new java.awt.TextArea();
	textArea_.setEditable(false);
    textArea_.setBackground(white_);
         
         
    //Creates a text field for user's inputs.
    text_ = new java.awt.TextField("", 1);
    // Creates a button to invoke the connected server component.
    java.awt.Button button = new java.awt.Button("Print Text");
    button.addActionListener(this);
	// Constructs and shows the GUI.
		java.awt.Panel panel = new java.awt.Panel(
								new java.awt.BorderLayout()
									);    
	System.out.println("___________________________");              						   
	frame_.setLayout ( new java.awt.BorderLayout() ) ;
	frame_.add(panel);
                 
	panel.add(textArea_,java.awt.BorderLayout.NORTH); 
    panel.add(text_,java.awt.BorderLayout.CENTER);
    panel.add(button,java.awt.BorderLayout.SOUTH);  
    
    frame_.setSize(400,300);
    frame_.pack();
    frame_.show();
    
    }

    // ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
         // Release the associated frame.
        if(frame_!=null)       
		    frame_.dispose();
		frame_ = null;
    }

    // ==================================================================
    //
    // Methods for OMG IDL chat::CCM_NamedComponent_Executor
    //
    // ==================================================================

    /**
     * The mutator method for the attribute name.
     *
     * @param n The name.
     */
    public void
    name(String n)
    {
        name_ = n;

		if (frame_ != null)
		   frame_.setTitle(name_ + "'s Client GUI");
    }

    /**
     * The accessor method for the attribute name.
     *
     * @return The name.
     */
    public String
    name()
    {
        return name_;
    }



    // ==================================================================
    //
    // Methods for interface java.awt.event.ActionListener
    //
    // ==================================================================

    /**
     * When the button is selectionned.
     *
     * @param e The associate ActionEvent.
     */
    public void
    actionPerformed(java.awt.event.ActionEvent e)
    {
        // Obtain the object reference associated to the
        // 'the_service' receptacle.
        Service service = get_context().get_connection_the_service();

        // Check if the connection is available.
        if(service == null)
        {
            System.err.println("The chat::Client::the_service receptacle is not set!");
            return;
        }
 
        // Calls the display service.
		service.display(name_ + ":" + text_.getText());
        
        // Reset entered text
        text_.setText(""); 
        
    }
    
    
    
    // ==================================================================
    //
    // Methods for OMG IDL chat::CCM_TextEventConsumer
    //
    // ==================================================================

    /**
     * To receive chat::Event events published by chat::Server components.
     *
     * @param event The received event.
     */
    public void
    push(TextEvent event)
    {
        // Put the text into the text area.
		textArea_.append(event.text + "\n");
    }
    
}
