/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________.

====================================================================*/

package DiningPhilosophers.cif;

import DiningPhilosophers.*;

/**
 * This is the implementation of the fatet 'the_fork'.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class ForkSegImpl
     extends DiningPhilosophers.ForkManagerSessionComposition.ForkSeg
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Tells if the fork is available. */
    private boolean available_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    ForkSegImpl()
    {
        available_ = true;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL DiningPhilosophers::Fork
    //
    // ==================================================================

    /**
     * Obtain the fork.
     *
     * @exception InUse
     *            Thrown if the fork has already used by another philosopher.
     */
    public void
    get()
    throws InUse
    {
        // Check if there is no current philosopher.
        if (! available_)
        {
            throw new InUse();
        }

        // The philosopher is now the current fork user.
        available_ = false;
    }

    /**
     * Release the fork.
     */
    public void
    release()
    {
        // Check if the philosopher is the current one.
        if (available_)
        {
            return;
        }

        // There is no current fork user.
        available_ = true;
    }
}
