/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Areski Flissi, Frederic Briclet.
Contributor(s): 

====================================================================*/
package org.objectweb.ccm.filetransfer.cif;

import org.objectweb.ccm.filetransfer.*;

import org
    .objectweb
    .ccm
    .filetransfer
    .SystemManagerSessionComposition
    .ComponentImpl;

/**
 * @author <a href="mailto:Areski.Flissi@lifl.fr">Areski Flissi</A>
 */
public class SystemManagerImpl
    extends ComponentImpl
    implements java.awt.event.ActionListener, java.awt.event.ItemListener {

    protected TV tv;

    protected org.omg.Components.Cookie ck[] =
        new org.omg.Components.Cookie[50];
    protected String named[] = new String[50];
    protected int rank = 0;
    protected String selected_comp;

    /** To refer to the GUI frame. */
    private java.awt.Frame frame_;
    /** To refer to the GUI user input field. */
    private java.awt.Label label_;
    private java.awt.TextField text_;
    /** To refer to the GUI user output area. */
    private java.awt.TextArea textArea_;

    private java.awt.Button button_tv;
    private java.awt.Button button_tv_rem;
    private java.awt.Choice choice_;
    private java.awt.CheckboxGroup cbg;
    private java.awt.Checkbox pc;
    private java.awt.Checkbox pda;

    int balancy_flag = 1;

    public static final java.awt.Color white_ = java.awt.Color.white;
    public static final java.awt.Color dark_ = java.awt.Color.darkGray;

    //  ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public SystemManagerImpl() {
    }

    //  ==================================================================
    //
    // Methods for OMG IDL Components::EnterpriseComponent
    //
    // ==================================================================

    /**
     * Complete the component configuration.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.     
     */
    public void configuration_complete()
        throws org.omg.Components.InvalidConfiguration {

        //Instantiating the GUI.

        java.awt.GridBagLayout gridbag = new java.awt.GridBagLayout();
        java.awt.GridBagConstraints c = new java.awt.GridBagConstraints();

        // Creates a AWT Frame.
        frame_ = new java.awt.Frame("The System Manager GUI");

        frame_.setSize(400, 300);

        frame_.setLayout(gridbag);

        c.fill = java.awt.GridBagConstraints.BOTH;
        c.weightx = 1.0;
        c.gridwidth = java.awt.GridBagConstraints.REMAINDER;

        // Creates a text area for displaying inputs.
        textArea_ = new java.awt.TextArea(15, 60);
        textArea_.setEditable(false);
        textArea_.setBackground(dark_);
        textArea_.setForeground(white_);
        gridbag.setConstraints(textArea_, c);

        // Component's name label
        c.gridwidth = 1;
        c.fill = java.awt.GridBagConstraints.BOTH;
        c.weighty = 0.0;
        c.weightx = 0.0;
        label_ = new java.awt.Label("Name:");
        gridbag.setConstraints(label_, c);

        // Creates a text field for user's inputs.
        c.weightx = 1.0;
        c.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        text_ = new java.awt.TextField("", 1);
        gridbag.setConstraints(text_, c);
        
        // To choose the destination node
        c.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        c.weighty = 0.0;
        c.weightx = 0.0;
        cbg = new java.awt.CheckboxGroup();
        pc = new java.awt.Checkbox("on PC",cbg,true);
        gridbag.setConstraints(pc, c);
        
        c.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        c.weighty = 0.0;
        c.weightx = 0.0;
        pda = new java.awt.Checkbox("on PDA",cbg,false);
        gridbag.setConstraints(pda, c);

        c.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        c.weighty = 0.0;
        c.weightx = 1.0;
        // Creates buttons.
        button_tv = new java.awt.Button("Create a TV component");
        button_tv.addActionListener(this);
        gridbag.setConstraints(button_tv, c);

        c.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        c.weighty = 0.0;
        c.weightx = 1.0;
        choice_ = new java.awt.Choice();
        choice_.addItemListener(this);
        choice_.disable();
        gridbag.setConstraints(choice_, c);

        c.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        c.weighty = 0.0;
        c.weightx = 1.0;
        button_tv_rem = new java.awt.Button("Remove a TV component");
        button_tv_rem.addActionListener(this);
        button_tv_rem.disable();
        gridbag.setConstraints(button_tv_rem, c);

        frame_.add(textArea_);
        frame_.add(label_);
        frame_.add(text_);
        frame_.add(pc);
        frame_.add(pda);
        frame_.add(button_tv);
        frame_.add(choice_);
        frame_.add(button_tv_rem);

        
        frame_.pack();
        frame_.show();

    }

    //  ==================================================================
    //
    // Methods for the OMG IDL org.omg.Components.SessionComponent
    //
    // ==================================================================

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void ccm_remove() throws org.omg.Components.CCMException {
        // Release the associated frame.

    }

    // ==================================================================
    //
    // Methods for OMG IDL testgroup::CCM_SystemManager
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.ccm.testgroup.ManageServiceOperations#create_component()
     */
    public void create_component(String component_name) {

        // Getting the component's home ref
        //        TVHome tvhome_ =
        //            this.get_context().get_connections_tv_homes()[0].objref;

        

        try {

            // Getting the HomeFinder
            //Fred modification
            /* org.omg.CosNaming.NameComponent[] nc =
               new org.omg.CosNaming.NameComponent[1];

               nc[0] = new org.omg.CosNaming.NameComponent("HomeFinder", "");
               org.omg.CORBA.Object objc =
               org
               .objectweb
               .ccm
               .CORBA
               .TheNameService
               .getNamingContext()
               .getNamingContext()
               .resolve(nc);
               
               org.objectweb.ccm.Deployment.components.HFinder.HFinderManager hf =
               org.objectweb.ccm.Deployment.components.HFinder.HFinderManagerHelper.narrow(objc);
               org.objectweb.ccm.Deployment.components.HFinder.HomeFinder hfinder = 
               hf.provide_the_home_finder();
            */
            org.omg.CosNaming.NamingContextExt context=
                org.omg.CosNaming.NamingContextExtHelper
                .narrow(org.objectweb.ccm.CORBA.TheNameService
                        .getNamingContext()
                        .getNamingContext());
            
            org.omg.Components.HomeFinder hfinder=
                org.omg.Components
                .HomeFinderHelper
                .narrow(context.resolve(context.to_name("OpenCCM/Service/HomeFinder")));   
               
            // Find the specific home by its name
            org.omg.CORBA.Object ccmhome_obj =
                hfinder.find_home_by_name(this.compute_home());

            TVHome tvh = TVHomeHelper.narrow(ccmhome_obj);

            // Create a new component instance
            tv = tvh.create();

            // Configure the component
            tv.the_name(component_name);

            // Interconnect components

            org.omg.CosNaming.NameComponent[] ncomp =
                new org.omg.CosNaming.NameComponent[1];

            //            ncomp[0] = 
            //                new org.omg.CosNaming.NameComponent("Channel" + component_name, "");

            // For demonstration use only. 
            // To do : replace with the previous commented lines
            // (always connect new TV component to existent "ChannelF2" component
            // to avoid unavailable names)

            ncomp[0] = new org.omg.CosNaming.NameComponent("ChannelF2", "");

            org.omg.CORBA.Object obj_ =
                org
                .objectweb
                .ccm
                .CORBA
                .TheNameService
                .getNamingContext()
                .getNamingContext()
                .resolve(ncomp);

            org.objectweb.ccm.filetransfer.Channel chann_ =
                ChannelHelper.narrow(obj_);

            this.ck[this.rank] =
                chann_.subscribe_to_tvs(tv.get_consumer_from_channels());

            // Configuration completion
            tv.configuration_complete();

            this.named[this.rank] = component_name;
            this.rank = this.rank + 1;

            // Register the new component into the NameService

            // Obtain the Name Service.
            org.objectweb.ccm.CORBA.NamingContext ns =
                org.objectweb.ccm.CORBA.TheNameService.getNamingContext();

            ns.rebind("TV" + component_name, tv);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /* (non-Javadoc)
     * @see org.objectweb.ccm.testgroup.ManageServiceOperations#create_home()
     */
    public void create_home(String home_name) {
        // TODO Auto-generated method stub

    }

    public void destroy_component(String name) {

        try {

            // Disable the connexion

            // Getting the Channel component connected to the TV component to destroy
            org.omg.CosNaming.NameComponent[] ncomp =
                new org.omg.CosNaming.NameComponent[1];

            //TO DO : replace <"F2"> with <name>
            ncomp[0] =
                new org.omg.CosNaming.NameComponent("Channel" + "F2", "");

            org.omg.CORBA.Object chann_obj =
                org
                    .objectweb
                    .ccm
                    .CORBA
                    .TheNameService
                    .getNamingContext()
                    .getNamingContext()
                    .resolve(ncomp);

            org.objectweb.ccm.filetransfer.Channel channel =
                ChannelHelper.narrow(chann_obj);

            // Unsubscribe the event  

            // Getting the cookie 
            int index = 0;

            for (int i = 0; i < rank; i++) {
                if (named[i].compareTo(name) == 0) {

                    index = i;
                }

            }

            channel.unsubscribe_to_tvs(this.ck[index]);

            // Getting the TV component to destroy

            ncomp[0] = new org.omg.CosNaming.NameComponent("TV" + name, "");

            org.omg.CORBA.Object tv_obj =
                org
                    .objectweb
                    .ccm
                    .CORBA
                    .TheNameService
                    .getNamingContext()
                    .getNamingContext()
                    .resolve(ncomp);

            org.objectweb.ccm.filetransfer.TV the_tv = TVHelper.narrow(tv_obj);

            // Remove the TV named component    
            the_tv.remove();

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /* (non-Javadoc)
     * @see org.objectweb.ccm.testgroup.ManageServiceOperations#destroy_home(java.lang.String)
     */
    public void destroy_home(String name) {
        // TODO Auto-generated method stub

    }

    public String compute_home() {

        // TO DO : put the load balancing code here
        /*if (balancy_flag == 1)
            balancy_flag = 2;
        else
            balancy_flag = 1;*/
            
        if (cbg.getSelectedCheckbox().getLabel()=="on PC") balancy_flag = 1;
        else balancy_flag = 2;  
            
            
        String the_home_name = "TVHome" + balancy_flag;
               

        return the_home_name;
    }

    //  ==================================================================
    //
    // Methods for interface java.awt.event.ActionListener
    //
    // ==================================================================

    /**
     * When the button is selectionned.
     *
     * @param e The associate ActionEvent.
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {

        Object source = e.getSource();
        int flag = 0;
        if (source == button_tv) {
               
            String target =  cbg.getSelectedCheckbox().getLabel();  
                
            if (rank == 0) {

                choice_.enable();
                button_tv_rem.enable();
                this.create_component(text_.getText());
                textArea_.append(
                                        " <"
                                            + text_.getText()
                                            + ">"
                                            + " TV created "
                                            + target
                                            + "\n");
                // Adding the new component to the choice list
                choice_.add(text_.getText());
                choice_.select(text_.getText());
                this.selected_comp = text_.getText();
            } else {
                // Check if component name already exists
                for (int i = 0; i < rank; i++) {
                    if (named[i].compareTo(text_.getText()) == 0) {
                        textArea_.append(
                            "This name <"
                                + text_.getText()
                                + "> already exists...operation cancelled"
                                + "\n");
                        flag = 1;
                        i = rank;
                    }
                }
                if (flag == 0) {
                    // Calls the manage system service : create component
                    choice_.enable();
                    button_tv_rem.enable();
                    this.create_component(text_.getText());
                    
                    
                    textArea_.append(
                        " <"
                            + text_.getText()
                            + ">"
                            + " TV created "
                            + target
                            + "\n");
                    
                    // Adding the new component to the choice list
                    choice_.add(text_.getText());
                    choice_.select(text_.getText());
                    this.selected_comp = text_.getText();
                }

            }

        } else if (source == button_tv_rem) {

            // Getting the component name to destroy
            if (this.selected_comp != null) {

                this.destroy_component(this.selected_comp);

                //this.destroy_component(text_.getText());
                textArea_.append(" <"+
                    this.selected_comp + ">" + " TV destroyed" + "\n");

                // Remove the name in list
                choice_.remove(this.selected_comp);
                for (int i = 0; i < rank; i++) {
                    if (named[i].compareTo(this.selected_comp) == 0) {
                        named[i] = "DESTROYED";
                        i = rank;
                    }
                }
                if (choice_.getItemCount() != 0) {
                    choice_.select(0);
                    this.selected_comp = choice_.getItem(0);
                } else {
                    choice_.disable();
                    button_tv_rem.disable();
                }

            } else {
                textArea_.append(
                    this.selected_comp
                        + " cannot be removed or does not exist..."
                        + "\n");
            }

        }
        // Reset entered text
        text_.setText("");

    }

    public void itemStateChanged(java.awt.event.ItemEvent evt) {

        this.selected_comp = evt.getItem().toString();

    }

}
