/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Romain Rouvoy_________________________.

====================================================================*/

package org.coach.reactorDemo;


/**
 * The demo1 application deployment.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</A>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</A>
 */

public class reactorDemo
{
    /**
     * The bootstrap of the demo1 application.
     */
    public static void
    main(String[] args)
    throws Exception
    {
       String _OTS = System.getProperties().getProperty("TRANSACTIONAL_PLUGIN","no").toLowerCase();

       // Init the ORB.
       System.out.println("Initializing the ORB...");

       // TODO: Need to report this ORB.init() problem to OpenORB developers!
       // Due to an OpenORB problem, the following line:
       //   org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);
       // are replaced by the next line.

       // Init the OpenCCM Components Runtime.
       org.omg.CORBA.ORB orb = org.objectweb.ccm.Components.Runtime.init(args);

       // Obtain the Name Service.
       System.out.println("Obtaining the Name Service...");
       org.omg.CORBA.Object obj = orb.resolve_initial_references("NameService");
       org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);
       
//        String[] ids = orb.list_initial_services();
//        System.out.println("Rfrences initiales disponibles : ");
//        for (int i=0;i<ids.length;i++)
//          System.out.println("-> "+ids[i]);

       org.omg.CosTransactions.Current current = null;
       if (_OTS.equals("yes"))
         {
            System.out.println("Obtaining the Transaction Service...");
            org.omg.CORBA.Object objOTS = orb.resolve_initial_references("TransactionCurrent");
            current = org.omg.CosTransactions.CurrentHelper.narrow( objOTS );
         }

       try
         {
            if ((current!=null)&&(_OTS.equals("yes")))
              {
                 System.out.println("Status = "+current.get_status());
                 System.out.println("Beginning the transaction...");
                 current.begin();
              }
            
            // Obtain the component servers.
            System.out.println("Obtaining Component Servers...");
            org.omg.CosNaming.NameComponent[] ncomp = new org.omg.CosNaming.NameComponent[1];
            ncomp[0] = new org.omg.CosNaming.NameComponent("ComponentServer1", "");
            obj = nc.resolve(ncomp);
            org.objectweb.ccm.Deployment.Server server1 = org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);
            ncomp[0].id = "ComponentServer2";
            obj = nc.resolve(ncomp);
            org.objectweb.ccm.Deployment.Server server2 = org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);

            // Obtain the container homes and archive installators.
            org.omg.Components.Deployment.ComponentServer server1_cs = server1.provide_component_server();
            org.omg.Components.Deployment.ComponentInstallation server1_inst = server1.provide_install();
            org.omg.Components.Deployment.ComponentServer server2_cs = server2.provide_component_server();
            org.omg.Components.Deployment.ComponentInstallation server2_inst = server2.provide_install();

            // Install archives.
            System.out.println("Installing archives...");
            // get the current directory and store it as a String
            String demoPath = null;
            try {
                demoPath = new java.io.File(".").getCanonicalPath()+ java.io.File.separator ;
            }catch(Exception e) {
                e.printStackTrace();
            }

            server1_inst.install("reactorDemo", "file:" + demoPath + "./archives/reactorDemo.jar");
            server2_inst.install("reactorDemo", "file:" + demoPath + "./archives/reactorDemo.jar");

            // install some plugins already provided by OpenCCM.
            server1_inst.install("openccm_plugins", "file:" + demoPath + "./archives/OpenCCM_Plugins.jar");
            server2_inst.install("openccm_plugins", "file:" + demoPath + "./archives/OpenCCM_Plugins.jar");

            // declaring the container types
            //
            //  Uncomment this part for a minimal configuration
            //  and comment the following one.
            //
            //
             String cont_config = 
             "container = CONTAINER.container ; " +
             "config_home = container.create_system_home("+
             "\"openccm_plugins\", " +
             "\"EmptyConfig\", "+
             "\"org.objectweb.ccm.Containers.Plugins.EmptyConfigurationHome.create\") ;"+
             "config = config_home.create_component(JAVA.null) ;" +
             "container.create_system_home("+
             "\"openccm_plugins\", " +
             "\"EmptyCoordinatorHome\", "+
             "\"org.objectweb.ccm.Containers.Plugins.EmptyCoordinatorHome.create\") ;"+
             "container.create_system_home("+
             "\"openccm_plugins\", " +
             "\"EmptyControllerHome\", "+
             "\"org.objectweb.ccm.Containers.Plugins.EmptyControllerHome.create\") ;"+
             "container.set_container_configuration(config) ;";
             
            org.omg.Components.Deployment.Container server1_cont = server1_cs.create_container(new org.omg.Components.ConfigValue[0]);
            org.omg.Components.Deployment.Container server2_cont = server2_cs.create_container(new org.omg.Components.ConfigValue[0]);

            // Install homes.
            System.out.println("Instantiating homes...");
           
            java.lang.String home_config =
              "container.set_home_configuration(container.get_container_configuration()) ;" +
              "container.set_component_configuration(container.get_container_configuration()) ;";

            org.omg.Components.CCMHome h2 = server1_cont.install_home("reactorDemo", "org.coach.reactorDemo.TweeHomeComponentImpl.create_home", new org.omg.Components.ConfigValue[0]);
            TweeHome sh = TweeHomeHelper.narrow(h2);

            org.omg.Components.CCMHome h1 = server2_cont.install_home("reactorDemo", "org.coach.reactorDemo.EenHomeComponentImpl.create_home", new org.omg.Components.ConfigValue[0]);
            EenHome ch = EenHomeHelper.narrow(h1);
          
            // Create components.
            System.out.println("Instantiating components");
            Twee s1 = sh.create();
            Twee s2 = sh.create();
            Een c1 = ch.create();
            Een c2 = ch.create();
            Een c3 = ch.create();
            // Configure components.
            System.out.println("Configuring components...");
            s1.the_name("De Ene Twee Server");
            s2.the_name("De Andere Twee Server");
            c1.the_name("Een 1");
            c2.the_name("Een 2");
            c3.the_name("Een 3");
            
            // Connect clients and server.
            System.out.println("Interconnecting components...");
            i_Aap aapRef1 = s1.provide_ditIsAap();
            i_Aap aapRef2 = s2.provide_ditIsAap();
            c1.connect_gebruiktAap(aapRef1);
            c2.connect_gebruiktAap(aapRef1);
            c3.connect_gebruiktAap(aapRef2);

            // Configuration completion.
            System.out.println("Configuration completion...");
            s1.configuration_complete();
            s2.configuration_complete();
            c1.configuration_complete();
            c2.configuration_complete();
            c3.configuration_complete();
            
            System.out.println("Trying to get to the reactor...");
	    myEnumUnionHolder eenEnum=new myEnumUnionHolder();
	    myEnumUnion aapje=new myEnumUnion();
	    aapje.Dummy((short)10);
	    eenEnum.value = aapje;
	    	
for (int k=1; k<2; k++)
          try {  
short aval = eenEnum.value.Dummy();
            System.out.println("before reactor eenEnum="+ aval + "server name is: " + s2.the_name());
            aapRef1.method3("een string",  eenEnum, "een andere string");
            short val = eenEnum.value.Dummy();
            System.out.println("eenEnum na reactor=" + val);
          } catch (Exception e) {
               System.out.println("Error......");
                 // dump stack
               e.printStackTrace();
          };
        
         } catch (Exception e) {
            if ((current!=null)&&(_OTS.equals("yes")))
              {
                 System.out.println("Error during deployment :");
                 e.printStackTrace();
                 System.out.print("Rolling Back ... ");
                 current.rollback();
                 System.out.println("Done");
                 // Force to exit.
                 System.exit(0);
              } else {
                 System.out.println("Error during deployment :");
                 e.printStackTrace();
        }
         }
       
       if ((current!=null)&&(_OTS.equals("yes")))
         {
            System.out.print("Do you want to commit the Deployment ? [Y/n] ");
            
            java.io.BufferedReader _buffer
              = new java.io.BufferedReader(new java.io.InputStreamReader(System.in)); 
            char _answer = (char) _buffer.read();
            
            if ((_answer=='n')||(_answer=='N'))
              {
                 System.out.print("Rolling Back ... ");
                 current.rollback();
                 System.out.println("Done");
              }        else {
                 System.out.print("Committing ... ");
                 current.commit(false);
                 System.out.println("Done");
              }
         }
       
       // Force to exit.
       System.exit(0);
    }
}







