/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.actor;

import java.io.*;

/** The java API's imports */

/**
 * This class defines the main frame
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class Main {
    /**
     * Default constructor
     */
    public Main(File contextPropertyFile, File[] browserPropertyFile) {
        System.out.println("Actor started");
        // Adds the interface repository into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("InterfaceRepository");
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // IR is not started
        }
            
        // Use Runtime.init to register value type factories
        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.ComponentPortDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.ComponentPortDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.ConfigValueHelper.id(),
                               new org.objectweb.ccm.Components.ConfigValueFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.ConnectionDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.ConnectionDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.ConsumerDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.ConsumerDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.CookieHelper.id(),
                               new org.objectweb.ccm.Components.CookieFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.EmitterDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.EmitterDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.FacetDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.FacetDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.FacetDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.FacetDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.PublisherDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.PublisherDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.ReceptacleDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.ReceptacleDescriptionFactory());

        org.objectweb.ccm.CORBA.TheORB.register_value_factory(org.omg.Components.SubscriberDescriptionHelper.id(),
                               new org.objectweb.ccm.Components.SubscriberDescriptionFactory());
    
        // Obtain the ORB.
        org.omg.CORBA.ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();
        org.coach.idltree.IdlNode.setOrb(orb);
        
        // Obtain the Name Service.
        System.err.println("Actor: Obtaining the Name Service...");
        try {
            org.omg.CORBA.Object obj = orb.resolve_initial_references("NameService");
            org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not started
        }
            
        try {
            org.coach.actor.namingBrowser.NameModel model = new org.coach.actor.namingBrowser.NameModel(orb);       
            org.coach.actor.namingBrowser.NameTree tree = new org.coach.actor.namingBrowser.NameTree(orb, model);
            tree.setTitle("COACH NameServer Browser");      
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }
}
