/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.actor.namingBrowser;

import javax.swing.tree.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;

public class NameNode extends DefaultMutableTreeNode {
    private static org.omg.CORBA.ORB orb;
    private boolean explored = false;
    private Binding binding;
    private NamingContext context;
    private org.omg.CORBA.Object obj;

    public NameNode(Binding bi, NamingContext nc) {
        binding = bi;
        context = nc;
        setUserObject(this);
    }

    public static void setOrb(org.omg.CORBA.ORB o) {
        orb = o;
    }

    public static org.omg.CORBA.ORB getOrb() {
        return orb;
    }
    
    public boolean getAllowsChildren() {
        return isContext();
    }

    public boolean isLeaf() {
        return !isContext();
    }

    public NamingContext getNamingContext() {
        return context;
    }

    public void explore() {
        explore(false);
    }

    public boolean isExplored() {
        return explored;
    }

    public boolean isContext() {
        return context != null;
    }

    public String toString() {
        return binding.binding_name[0].id + " " + binding.binding_name[0].kind;
    }

    public String getIor() {
        String ior = "context!";
        org.omg.CORBA.Object obj = getObject();
        if (obj != null) {
            try {
                ior = orb.object_to_string(obj);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return ior;
    }

    public org.omg.CORBA.Object getObject() {
        if (obj != null) {
            return obj;
        }
        if (!isContext()) {
            try {
                NameNode p = (NameNode)getParent();
                NamingContext ctx = p.getNamingContext();
                obj = ctx.resolve(binding.binding_name);
                return obj;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public boolean ping() {
        if (!isContext()) {
            String ior = getIor();
            org.omg.CORBA.Object obj = orb.string_to_object(ior);
            if (obj != null) {
                try {
                    obj._is_a("dummy");
                } catch (Exception e) {
                    return false;
                }
                return true;
            }
        }
        return false;
    }

    public void explore(boolean force) {
        if(!isExplored() || force) {
            removeAllChildren();
            BindingListHolder blHolder = new BindingListHolder();
            BindingIteratorHolder biHolder = new BindingIteratorHolder();

            context.list(1000, blHolder, biHolder);

            Binding[] children = blHolder.value;

            NamingContext nc;
            try {
                for(int i=0; i < children.length; i++) {
                   if (children[i].binding_type == BindingType.ncontext) {
                        nc = NamingContextHelper.narrow(context.resolve(children[i].binding_name));
                    } else {
                        nc = null;
                    }
                    add(new NameNode(children[i], nc));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

            explored = true;
        }
    }
    
    private void removeAllBindings() {
        int cnt = getChildCount() - 1;
        for (int i = 0; i < cnt; i++) {
            removeBinding(i);
        }
    }

    public void removeBinding(int n) {
        NameNode child = (NameNode)getChildAt(n);
        child.removeBinding();
    }

    public void removeBinding() {
        try {
            NamingContext ctx = ((NameNode)getParent()).getNamingContext();
            ctx.unbind(binding.binding_name);
            removeFromParent();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void printPath(NameComponent[] nc) {
        System.out.print("path:");
        for (int i = 0; i < nc.length; i++) {
            System.out.print(nc[i].id);
        }
    } 
}
