/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import java.util.*;

/**
 * Contains utility functions, mostly for string manipulation.
 *
 * @author    Harold Batteram
 */
public class Util {   
    public static String unscopeName(String name) {
        if (name == null) {
            return name;
        }
        int idx = name.lastIndexOf("::");
        if (idx >= 0) {
            return name.substring(idx + 2);
        }
        return name;
    }
        
    /**
     * Splits a string like xxx.yyy.zzz into a string array with elements xxx, yyy and zzz
     *
     * @param    name    the string that needs to be split.
     * @return    the list of elements resulting from the split.
     */
    public static String[] splitName(String name) {
        return splitName(name, ".");
    }
    
    /**
     * Splits a string like xxx.yyy.zzz into a string array with elements xxx, yyy and zzz
     *
     * @param    name        the string that needs to be split.
     * @param    separator    the separator to be used in splitting.
     * @return    the list of elements resulting from the split.
     */
    public static String[] splitName(String name, String separator) {
        if (name == null) {
            return new String[0];
        }
        StringTokenizer st = new StringTokenizer(name, separator);
        Vector v = new Vector();
        if (name.startsWith(separator)) {
            v.addElement("");
        }
        while (st.hasMoreTokens()) {
            v.addElement(st.nextToken());
        }
        String[] s = new String[v.size()];
        v.copyInto(s);
        return s;
    }
    
    /**
     * Concatenates a list of strings into one string where the separator between the elements is a ".".
     *
     * @param    split    the list of elements to be concatenated.
     * @return    the concatenated string.
     */
    public static String combineName(String[] split) {
        return combineName(split, ".");
    }
      
    /**
     * Concatenates a list of strings into one string with a specified separator between the elements.
     *
     * @param    split        the list of elements to be concatenated.
     * @param    separator    the string to be used as a separator between the elements.
     * @return    the concatenated string.
     */
    public static String combineName(String[] split, String separator) {
        String s = "";
        for (int i = 0; i < split.length; i++) {
            if (i != 0) {
                s += separator;
            }
            s += split[i];
        }
        return s;
    }

    public static String replaceAll(String source, String what, String into) {
        int index = source.indexOf(what);
        if (index == -1) return source;
        String s1 = source.substring(0, index);
        String s2 = source.substring(index + what.length(), source.length());
        return  s1 + into + replaceAll(s2, what, into);
    } 
}
