package org.coach.tracing.server.viewer;

import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;

public class ScaleManager  {
    public final static float DRAW_LINE_WIDTH = 1;
    private float scale = (float)1.0;
    private float scaleDelta = (float)0.25;
    private float minScale = (float)0.25;
    private Font font = null;
    private RenderingHints rh = null;
    private BasicStroke bs = null;
    private BasicStroke bs_dashed = null;
    private BasicStroke bs_circle = null;
    private BasicStroke bs_circle_dashed = null;
    private float dash_size = 5;
    private int defaultIndex = 3;
        
    private static ScaleManager singleton = new ScaleManager();
    
    private ScaleManager() {
        setScale((float)1.0);
    }
    
    public static ScaleManager getScaleManager() {
        return singleton;
    }

    //access methods
    private synchronized void setScale(float s) {
        scale = s;
        font = new Font("Arial", Font.BOLD, (int)(10 * scale));
        rh = new RenderingHints(null);
        rh.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        rh.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        float[] da1 = { (float)scale * dash_size, (float)scale * dash_size };
        float[] da2 = { (float)scale * 3, (float)scale * 3 };
        bs = new BasicStroke(DRAW_LINE_WIDTH * scale, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER);
        bs_dashed = new BasicStroke(DRAW_LINE_WIDTH * scale, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 1.0f, da1, 0);
        bs_circle = new BasicStroke(DRAW_LINE_WIDTH * scale, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER);
        bs_circle_dashed = new BasicStroke(DRAW_LINE_WIDTH * scale, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 1.0f, da2, 0);
    }
        
    public void scaleUp() {
        setScale(scale + scaleDelta);
    }
    
    public void scaleDown() {
        if (scale > minScale) {
            setScale(scale - scaleDelta);
        }
    }
       
    public synchronized float getScale() {
        return scale;
    }

    public synchronized Font getFont() {
        return font;
    }

    public Dimension getStringDimension(String s, Component c) {
        FontMetrics fm = c.getFontMetrics(font);
        //    return new Dimension(fm.stringWidth(s), fm.getHeight());
        //    return new Dimension((int)fm.getStringBounds(s, c.getGraphics()).getWidth(), fm.getHeight());
        return new Dimension((int)fm.getStringBounds(s, c.getGraphics()).getWidth(), (int)fm.getStringBounds(s, c.getGraphics()).getHeight());
    }

    public Dimension getStringDimension(String s, Graphics2D c) {
        FontRenderContext frc = c.getFontRenderContext();
        Font f = c.getFont();
        GlyphVector gv = f.createGlyphVector(frc, s);
        Rectangle2D r2d = gv.getLogicalBounds();
        //    return new Dimension(fm.stringWidth(s), fm.getHeight());
        //    return new Dimension((int)fm.getStringBounds(s, c.getGraphics()).getWidth(), fm.getHeight());
        return new Dimension((int)r2d.getWidth(), (int)r2d.getHeight());
    }

    public RenderingHints getHints() {
        return rh;
    }

    public BasicStroke getStroke() {
        return bs;
    }

    public BasicStroke getStrokeDashed() {
        return bs_dashed;
    }

    public BasicStroke getStrokeCircle() {
        return bs_circle;
    }

    public BasicStroke getStrokeCircleDashed() {
        return bs_circle_dashed;
    }
}
