/*====================================================================

created on 20 aot 2003
OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Hameau Fabien.
Contributor(s): ______________________________________.

*/
package org.objectweb.ccm.Containers.MetaInformation;

/**
 * @author hameau
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class MI_TypeFactory
extends TypeImpl
{
  private InterfaceTypeImpl _tmp_inf;
  private OperationTypeImpl _tmp_op;
  private AttributeTypeImpl _tmp_att;
  private FacetPortTypeImpl _tmp_fpt;
  private ReceptaclePortTypeImpl _tmp_rpt;
  private ConsumerPortTypeImpl _tmp_cpt;
  private EmitterPortTypeImpl _tmp_ept;
  private PublisherPortTypeImpl _tmp_ppt;
  
  /**
    * create_interface_type
    * 
    * This operation returns a new and 
    * configurated InterfaceType class instance
    *
    * @param name    - Name of interface.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the interface type MI
    * @param ik      - Type_kind of related type object, the 
    *                   value should be :
    *                   * COMPONENT_TYPE
    *                   * HOME_TYPE
    *                   * FACET_TYPE
    *                   * RECEPTACLE_TYPE
    * @param ref     - Reference to the related type MI object.
    *
     @return InterfaceType
    * 
    * Created on 21 aot 2003
    */
  
  public InterfaceType create_interface_type(
    String name,
    String rep_id,
    String entrypt,
    type_kind ik,
    TypeImpl ref
    )
  {
    _tmp_inf = new InterfaceTypeImpl();
    
    _tmp_inf._type      = ref;
    _tmp_inf._rep_id    = rep_id;
    _tmp_inf._type_instance_factory_entrypt = entrypt;
    _tmp_inf._name      = name;
    _tmp_inf._type_kind = ik;

    
    return _tmp_inf;
  }
  
  /**
    * create_operation_type
    * 
    * This operation returns a new and 
    * configurated OperationType class instance
    *
    * @param name    - Name of operation.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the operation type MI
    * @param ik      - Type_kind of related type obkect (here
    *                  should be set to INTERFACE_TYPE value.
    * @param ref     - Reference to the related InterfaceType object.
    *
     @return OperationType
    * 
    * Created on 21 aot 2003
    */
  
  public OperationType create_operation_type(
    String name,
    String rep_id,
    String entrypt,
    type_kind ik,
    TypeImpl ref
    
  )
  {
    _tmp_op = new OperationTypeImpl();
    
    _tmp_op._type      = ref;
    _tmp_op._rep_id    = rep_id;
    _tmp_op._name      = name;
    _tmp_op._type_kind = ik;
    _tmp_op._type_instance_factory_entrypt = entrypt;
        
    return _tmp_op;
  }
  
  /**
    * create_attribute_type
    * 
    * This operation returns a new and 
    * configurated OperationType class instance
    *
    * @param name    - Name of attribute.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the attribute type MI
    * @param ik      - Type_kind of related type object. This
    *                  value should be: 
    *                   * COMPONENT_TYPE
    *                   * HOME_TYPE
    *                   * INTERFACE_TYPE
    * @param ref_type - Reference to the related object.
    * @param ref_int  - reference to the reated interface.
    *
     @return AttributeType
    * 
    * Created on 21 aot 2003
    */
  
    public AttributeType create_attribute_type(
      String name,
      String rep_id,
      String entrypt,
      type_kind ik,
      TypeImpl ref_type
    )
    {
      _tmp_att = new AttributeTypeImpl();
    
      _tmp_att._type      = ref_type;
      _tmp_att._rep_id    = rep_id;
      _tmp_att._name      = name;
      _tmp_att._type_kind = ik;
      _tmp_att._type_instance_factory_entrypt = entrypt;
        
      return _tmp_att;
    }
    
  /**
    * create_facet_port_type
    * 
    * This operation returns a new and 
    * configurated OperationType class instance
    *
    * @param name    - Name of facet_port.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the facet_port type MI
    * @param ik      - Type_kind of related type object. This
    *                  value should be COMPONENT_TYPE.
    * @param ref     - Reference to the related object.
    *
     @return OperationType
    * 
    * Created on 21 aot 2003
    */

    public FacetPortType create_facet_port_type(
      String name,
      String rep_id,
      String entrypt,
      type_kind ik,
      TypeImpl ref
   )
    {
      _tmp_fpt = new FacetPortTypeImpl();
  
      _tmp_fpt._rep_id    = rep_id;
      _tmp_fpt._name      = name;
      _tmp_fpt._type_kind = ik;
      _tmp_fpt._type_instance_factory_entrypt = entrypt;
      
      _tmp_fpt._interface_type = null;
      
      return _tmp_fpt;
    }
    
  
  public FacetPortType facet_bind(InterfaceType type_ref)
  {
   try
    {
       _tmp_fpt._interface_type = type_ref;
    }
    catch (ClassCastException e)
    {
      System.err.println("Enable to cast from Type to InterfacesType :-( in facet");    
    };
    
    return _tmp_fpt;
  }
    
  /**
    * create_receptacle_port_type
    * 
    * This operation returns a new and 
    * configurated OperationType class instance
    *
    * @param name    - Name of facet_port.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the receptacle port type MI
    * @param ik      - Type_kind of related type object. This
    *                  value should be COMPONENT_TYPE.
    * @param ref     - Reference to the related object.
    *
     @return OperationType
    * 
    * Created on 21 aot 2003
    */

    public ReceptaclePortType create_receptacle_port_type(
      String name,
      String rep_id,
      String entrypt,
      type_kind ik,
      TypeImpl ref,
      boolean multiplex,
      int max_connections
  
    )
    {
      _tmp_rpt = new ReceptaclePortTypeImpl();
  
      _tmp_rpt._is_multiple = multiplex;
      _tmp_rpt._max_connections = max_connections;
      _tmp_rpt._type      = ref;
      _tmp_rpt._rep_id    = rep_id;
      _tmp_rpt._name      = name;
      _tmp_rpt._type_kind = ik;
      _tmp_rpt._type_instance_factory_entrypt = entrypt;
      _tmp_rpt._interface_type = null;
      
      return _tmp_rpt;
    }
    
    public ReceptaclePortType receptacle_bind(InterfaceType type_ref)
    {
     try
      {
        _tmp_rpt._interface_type = type_ref;
      }
      catch (ClassCastException e)
      {
        System.err.println("Enable to assign InterfacesType :-( in receptacle");    
      };
    
      return _tmp_rpt;
    }
    
  /**
    * create_consumer_port_type
    * 
    * This operation returns a new and 
    * configurated ReceptaclePortType class instance
    *
    * @param name    - Name of consumer port.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the consumer port type MI
    * @param ik      - Type_kind of related type object. This
    *                  value should be COMPONENT_TYPE.
    * @param ref     - Reference to the related object.
    *
     @return ConsumerPortType
    * 
    * Created on 21 aot 2003
    */

    public ConsumerPortType create_consumer_port_type(
      String name,
      String rep_id,
      String entrypt,
      type_kind ik,
      TypeImpl ref
  
    )
    {
      _tmp_cpt = new ConsumerPortTypeImpl();
  
      _tmp_cpt._type      = ref;
      _tmp_cpt._rep_id    = rep_id;
      _tmp_cpt._name      = name;
      _tmp_cpt._type_kind = ik;
      _tmp_cpt._type_instance_factory_entrypt = entrypt;
      
      return _tmp_cpt;
    }
    
  /**
    * create_emitter_port_type
    * 
    * This operation returns a new and 
    * configurated EmitterPortType class instance
    *
    * @param name    - Name of consumer port.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the emitter port type MI
    * @param ik      - Type_kind of related type object. This
    *                  value should be COMPONENT_TYPE.
    * @param ref     - Reference to the related object.
    *
     @return EmitterPortType
    * 
    * Created on 21 aot 2003
    */

    public EmitterPortType create_emitter_port_type(
      String name,
      String rep_id,
      String entrypt,
      type_kind ik,
      TypeImpl ref
    )
    {
      _tmp_ept = new EmitterPortTypeImpl();
  
      _tmp_ept._type      = ref;
      _tmp_ept._rep_id    = rep_id;
      _tmp_ept._name      = name;
      _tmp_ept._type_kind = ik;
      _tmp_ept._type_instance_factory_entrypt = entrypt;
      
      return _tmp_ept;
    }
    
  /**
    * create_publisher_port_type
    * 
    * This operation returns a new and 
    * configurated EmitterPortType class instance
    *
    * @param name    - Name of publisher port.
    * @param rep_id  - IDL OMG repository identifier
    * @param entrypt - Entry point of the publisher port type MI
    * @param ik      - Type_kind of related type object. This
    *                  value should be COMPONENT_TYPE.
    * @param ref     - Reference to the related object.
    *
     @return PublisherPortType
    * 
    * Created on 21 aot 2003
    */

    public PublisherPortType create_publisher_port_type(
      String name,
      String rep_id,
      String entrypt,
      type_kind ik,
      TypeImpl ref
    )
    {
      _tmp_ppt = new PublisherPortTypeImpl();
  
      _tmp_ppt._type      = ref;
      _tmp_ppt._rep_id    = rep_id;
      _tmp_ppt._name      = name;
      _tmp_ppt._type_kind = ik;
      _tmp_ppt._type_instance_factory_entrypt = entrypt;
      
      return _tmp_ppt;
    }

}
