/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base;

/**
 * The DeployerFailureException is a specific DeployerExceptionWrapper
 * which indicate that the deployment of a wrapper failed.  
 *
 * @author <a href="mailto:briclet@lifl.fr">Frederic Briclet</a>
 *
 * @version 0.1
 */
public class DeployerFailureException 
    extends DeployerExceptionWrapper 
{
    //  ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    //The deployer that failed
    public DeployerContext deployer;

    //  ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Main constructor taking in parameter the deployer which failed in
     * its processing, the initial exception and a short explaination.
     * @param deployer The deployer that failed
     * @param e The initial exception that cause the failure
     * @param message A short explaination of the failure
     */
    public 
    DeployerFailureException(DeployerContext deployer,
                             Exception e,
                             String message) 
    {
        super(e, message);
        this.deployer = deployer;
    }

    //  ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    //  ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * Simple accessor to retrieved the deployer that failed.
     * @return
     */
    public DeployerContext 
    getDeployer() 
    {
        return deployer;
    }

}
