/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base;

import java.net.ServerSocket;
import java.util.LinkedList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Enumeration;
import java.io.File;
import java.io.InputStream;
import java.io.Reader;
import java.io.IOException;
import java.util.zip.ZipFile;
import org.ist.coach.DCI.DCIInformation;
import org.objectweb.ccm.descriptor.ZipEntryRetriever;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer.DestinationDeployerFactory;
import org.objectweb.ccm.descriptor.StreamManager;
import org.ist.coach.DCI.AssemblyManager;
import org.omg.Components.CCMObject;
import org.omg.Components.HomeFinder;
import org.omg.Components.HomeRegistration;
import org.omg.CosNaming.NamingContextExt;

/**
 * This class represent the root context of the current deployer tree. It provides
 * all the services required by the deployer.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public abstract class RootDeployerContext 
       extends DeployerContext 
       implements StreamManager{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /*All the service provides by the root context*/
    
    //The current name service to use
    private NamingContextExt naming;
    //all the component created by the deployer
    private LinkedList components;
    //The symbol conversion tool
    private SymbolConvertion sc;
    //The home finder service
    private HomeFinder homefinder;
    //The home registration service
    private HomeRegistration homeregistration;
    //The dci information service to found node
    private DCIInformation dciInfo;
    //The current zip entry retriever use to find file
    private ZipEntryRetriever zipRetriever;
    //The current server socket used to install archive
    private ServerSocket ssocket;
    //The current deployment logger
    private DeploymentLogger deploymentLogger;
    //The upper assembly manager  
    private AssemblyManager assemblyManager;
    //The factory use to produce the right destination deployer
    private DestinationDeployerFactory destinationDeployerFactory;
    //The current error manager
    private ErrorManager errorManager;
    //The current workdirectory
    private String workDir;
    //All the temporary file create for deployment purpose
    private Hashtable temporaryFiles;
    //All the input stream open for deployment
    private LinkedList openInputStreams;
    //All the reader open for deployment
    private LinkedList openReaders;
    //All the open zip file
    private LinkedList openZipFiles;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public RootDeployerContext(){
        components=new LinkedList();
        temporaryFiles=new Hashtable();
        openInputStreams=new LinkedList();
        openReaders=new LinkedList();
        openZipFiles=new LinkedList();
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public accessor methods 
    //
    // ==================================================================

    public void 
    connectNamingContextExt(NamingContextExt naming)
    {
        this.naming=naming;
    }
    
    public NamingContextExt
    getNamingContextExt()
    {
        return naming;
    }
    
    public void 
    connectHomeFinder(HomeFinder homefinder)
    {
        this.homefinder=homefinder;
    }
    
    public HomeFinder 
    getHomefinder()
    {
        return homefinder;
    }
    
    
    public void 
    connectHomeregistration(HomeRegistration homeregistration)
    {
        this.homeregistration=homeregistration;
    }
    
    public HomeRegistration 
    getHomeRegistration()
    {
        return homeregistration;
    }
    
    public void 
    connectDCIInformation(DCIInformation dciInfo)
    {
        this.dciInfo=dciInfo;
    }
    
    public DCIInformation 
    getDCIInformation()
    {
        return dciInfo;
    }
    
    public void 
    connectZipEntryRetriever(ZipEntryRetriever zipRetriever)
    {
        this.zipRetriever=zipRetriever;
    }
    
    public ZipEntryRetriever 
    getZipEntryRetriever()
    {
        return zipRetriever;
    }
    
    public void 
    connectServerSocket(ServerSocket ssocket)
    {
        this.ssocket=ssocket;
    }
    
    public ServerSocket 
    getServerSocket()
    {
        return ssocket;        
    }
    
    public void 
    connectComponent(CCMObject comp)
    {
        components.addLast(comp);   
    }
    
    public void 
    disconnectComponent(CCMObject comp)
    {
        components.remove(comp);   
    }
    public CCMObject [] 
    getComponents()
    {
        return (CCMObject[])components.toArray(new CCMObject[components.size()]);
    }
    
    public void 
    connectConvertTool(SymbolConvertion sc)
    {
        this.sc=sc;
    }
    
    public String 
    convertSymbol(String ch)
    {
        return sc.convertSymbol(ch);
    }
    
    public RootDeployerContext 
    getRootDeployerContext()
    {
        return this;
    }    
    
    
    /**
     * @return
     */
    public DeploymentLogger 
    getDeploymentLogger() 
    {
        return deploymentLogger;
    }

    /**
     * @param logger
     */
    public void 
    connectDeploymentLogger(DeploymentLogger logger) 
    {
        deploymentLogger = logger;
    }

    /**
     * @return
     */
    public AssemblyManager 
    getAssemblyManager() 
    {
        return assemblyManager;
    }

    /**
     * @param manager
     */
    public void 
    connectAssemblyManager(AssemblyManager manager) 
    {
        assemblyManager = manager;
    }

    /**
     * @return
     */
    public DestinationDeployerFactory 
    getDestinationDeployerFactory() 
    {
        return destinationDeployerFactory;
    }

    /**
     * @param factory
     */
    public void 
    connectDestinationDeployerFactory(DestinationDeployerFactory factory) 
    {
        destinationDeployerFactory = factory;
    }

    /**
     * @return
     */
    public ErrorManager 
    getErrorManager() 
    {
        return errorManager;
    }

    /**
     * @param manager
     */
    public void 
    connectErrorManager(ErrorManager manager) 
    {
        errorManager = manager;
    }
    
    public void
    setWorkDir(String workDir)
    {
        this.workDir=workDir;
    }
    
    public String
    getWorkDir()
    {
        return workDir;
    }
    
    public void
    connectTemporaryFile(File file)
    {
        temporaryFiles.put(System.currentTimeMillis()+"",file);
    }

    public void
    connectTemporaryFile(File file, String identifier)
    {
        temporaryFiles.put(identifier,file);
    }
    
    public boolean
    isAlreadyDownloaded(String identifier)
    {
        return temporaryFiles.containsKey(identifier);
    }

    public File
    getTemporaryFile(String identifier)
    {
        return (File)temporaryFiles.get(identifier);
    }
    
    public void
    deleteAllTemporaryFiles()
    {
        if(temporaryFiles.size()<1)return;

        for(Enumeration enum=temporaryFiles.keys();enum.hasMoreElements();){        
            String key=(String)enum.nextElement();
            File f=(File)temporaryFiles.remove(key);
            if(!f.delete())
                f.deleteOnExit();
            }
    }
    
    public void
    connectOpenInputStream(InputStream isr)
    {
        openInputStreams.add(isr);
    }
    
    public void
    closeAllOpenInputStreams()
    throws IOException
    {
       if(openInputStreams.size()>0)            
        for(InputStream r=(InputStream)openInputStreams.removeFirst();
            openInputStreams.size()>0;
            r=(InputStream)openInputStreams.removeFirst())
                r.close();
    }
    
    public void
    connectOpenReader(Reader reader)
    {
        openReaders.add(reader);
    }
    public void
    closeAllOpenReaders()
    throws IOException
    {
      if(openReaders.size()>0)
        for(Reader r=(Reader)openReaders.removeFirst();
            openReaders.size()>0;
            r=(Reader)openReaders.removeFirst())
                r.close();
    }

    public void
    connectOpenZipFile(ZipFile zf)
    {
        openZipFiles.add(zf);
    }
    
    public void
    closeAllOpenZipFile()
    throws IOException
    {
      if(openZipFiles.size()>0)    
        for(ZipFile r=(ZipFile)openZipFiles.removeFirst();
            openZipFiles.size()>0;
            r=(ZipFile)openZipFiles.removeFirst())
                r.close();
    }    
    public void
    clearAllStream()
    throws IOException
    {
        closeAllOpenReaders();
        closeAllOpenInputStreams();
        closeAllOpenZipFile();
        deleteAllTemporaryFiles();
    }
}
