/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.binder;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.util.CannotResolveReferenceException;
import org.omg.CORBA.Object;
import org.omg.Components.AlreadyConnected;
import org.omg.Components.CCMObject;
import org.omg.Components.EventConsumerBase;
import org.omg.Components.InvalidConnection;
import org.omg.Components.InvalidName;
import org.omg.Components.NoConnection;

/**
 * The emitsport deployer is in charge to resolve the emits port
 * involve in the event connection.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class EmitsportDeployer 
       extends EmitsportDeployerContext
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================    
    private CCMObject emitter;
    //  ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * This method return the component emitter involve in the event connection.
     * @return the component emitter
     * @exception CannotResolveReferenceException is thrown if the emitter is not found
     */    
    private org.omg.Components.CCMObject 
    getCCMObjectEmitter() 
    throws CannotResolveReferenceException
    
    {        
        Object ccm;
        if(getComponentinstantiationDeployer()!=null)
             ccm =getComponentinstantiationDeployer().getComponent(); 
        else ccm=getFindbyDeployer().resolveReference(); 
        
        return org.omg.Components.
              CCMObjectHelper.narrow(ccm);
    }    
    /**
     * Connect the given @param consumedObject to the current event producer.
     * 
     * @param consumedObject The consumer object to connect
     * @throws CannotResolveReferenceException is thrown if the emitter is not found
     * @throws Invalidname if the precised emiter port name is invalid
     */    
    public void 
    connectEmiterToConsumer(EventConsumerBase consumedObject)
    throws CannotResolveReferenceException, 
           InvalidName, 
           AlreadyConnected, 
           InvalidConnection
    {
        emitter=getCCMObjectEmitter();
        emitter.connect_consumer(getEmitsport()
                                .getEmitsidentifier()
                                .getValue()
                                 ,consumedObject);
    }
    /**
     * Disconnect the previously connected consumer.
     * 
     * @throws NoConnection is thrown if the connection is already down
     */
    public void 
    disconnectConsumer() 
    throws InvalidName,
           NoConnection
    {
        emitter.disconnect_consumer(getEmitsport()
        .getEmitsidentifier()
        .getValue());
    }
        

}
