/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import java.lang.reflect.Constructor;
import java.util.Hashtable;
import java.util.Iterator;

import org.objectweb.ccm.descriptor.deployerconfig.DeployerconfigImpl;
import org.objectweb.ccm.descriptor.deployerconfig.Destination;

/**
 * DefaultFactory to resolve the right DestinationDeployer compliant with
 * target host requirements. 
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class DefaultDestinationDeployerFactory
       implements DestinationDeployerFactory 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private DeployerconfigImpl deployerConfig;
    private Hashtable supportedDestination;
     
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    public 
    DefaultDestinationDeployerFactory(DeployerconfigImpl deployerConfig)
    {            
        this.deployerConfig=deployerConfig;
        supportedDestination=new Hashtable();
        setupSupportedDestination();
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
   
    private void setupSupportedDestination(){
        for(Iterator it=deployerConfig.getSupporteddestination().getDestinationList().iterator();
            it.hasNext();)
            {
             Destination dest=(Destination)it.next();
             addDeployer(dest.getType(),dest.getDeployerclass());   
            }                
    }
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================   
    /**
     * getDestinationDeployer return the right DestinationDeployer able to
     * work with the destination type precised with @param destinationtype
     * 
     * @param destinationtype the destinationtype key
     * @throws UnknownDestinationDeployer if this target type is not supported 
     */ 
    public DestinationDeployerBase 
    getDestinationDeployer(String destinationtype)
    throws UnknownDestinationDeployer 
    {
       
        if(!supportedDestination.containsKey(destinationtype))
            throw new UnknownDestinationDeployer(destinationtype);
        String clazz=(String)supportedDestination.get(destinationtype);
        try{
            Constructor constru=Class.forName(clazz)
                                .getConstructor(new Class[0]);
            return (DestinationDeployerBase)constru.newInstance(new Object[0]);
        }
        catch(Exception e){
            e.printStackTrace();
            throw new UnknownDestinationDeployer(destinationtype);
        }    
    }
    
    /**
     * addDeployer is used to add a new kind of destination to support. 
     * The prefixIdentifier is the kind key used in Componentassembly 
     * descriptor to precise the destination kind.
     * 
     * @param prefixIdentifier the destination kind key.
     * @param className The full classname include package name
     * where the class can be found for being instantiate.
     */
    public void 
    addDeployer(String prefixIdentifier,
                String className) 
    {
        supportedDestination.put(prefixIdentifier,className);
    }

}
