/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/


package org.objectweb.ccm.descriptor.softpkg.ccm;

//Import all the packages required
import org.objectweb.ccm.descriptor.ZipEntryRetriever;
import java.io.InputStreamReader;
/**
 *
 * Fileinarchive is used to read xml file information in the archive.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class FileinarchiveDeployer 
       extends FileinarchiveDeployerContext 
    {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
     private ZipEntryRetriever zer;
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Retrieve a reader on the file denote by this fileinarchive 
     * element
     */
    public java.io.InputStreamReader 
    getInputStreamReader()
    throws java.io.IOException
    {   
        InputStreamReader isr=new InputStreamReader(getInputStream());
        getRootDeployerContext().connectOpenReader(isr);
        return isr;
    }
    
    /**
     * Retrieve a inputstream on the file denote by this fileinarchive 
     * element. 
     */
    public java.io.InputStream
    getInputStream()
    throws  java.io.IOException
    {   
        if(getLinkDeployer()!=null)
            return getLinkDeployer().getInputStream();
        return getZipEntryRetriever()
                .getZipEntryISIgnoringDelimiterType(getFileinarchive().getName());
    }
    
    /**
     * getInnerArchive return a ZipEntry wrapper on a zip entry. The considered 
     * entry is saw as a zip file. 
     * @return A ZipEntryRetriever plug on the denote archive in archive
     * @throws Exception if a problem occured
     */    
    public ZipEntryRetriever
    getInnerArchive()
    throws Exception
    {
        if(zer==null){
            zer=new ZipEntryRetriever(getInputStream());
            zer.connectStreamManager(getRootDeployerContext());
        }

        return zer;
    }
    /**
     * Start a specific Http server in charge to upload the specific 
     * file in the archive.
     * The server will process only one request. This server is started
     * in another thread.
     * @return The stringified URL to use to download the file
     * @throws java.io.IOException if a I/O problem occured
     */
    public String 
    startHttpServer()
    throws java.io.IOException
    {
               
        (new Thread
            (new org.objectweb.ccm.deploytool.MicroServerHttp
                (getServerSocket(),
                 getInputStream(),
                 getFileinarchive().getName().trim()))).start();
        return 
            "http://" + 
            java.net.InetAddress.getLocalHost().getHostAddress() + ':' +
            getServerSocket().getLocalPort() + '/' +
            getFileinarchive().getName().trim();
    }

    /**
     * Provides the ressource
     */
     public String 
     provideRessource()
     throws java.io.IOException
     {
        if(getLinkDeployer()!=null)
            return getLinkDeployer().getReference();
        else return startHttpServer();
     }

}
