/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.softpkg.ccm;

import java.util.LinkedList;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.DeployerInactiveState;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer.CodeDeployerVisitor;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer.DependencyDeployerVisitor;
import org.omg.Components.ConfigValue;
import org.omg.Components.Deployment.ComponentInstallation;

/**
 * The implementationDeployer is used to resolve the dependencies
 * on a implementation and install the implementation.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ImplementationDeployer 
       extends ImplementationDeployerContext 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================    
    private ComponentInstallation comp;
    private LinkedList config;    
    private String entryPoint;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================    

    /**
     * Internal method to resolve all the dependency describes in the 
     * implementation.
     * @return the resulting configvalue
     * @throws FatalDeploymentException if a fatal error occured 
     */
    private ConfigValue[] 
    processDependencies()
    throws FatalDeploymentException
    {
        config=new LinkedList();
        traverse(this,getDependencyDeployers());
        return (ConfigValue[])config.toArray(new ConfigValue[config.size()]);
    }
    

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * Install the current implementation including all the dependency
     * @param comp the componentinstallation to use for installing
     * @throws FatalDeploymentException if a fatal error occured 
     */    
    public ConfigValue[] 
    install(ComponentInstallation comp)
    throws FatalDeploymentException
    {
        this.comp=comp;
        traverse(this,getCodeDeployers());
        return processDependencies();
    }
    /**
     * The Dependency deployer visitor to solve dependency and code.
     * 
     * @param deployer The dependency to execute
     * @param status The implementationdeployer current status
     * @throws FatalDeploymentException if a fatal error occured
     */    
    public void 
    visit(DependencyDeployer deployer,DeployerInactiveState status)
    throws FatalDeploymentException
    {
        try{
            deployer.resolveDependency(comp);   
        }
        catch(DependencyResolutionFailureException e)
        {
            getErrorManager().submitException(e);    
        }            
    }
    /**
     * The code deployer visitor to install code.
     * 
     * @param codeDeployer the codeDeployer to execute
     * @param status the implementation current status
     * @throws FatalDeploymentException if a fatal error occured 
     */            
    public void 
    visit(CodeDeployer codeDeployer, DeployerInactiveState status)
    throws FatalDeploymentException
    {
        try{
            entryPoint=codeDeployer.intallCode(comp,getImplementation().getId());
        }
        catch(CodeInstallationFailureException e)
        {
            getErrorManager().submitException(e);
        }
    }
    /**
     * Visitor method taking in parameter the visitor to execute on all
     * the dependencies.
     * 
     * @param depVisitor the dependency visitor
     * @throws DependencyResolutionFailureException if an error 
     * occured during the visit.
     */
    public void 
    visitDependencyDeployers(DependencyDeployerVisitor depVisitor)
    throws DependencyResolutionFailureException
    {
        DependencyDeployer depDepl[]=getDependencyDeployers();
        for(int i=0;i<depDepl.length;i++)
            depVisitor.visit(depDepl[i]);
    }
    /**
     * Visitor method taking in parameter the visitor to execute on all
     * the codesdeployer.
     * 
     * @param codeVisitor the codeVisitor
     * @throws CodeInstallationFailureException if an error
     *  occured during the visit.
     */    
    public void
    visitCodeDeployers(CodeDeployerVisitor codeVisitor)
    throws CodeInstallationFailureException
    {
        CodeDeployer codeDepl[]=getCodeDeployers();
           for(int i=0;i<codeDepl.length;i++)
                 codeVisitor.visit(codeDepl[i]);
    }
    /**
     * Return the implementation entrypoint
     * @return the implementation entrypoint
     */
    public String 
    getEntryPoint()
    {
        return  entryPoint;
    }
    
}
