/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet
                      Tran Huynh (tran.huynh@fr.thalesgroup.com)
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.visitorIDL3.xmi;

/**
 * This class generates XMI from IDL3.
 */
public class IDL3_XMI
    extends org.objectweb.ccm.visitorIDL3.common.Generator {

    /**
     * Name of the file to generate.
     */
    private java.lang.String filename_;

    /**
     * Location of the XMI DTD.
     */
    private java.lang.String xmidtd_;

    /**
     * Constructor.
     *
     * @param repository The Interface Repository.
     *
     * @exception fr.lifl.cim.jidlscript.lang.JISError
     */
    public
    IDL3_XMI(org.objectweb.ccm.IDL3.Repository repository)
        throws fr.lifl.cim.jidlscript.lang.JISError {

        super(repository);

        // Create a printer.
        XMIPrinter printer = new XMIPrinter(this, "    ");

        // Use this printer.
        setPrinter(printer);
    }

    /**
     * Setters.
     */

    /**
     * This method sets the XMI DTD.
     *
     * @param xmidtd The XMI DTD.
     */
    public void
    setDTD(java.lang.String xmidtd) {
        xmidtd_ = xmidtd;
    }

    /**
     * This method sets the name of the file to generate.
     *
     * @param filename The filename.
     */
    public void 
    setFilename(java.lang.String filename) {
        filename_ = filename;
    }

    /**
     * Getters.
     */

    /**
     * This method returns the XMI DTD.
     *
     * @return The location of the XMI DTD.
     */
    public java.lang.String
    getDTD() {
        return xmidtd_;
    }

    /**
     * This method returns the name of the file to generate.
     *
     * @return The file name.
     */
    public java.lang.String
    getFilename() {
        return filename_;
    }

    /**
     * This method visits the Interface Repository
     * and generates the file.
     *
     * @param name The IR Object to visit.
     * @param filename The name of the file to generate.
     * @param xmidtd The XMI DTD.
     */
    public void
    generate(java.lang.String name,
             java.lang.String filename,
             java.lang.String xmidtd) {
        setDTD(xmidtd);
        setFilename(filename);
        super.visitObj(name);
    }
}
