/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.ast.utils.api;

// Package dependencies.
import org.objectweb.corba.ast.api.CompositionDecl;
import org.objectweb.corba.ast.api.HomeDecl;
import org.objectweb.corba.ast.api.HomeExecutorDecl;
import org.objectweb.corba.ast.api.ProxyHomeDecl;
import org.objectweb.corba.ast.api.ComponentDecl;
import org.objectweb.corba.ast.api.ExecutorDecl;
import org.objectweb.corba.ast.api.SegmentDecl;

/**
 * This interface provides easy access to composition elements.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public interface CompositionInfo
{
    /**
     * Get the Composition declaration.
     *
     * @return The Composition declaration.
     **/
    public CompositionDecl
    getComposition();

    /**
     * Get the Home declaration.
     *
     * @return The Home declaration.
     **/
    public HomeDecl
    getHome();

    /**
     * Get the Home executor declaration.
     *
     * @return The Home executor declaration.
     **/
    public HomeExecutorDecl
    getHomeExecutor();

    /**
     * Get the Proxy Home declaration.
     *
     * @return The Proxy Home declaration.
     **/
    public ProxyHomeDecl
    getProxyHome();

   /**
     * Get the Component declaration.
     *
     * @return The Component declaration.
     **/
    public ComponentDecl
    getComponent();

    /**
     * Get the Component executor declaration.
     *
     * @return The Component executor declaration.
     **/
    public ExecutorDecl
    getComponentExecutor();

    /**
     * Get segment declarations.
     *
     * @return Segment declarations.
     **/
    public SegmentDecl[]
    getSegments();
}
