/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.generator.translator.ast2idl.api;

import org.objectweb.corba.ast.api.TypeKind;
import org.objectweb.corba.ast.api.Declaration;

/**
 * Utility interface to translate IDL types in string format.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.5
 */

public interface AST_IDLTranslator
{
    /**
     * Translate a type into its idl format.
     *
     * @param typeref - The type reference to translate.
     *
     * @return The resulting string.
     **/
    public String
    toIDL(org.objectweb.corba.ast.api.TypeRef typeref);

    /**
     * Add postfix to the IDL name if typeref is an array.
     *
     * @param typeref - The array to translate.
     *
     * @return The resulting string.
     **/
    public String
    toIDLPostfix(org.objectweb.corba.ast.api.TypeRef typeref);

    /**
     * Get the string value of an Any.
     *
     * @param anyvalue - The Any to translate.
     * @param typeref - The value's type.
     *
     * @return The resulting string.
     **/
    public String
    toString(org.objectweb.corba.ast.api.AnyValue anyvalue,
             org.objectweb.corba.ast.api.TypeRef typeref);

    /**
     * Check if id is an idl keyword.
     *
     * @param id - The string to check
     *
     * @return A valid string.
     */
    public String
    checkKeywords(String id);

    /**
     * Get a valid absolute name from a declaration,
     * i.e. check keywords.
     *
     * @param decl - The declaration to get the name.
     *
     * @return A valid absolute name.
     */
    public String
    getAbsoluteName(Declaration decl);
}
