# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  This class feeds the AST from the differents CIDL elements in the
#  UML repository. This file is build according to the UML Profile
#  Profile for CORBA Components (Version 3.1) [1].
#
# ====================================================================


import NotCreatedException;
import CORBA3Feeder;

class CIDLFeeder (CORBA3Feeder.CORBA3Feeder)
{

    # =================================================================
    # Empty Constructor for the CIDLFeeder.
    # =================================================================
    proc __CIDLFeeder__(self)
    {
        self.__CORBA3Feeder__();
        JAVA.declare("org.objectweb.corba.ast.api");
    }


    # =================================================================
    #   CORBA Module from the UML Profile for CORBA (3.5.4)
    # =================================================================
    proc createCORBAModule (self, element, currentScope)
    {
        # CORBA 2 Module
        createCORBA2Module = CORBA2Feeder.createCORBAModule;
        createCORBA2Module(self, element, currentScope);

        # CIDL Module
        oldPrefix = self.prefix;
        cidlModule = currentScope.declareCidlModule(element.getName());
        for tv in element.getTaggedValue().toArray() {
            if (tv.getType().getTagType() == "typePrefix" || tv.getType().getTagType() == "CORBATypePrefix") {
                self.prefix = tv.getDataValue().get(0);
            }
        }
        cidlModule.setPrefix(self.prefix);
        cidlModule.create();

        for oe in element.getOwnedElement().toArray() {
            if (self.umlOperations.isStereotyped(oe, "CORBAComponentImpl")) {
                self.createCIDLCORBAComponentImpl(oe, cidlModule);
            }
        }

        self.prefix = oldPrefix;
        cidlModule.create();
        self.moduleName = element.getName();

        ## Generation of the CIDL file corresponding to this module
        # JAVA.declare("org.objectweb.corba.generator.cidl.lib");
        # JAVA.declare("org.objectweb.corba.generator.common.lib");
        # gen = org.objectweb.corba.generator.cidl.lib.CIDLGenerator(self.ast);
        # gen.initialize("test.cidl", "XMItoCIDL");
        # gen.generate(cidlModule, org.objectweb.corba.ast.api.DeclarationKind.dk_cidl);
    }


    # =================================================================
    #   4.2.2.1 - CORBA Component Impl
    # =================================================================
    proc createCIDLCORBAComponentImpl (self, element, currentScope)
    {
        impl = self.getImplements(element);
        # Gets the Category Kind of the Composition
        category = "session";
        for tv in element.getTaggedValue().toArray() {
            if (tv.getType().getTagType() == "category") {
                category = tv.getDataValue().get(0).toLowerCase();
            }
        }
        if (category == "entity") {
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_entity;
        } else if (category == "process") {
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_process;
        } else if (category == "service") {
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_service;
        } else {
            category = "session";
            categoryKind = org.objectweb.corba.ast.api.CategoryKind.ck_session;
        }
        # Composition creation
        category = category.substring(0,1).toUpperCase() + category.substring(1).toLowerCase();
        comp = currentScope.startComposition(impl.getName() + category + "Composition");
        comp.setCategoryKind(categoryKind);
        comp.create();
        # home executor
        homeExecutor = comp.startHomeExecutor("HomeImpl");
        homeExecutor.create();
        # implements
        homeExecutor.setHomeType(self.getType(self.getHome(impl)));
        # manages
        exec = homeExecutor.startExecutor("ComponentImpl");
        # segment
        for connect in self.umlOperations.oppositeAssociationEnds(element) {
            segmentElement = connect.getParticipant();
            if (self.umlOperations.isStereotyped(segmentElement, "CORBASegment")) {
                self.createCORBASegmentOnHomeExecutor(segmentElement, exec);
            }
        }
        exec.create();
        homeExecutor.create();
        comp.create();
    }


    proc getImplements (self, element)
    {
        for asso in self.umlOperations.associations(element) {
            if (self.umlOperations.isStereotyped(asso, "CORBAImplements")) {
                for connect in asso.getConnection().toArray() {
                    if (connect.getParticipant() != element) {
                        return connect.getParticipant();
                    }
                }
            }
        }
        return JAVA.null;
    }


    proc getHome(self, element)
    {
        for asso in self.umlOperations.associations(element) {
            if (self.umlOperations.isStereotyped(asso, "CORBAManages")) {
                for connect in asso.getConnection().toArray() {
                    if (connect.getParticipant() != element) {
                        return connect.getParticipant();
                    }
                }
            }
        }
        return JAVA.null;
    }


    # =================================================================
    #   4.2.2.1 - CORBA Component Impl
    # =================================================================
    proc createCORBAComponentImpl (self, element, currentScope)
    {
    }


    # =================================================================
    #   4.2.2.1 - CORBA Implements
    # =================================================================
    proc createCORBAImplements (self, element, currentScope)
    {
    }


    # =================================================================
    #   4.2.2.2 - CORBA Home Impl
    # =================================================================
    proc createCORBAHomeImpl (self, element, currentScope)
    {
    }


    # =================================================================
    #   4.2.2.4 - CORBA Segment
    # =================================================================
    proc createCORBASegment (self, element, currentScope)
    {
    }


    # =================================================================
    #   4.2.2.4 - CORBA Segment
    # =================================================================
    proc createCORBASegmentOnHomeExecutor (self, element, currentScope)
    {
        segment = currentScope.startSegment(element.getName());
        # provides facet
        for artifactEnds in self.umlOperations.oppositeAssociationEnds(element) {
            artifact = artifactEnds.getParticipant();
            if (self.umlOperations.isStereotyped(artifact, "CORBAArtifact")) {
                segment.getFacetList().add(artifact.getName());
            }
        }
        # provides sink
        for tv in element.getTaggedValue().toArray() {
            if (tv.getType().getTagType() == "features") {
                featuresList = tv.getDataValue().get(0);
                featuresToken = java.util.StringTokenizer(featuresList, "[, ]");
                while (featuresToken.hasNext()) {
                    featuresToken.nextToken();
                    feature_name = featuresToken.substring(0, featuresToken.index(":"));
                    storage_member_name = featuresToken.substring(featuresToken.index(":")+1);
                    currentScope.getFeatureDelegationList().add(feature_name, storage_member_name);
                }
            }
        }
        segment.create();
    }


    # =================================================================
    #   4.2.2.4 - CORBA Artifact
    # =================================================================
    proc createCORBAArtifact (self, element, currentScope)
    {
    }

}
