/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ---------------------------------------------

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;

/**
 * This is the implementation of the AssemblyManagerHome component type.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */

public class AssemblyManagerHomeImpl
    extends org.objectweb.openccm
            .OpenCCM_DCI.AssemblyManagerSessionComposition
            .AssemblyManagerHomeImpl
            
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    //Path refering to the xml file used to create the lasted assembly
    private String xml_file_path;
    //the xml code used to create the lasted assembly
    private String xml_assembly;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    AssemblyManagerHomeImpl()
    {
      super();
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for creation of component segments.
    //
    // ==================================================================

    /**
     * Create an executor segment from its identifier.
     *
     * @param segid The executor segment identifier.
     */
    public org.omg.Components.ExecutorSegmentBase
    create_executor_segment(int segid)
    {
        switch(segid){
            case 0
                : if(xml_assembly!=null){
                    String tmp=xml_assembly;
                    xml_assembly=null;
                    return new AssemblyManagerXMLImpl(tmp);                                     
                    }
                else{
                    String tmp=xml_file_path;
                    xml_file_path=null;
                    return new AssemblyManagerImpl(tmp);
                }
                     
            case org.objectweb.openccm.OpenCCM_DCI.
                AssemblyManagerSessionComposition.
                AssemblySeg._segment_id_value
                : return new AssemblySegImpl();
            
        default:System.err.println("AMH >>Invalid segment request"); 
            return null;
        }
        
    }

    // ==================================================================
    //
    // Methods for the deployment.
    //
    // ==================================================================

    /**
     * This method is called by the OpenCCM Component Server
     * to create a home instance.
     */
    public static org.omg.Components.HomeExecutorBase
    create_home()
    {
        
        return new AssemblyManagerHomeImpl();
    }
    
    /**
     * Implementation of the  OpenCCM_DCI::create_assembly_manager(string) 
     * factory method. 
     * It allows to create a AssemblyManager specifying the assembly descriptor
     * file to use.
     * @return The assembly manager component
     */
    public  org.omg.Components.EnterpriseComponent
        create_assembly_manager(String xml_file_path){

        this.xml_file_path=xml_file_path;
        return  create();
    }

    /**
     * Implementation of the  OpenCCM_DCI::create_assembly_manager_xml(string) 
     * factory method. 
     * It allows to create a AssemblyManager specifying directly the XML code
     * to use.
     * @return The assembly manager component
     */
    public  org.omg.Components.EnterpriseComponent
        create_assembly_manager_xml(String xml_assembly){
        
        this.xml_assembly=xml_assembly;        
        return create();
    }

    /**
     * static block to register the AssemblyManagerRemoved event
     * which is used to manage the assembly manager removall.     
     */
    static
    {
        // Required to register the Food valuetype factory to the ORB.
        AssemblyManagerRemovedDefaultFactory.register();
    }

    
    
}
