/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.CORBA;

/** The administration console's API imports */
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;


/** To use the ORB interface. */
import org.omg.CORBA.ORB;

/**
 * Basic implementation of the Context interface
 * for managing CORBA initial references.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public class InitialReferenceContext 
    implements Context, Wrapper {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The orb. */
    private ORB orb_ = null;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Obtains the associated ORB.
     *
     * @return The associated ORB.
     */
    protected ORB getORB() {
        return orb_;
    }

    protected Entry getLocalEntry(String name) {
        if (getORB() != null) {
            Object value = getLocalValue(name);
            return (value != null) ? new DefaultEntry(value, new DefaultName(name), this) : null;
        }
        return null;
    }

    protected Object getLocalValue(String name) {
        // Lookups the name into the CORBA initrial references.
        if (getORB() != null) {
            try {
                return getORB().resolve_initial_references(name);
            } catch (org.omg.CORBA.ORBPackage.InvalidName ex) {
                return null;
                // or MUST BE wrapped in the API by an exception ???
            }
        }
        return null;
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

	/**
	 * Returns an array containing the entries contained by the target context.
	 * @return A new array of entry.
	 */
    public Entry[] getEntries() {
        String[] objects = getORB().list_initial_services();
        Entry[] values = new Entry[objects.length];
        for (int i = 0; i < objects.length; i++)
            values[i] = getLocalEntry(objects[i]);
        return values;
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        orb_ = (ORB) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return orb_;
    }

}
