/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.CORBA;

import org.objectweb.util.browser.core.common.ClassResolver;

/************************************************************************
 **
 **  Ce programme permet d'obtenir le type_id d'un objet CORBA
 **  ainsi que la classe Helper associe permettant de faire un narrow.
 **
 **  Auteur : Dr. Philippe Merle - LIFL & INRIA - 2001
 **
 ************************************************************************/

/************************************************************************
 **
 **  Cette 1re classe permet de partiellement dcoder une IOR.
 **
 **  Auteur initial : Marek Malowidzki - 1999
 **
 ************************************************************************/

final class ModestInputStream {
    private byte bytes[];
    private int index;
    private boolean little_endian;

    public ModestInputStream(byte bytes[]) {
        this.bytes = bytes;
        little_endian = bytes[0] != 0;
        index = 1;
    }

    public boolean getEndian() {
        return little_endian;
    }

    public byte readOctet() {
        return bytes[index++];
    }

    public int readULong() {
        int al = index % 4;
        if (al != 0)
            index += 4 - al;
        if (little_endian)
            return ((bytes[index++]) & 0x000000ff) | ((bytes[index++] << 8) & 0x0000ff00) | ((bytes[index++] << 16) & 0x00ff0000) | ((bytes[index++] << 24) & 0xff000000);
        else
            return ((bytes[index++] << 24) & 0xff000000) | ((bytes[index++] << 16) & 0x00ff0000) | ((bytes[index++] << 8) & 0x0000ff00) | ((bytes[index++]) & 0x000000ff);
    }

    public String readString() {
        int len = readULong() - 1;
        String str = new String(bytes, index, len);
        index += len;
        if (readOctet() != 0)
            throw new org.omg.CORBA.INV_OBJREF("String should end with byte 0");
        return str;
    }
}

/************************************************************************
 **
 ** Classe permettant d'obtenir le type_id d'un objet CORBA
 ** ainsi que sa classe Helper associe.
 **
 ************************************************************************/
/**
 * This class allows to obtain a CORBA Object type_id and its associated Helpper class
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */
public class TypageCORBA {
    // **********************************************************************
    //
    // Ces 3 fonctions suivantes sont issues de ORBacus 4.0.5 for Java.
    // Toutefois elles sont indpendantes de cet ORB car celles-ci ne font
    // que des manipulations de chaines de caracteres.
    //
    // Copyright (c) 2000
    //
    // Object Oriented Concepts, Inc.
    // Billerica, MA, USA
    //
    // All Rights Reserved
    //
    // **********************************************************************
    //
    // Check the given name against Java keywords and reserved suffixes
    //
    public static String fixName(String name) {
        // Version simplifiee !!!
        return name;
    }

    //
    // Convert a repository ID into a class name.
    // See the IDL-to-Java mapping, section 1.13.8.
    //
    public static String idToClassName(String id, String suffix) {
        String result = null;

        if (id != null) {
            if (id.startsWith("IDL:")) {
                try {
                    StringBuffer buf = new StringBuffer();

                    int end = id.lastIndexOf(':');
                    String s;
                    if (end < 0)
                        s = id.substring(4);
                    else
                        s = id.substring(4, end);

                    //
                    // If the ID contains a prefix, then reverse the
                    // dotted components of the prefix
                    //
                    int slash = s.indexOf('/');
                    if (slash > 0) {
                        String prefix = s.substring(0, slash);
                        String rest = s.substring(slash + 1);
                        java.util.StringTokenizer tokenizer = new java.util.StringTokenizer(prefix, ".");
                        while (tokenizer.hasMoreTokens()) {
                            String tok = tokenizer.nextToken();
                            buf.insert(0, fixName(tok) + ".");
                        }

                        s = rest;
                    }

                    //
                    // "Fix" the remainder of the ID
                    //
                    java.util.StringTokenizer tokenizer = new java.util.StringTokenizer(s, "/");
                    while (tokenizer.hasMoreTokens()) {
                        String tok = tokenizer.nextToken();
                        buf.append(fixName(tok));
                        if (tokenizer.hasMoreTokens())
                            buf.append('.');
                    }

                    result = buf.toString() + suffix;
                } catch (IndexOutOfBoundsException ex) { // if id has bad format
                    result = null;
                }
            }
        }

        return result;
    }

    //
    // Convert a repository ID into a class.
    // See the IDL-to-Java mapping, section 1.13.8.
    //
    public static Class idToClass(String id, String suffix) {
        Class result = null;
        String className = idToClassName(id, suffix);

        if (className != null) {
            try {
                result = ClassResolver.resolve(className);
            } catch (ClassNotFoundException ex) {
                // ignore
            }
        }

        return result;
    }

    // **********************************************************************
    //
    // **********************************************************************

    static String decode_type_id(String ior) {
        int nbytes = ior.length() - 4;
        if ((nbytes & 1) == 1)
            throw new org.omg.CORBA.INV_OBJREF("Bad string length");
        nbytes /= 2;
        byte bytes[] = new byte[nbytes];
        for (int i = 0, j = 4; i < nbytes; i++, j += 2) {
            String bstr = ior.substring(j, j + 2);
            try {
                bytes[i] = (byte) Integer.parseInt(bstr, 16);
            } catch (NumberFormatException exc) {
                throw new org.omg.CORBA.INV_OBJREF("Bad byte value: " + bstr);
            }
        }

        ModestInputStream in = new ModestInputStream(bytes);
        return in.readString();
    }

    // **********************************************************************
    //
    // **********************************************************************

    protected String type_id_;

    public TypageCORBA(org.omg.CORBA.Object objref, org.omg.CORBA.ORB orb) {
        String ior = null;
        try {
            ior = orb.object_to_string(objref);
        } catch (Exception e) {
            throw new ObjectToStringNotSupportedException();
        }
        type_id_ = decode_type_id(ior);
    }

    public TypageCORBA(String ior) {
        type_id_ = decode_type_id(ior);
    }

    public String getTypeID() {
        return type_id_;
    }

    public String getJavaInterfaceName() {
        return idToClassName(type_id_, "");
    }

    public Class getJavaInterface() {
        return idToClass(type_id_, "");
    }

    public String getJavaHelperClassName() {
        return idToClassName(type_id_, "Helper");
    }

    public Class getJavaHelperClass() {
        return idToClass(type_id_, "Helper");
    }

}
