/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.Deployment;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import java.net.URL;

import org.objectweb.util.browser.api.IconProvider;

import org.omg.Components.Deployment.Assembly;

/**
 * This class provides an icon depending on the Assembly status (inactive or inservice).
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class AssemblyIcon 
  implements IconProvider 
{

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    /** The associated icons. */
    protected ImageIcon assemblyBuilt_, assemblyNotBuilt_;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    /**
     * Default constructor.
     */
    public AssemblyIcon()
    {
        assemblyBuilt_ = createIcon("icons/assembly_built.png");
        assemblyNotBuilt_ = createIcon("icons/assembly_tear_down.png");
    }

    //==================================================================
    //
    // Internal method.
    //
    //==================================================================

    protected Assembly getAssembly(Object obj){
        return (Assembly)obj;
    }

    /** Returns an ImageIcon, or null if the path is invalid. */
    protected static ImageIcon createIcon(String imageName) 
    {
        if (imageName != null) {
            URL urlFile = null;
            urlFile = Thread.currentThread().getContextClassLoader().getResource(imageName);            
            if(urlFile==null){            
				try {
					urlFile = new URL(imageName);
				} catch (java.net.MalformedURLException e) {
                    System.out.println(imageName + ": Malformed URL !");
				}
            }
            if(urlFile!=null){
                return new ImageIcon(urlFile);
            }
        }
        return null;
    }

    //==================================================================
    //
    // Public methods for IconProvider interface.
    //
    //==================================================================

    /**
     * Returns a javax.swing.Icon according to the given object.
     * Provides an icon representing the status of the Assembly
     */
    public Icon newIcon(Object object)
    {
        Assembly assembly = getAssembly(object);
        org.omg.Components.Deployment.AssemblyState state = assembly.get_state();
        if(!state.equals(org.omg.Components.Deployment.AssemblyState.INACTIVE))
            return assemblyBuilt_;
        else
            return assemblyNotBuilt_;
    }
    
}