/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.menu;

/** The administration console's imports */
import org.objectweb.util.browser.core.common.DynamicTree;

/** The java API's imports */
import java.awt.event.KeyEvent;
import java.awt.Event;
import javax.swing.JPanel;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JToolBar;
import javax.swing.ImageIcon;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.KeyStroke;
import java.net.URL;

/**
 * This class represents the menu,
 * it creates a JMenu and its associated JToolBar.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class Menu extends JPanel {

    /** The menuBar */
    protected JMenuBar menuBar_;

    /** The menuToolBar */
    protected JToolBar toolBar_;

    /** The tree */
    protected DynamicTree tree_;

    /** Returns an ImageIcon, or null if the path is invalid. */
    protected static ImageIcon createIcon(String imageName) {
        if (imageName != null) {
            URL urlFile = null;
            urlFile = Thread.currentThread().getContextClassLoader().getResource(imageName);            
            if(urlFile==null){            
				try {
					urlFile = new URL(imageName);
				} catch (java.net.MalformedURLException e) {
                    System.out.println(imageName + ": Malformed URL !");
				}
            }
            if(urlFile!=null){
                return new ImageIcon(urlFile);
            }
        }
        return null;
    }

    protected void createFileMenu() {
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);

        JMenuItem menuItem = null;
        JButton button = null;

        Action loadConfigurationFileAction = new LoadConfigurationFileAction("Load a configuration file", createIcon("empty.png"), "Load a new configuration file", new Integer(KeyEvent.VK_O), tree_, this);
        Action refreshAction = new RefreshAction("Refresh the tree", createIcon("icons/reload.png"), "Refreshes the tree", new Integer(KeyEvent.VK_R), tree_);
        Action debugConsoleAction = new DisplayDebugConsoleAction("Display the debug console", createIcon("empty.png"), "Display the debug console", new Integer(KeyEvent.VK_D), tree_);
        Action quitAction = new QuitAction("Quit", createIcon("icons/power.png"), "Quit the application", new Integer(KeyEvent.VK_Q), tree_);
        
        // Add to the menubar

        menuItem = new JMenuItem(loadConfigurationFileAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK));
        fileMenu.add(menuItem);

        menuItem = new JMenuItem(refreshAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.CTRL_MASK));
        fileMenu.add(menuItem);

        menuItem = new JMenuItem(debugConsoleAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, Event.CTRL_MASK));
        fileMenu.add(menuItem);

        fileMenu.addSeparator();

        menuItem = new JMenuItem(quitAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, Event.ALT_MASK));
        fileMenu.add(menuItem);

        // Adding to the toolbar

        button = new JButton(quitAction);
        button.setText("");
        toolBar_.add(button);

        button = new JButton(refreshAction);
        button.setText("");
        toolBar_.add(button);

        menuBar_.add(fileMenu);
    }

    protected void createTreeMenu() {
        JMenu treeMenu = new JMenu("Tree");
        treeMenu.setMnemonic(KeyEvent.VK_T);

        JMenuItem menuItem = null;
        JButton button = null;

        // To load a new IOR file
        // Add to menu
        Action loadIORFileAction = new LoadIORFileAction("Add a CORBA Object", createIcon("empty.png"), "Load a new IOR file", new Integer(KeyEvent.VK_L), tree_, this);
        menuItem = new JMenuItem(loadIORFileAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, Event.CTRL_MASK));
        treeMenu.add(menuItem);

        menuBar_.add(treeMenu);
    }

    /**
     * Constructor
     */
    public Menu(DynamicTree tree) {
        tree_ = tree;
        menuBar_ = new JMenuBar();
        toolBar_ = new JToolBar();
        toolBar_.setFloatable(false);
        createFileMenu();
        createTreeMenu();
    }

    /**
     * Gets the GUI menu bar
     *
     * @return The associated JMenuBar
     */
    public JMenuBar getMenuBar() {
        return menuBar_;
    }

    /**
     * Gets the GUI tool bar
     *
     * @return The associated JToolBar
     */
    public JToolBar getToolBar() {
        return toolBar_;
    }

}
