/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.services.pss.common.lib;

/**
 *  This is a base class for Storage Objects implemented with JDO.
 */
public abstract class StorageObjectBase
           implements org.objectweb.openccm.services.pss.common.api.StorageObject
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Storage Object PID
     */
    protected byte[] _pid;

    /**
     * Storage Home Base.
     */
    private org.omg.CosPersistentState.StorageHomeBase _storage_home;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public StorageObjectBase()
    {
        _pid = null;
        _storage_home = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.StorageObject
    //
    // ==================================================================

    /**
     * Destroy the associated Storage Object.
     */
    public abstract void
    destroy_object();

    /**
     * Test if the target incarnation represents an actual storage object.
     *
     * @return True if the target incarnation represents an actual storage object, else false.
     */
    public boolean
    object_exists()
    {
        // Don't know how to implement this method.
        return true;
    }

    /**
     * Get the PID.
     */
    public abstract byte[]
    get_pid();

    /**
     * Get the short PID.
     */
    public byte[]
    get_short_pid()
    {
        return org.objectweb.openccm.services.pss.common.lib.PIDHelper.get_short_pid( get_pid() );
    }

    /**
     * Get the storage home instance that manages the target storage object instance.
     *
     * @return The storage home instance.
     */
    public org.omg.CosPersistentState.StorageHomeBase
    get_storage_home()
    {
        return _storage_home;
    }

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.services.pss.common.api.StorageObject
    //
    // ==================================================================

    /**
     * Initialize the storage object.
     *
     * @param storage_home - The storage home managing this object.
     */
    public void
    initialize(org.omg.CosPersistentState.StorageHomeBase storage_home)
    {
        _storage_home = storage_home;
    }
}
