/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.api;

/** The console's imports */
import org.objectweb.util.browser.gui.core.DynamicTree;
import org.objectweb.util.browser.gui.popup.AdvancedJPopupMenu;

/**
 * Standard interface for JPopupMenu factories.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */

public interface MenuFactory {

    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String MENU_FACTORY = MenuFactory.class.getName();

    /**
     * Return a JPopupMenu associated to the param's object.
     *
     * @param object The object you want to create the associated menu
     * @param tree The tree on which the popup menu will be displayed
     * @return The associated menu.
     */
    public AdvancedJPopupMenu newMenu(Object object, DynamicTree tree);

    /**
     * Return a JPopupMenu associated to the param's object.
     * 
     * @param tree The tree on which the popup menu will be displayed
     * @return The parent menu.
     */
    public AdvancedJPopupMenu newParentMenu(DynamicTree tree);

}
