/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.panel;

/** To use the PanelFactory interface */
import org.objectweb.util.browser.api.PanelFactory;

/** To use the NodeViewPanel abstract class */
import org.objectweb.util.browser.gui.panel.NodeViewPanel;

/**
 * Basic implementation for Panel factories.
 *
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 *
 */
public class DefaultPanelFactory implements PanelFactory {

    /** The Associated class */
    protected Class class_ = null;

    /**
     * Return an JPanel associated to the param's object.
     *
     * @return The associated panel.
     */
    public NodeViewPanel newPanel(Object object) {
        if (class_ != null) {
            try {
                return (NodeViewPanel) class_.newInstance();
            } catch (java.lang.InstantiationException e) {
                System.out.println(class_.getName() + " : Instanciation exception");
            } catch (java.lang.IllegalAccessException e) {
                System.out.println(class_.getName() + " : Illegal access exception");
            }
        }
        return null;
    }

    /**
     * Fixes the panel class to display
     * 
     * @param theClass The class to instantiate
     */
    public void setClassName(Class theClass) {
        class_ = theClass;
    }

}
