/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.popup;

/** To use the AssociatedMenu interface */
import org.objectweb.util.browser.api.AssociatedMenu;

/** To use the PropertyLoader interface */
import org.objectweb.util.browser.api.BrowserProperty;

/** To use the Entry interface */
import org.objectweb.util.browser.naming.api.Entry;

/** To use the MenuFactory interface */
import org.objectweb.util.browser.api.MenuFactory;

/** To use the AssociatedElementImpl */
import org.objectweb.util.browser.gui.common.AssociatedElementImpl;

/** To use the DynamicTree class */
import org.objectweb.util.browser.gui.core.DynamicTree;

/** To use the JPopupMenu class */
import org.objectweb.util.browser.gui.popup.AdvancedJPopupMenu;

/**
 * Implementation of AssociatedMenu API without any cache 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class AssociatedMenuImpl
    extends AssociatedElementImpl
    implements AssociatedMenu {

    /** The tree */
    protected DynamicTree tree_ = null;

    /**
     * Constructs a AssociatedMenuImpl and initializes cache
     */
    public AssociatedMenuImpl(BrowserProperty loader, DynamicTree tree) {
        super();
        context_ = new MenuPropertyContainer(loader);
        tree_ = tree;
    }

    /** 
     * Returns the menu associates to the node
     * Equivalent to a getIcon(node,false) call
     *
     * @param node The node 
     * @return The associated menu or null if no context is found
     */
    public AdvancedJPopupMenu getMenu(Object node) {
        return getMenu(node, false);
    }

    /** 
     * Returns the menu associates to the node
     *
     * @param node The node 
     * @param disabledJavaLangObject If true, then java.lang.Object is not consider as a valid entry
     * @return The associated menu or null if no context is found
     */
    public AdvancedJPopupMenu getMenu(
        Object node,
        boolean disabledJavaLangObject) {
        if (context_ != null) {
            Entry entry = context_.getRecursiveProperty(node);
            if (entry != null && !(entry.getOWName().getOWId().equals("java.lang.Object") && disabledJavaLangObject)) {
                MenuFactory menuFactory = (MenuFactory) entry.getOWValue();
                return menuFactory.newMenu(node, tree_);
            }
        }
        return null;
    }

    /**
     * Returns the menu for childrens of the given node at the tree level
     * @param parent The object containing the parent menu
     * @param node The node on which applying the actions
     * @return The associated menu or null if no context is found
     */
    public AdvancedJPopupMenu getParentMenu(Object parent) {
        if (context_ != null) {
            Entry parentFactoryEntry = context_.getRecursiveProperty(parent);
            if (parentFactoryEntry != null) {
                MenuFactory parentFactory = (MenuFactory) parentFactoryEntry.getOWValue();
                return parentFactory.newParentMenu(tree_);
            }
        }
        return null;
    }

}
