/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2002 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cpp.lib;

// Package dependencies.
import org.objectweb.util.cmdline.api.Iterator;
import org.objectweb.util.cmdline.lib.DefaultOptionBase;
import org.objectweb.util.cpp.api.PreprocessorApplication;

/**
 * Implementation of the --cpp option.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class OptionCPP
     extends DefaultOptionBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the PreprocessorApplication. */
    protected PreprocessorApplication application_;

    // ==================================================================
    //
    // Constructos.
    //
    // ==================================================================

    /**
     * The constructor with the owner PreprocessorApplication.
     *
     * @param application Reference to the PreprocessorApplication.
     */
    public
    OptionCPP(PreprocessorApplication application)
    {
        // Calls the DefaultOptionBase constructor.
        super();

        setLabels(new String[] { "--cpp" });
        setArguments(new String[] { "path" });
        setDescription(new String[] { "Set the path to the external C preprocessor used"});
        setMandatory(false);

        // Inits internal state.
        application_ = application;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for class org.objectweb.util.lib.DefaultPrintableBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        checkAlreadySet(iterator);
        String path = consumeArgument(iterator);
        application_.setPreprocessor(new PreprocessorCPP(path));
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
