/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.monolog.provider.api;

import java.util.Properties;

import org.objectweb.util.monolog.api.LevelFactory;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * @author Jerome Offroy
 * 
 */
public interface LoggerProvider { //TODO split into multiple interfaces

    /***********************************************************************
     * Factories  interface
     **********************************************************************/
    /**
     * Shortcut that returns the LevelFactory
     */
    public LevelFactory getLevelFactory();
    
    /**
     * Configure Logger for OpenCCM modules
     */
    public void configure(String file, boolean useclasspath);

    /**
     * returns list of properties for logging system
     */
    public Properties getProperties();

    /**
    * returns the unique LoggerFactory used in OpenCCM.
    * initialize it if not already done.
    */
    public LoggerFactory getLoggerFactory();

    /***********************************************************************
     * Logger  interface
     **********************************************************************/
    /**
    * Shortcut to get the Logger by its topic name.
    * @param topic is the topic of the returned logger
    * @return always a logger instance (never null value).
    */
    public Logger getLogger(String topic);

    /**
     * Shortcut to get the Logger by its topic name and  by class .
     * @param topic is the topic of the returned logger
     * @param caller class reference 
     * @return Logger
     */
    public Logger getLogger(String topic, Class caller);

    /**
     * Method createLogger return a new Logger with specified topic.
     * @param topic
     * @param caller
     * @param description
     * @return Logger
     */
    public Logger createLogger(String topic, Class caller, String[] description);

    /***********************************************************************
     * Topics interface
     **********************************************************************/
    /**
     * get Topic Level for a specified Topic
     */
    public String getTopicLevel(String topic);

    /**
     * get all Topics from Logger Factory(Monolog). 
     * Assumes that all Loggers are TopicalLoggers.
     */
    public String[] getTopics();

    /**
     * set Topic Level for a specified Topic
     */
    public void setTopicLevel(String topic, String l);

}