/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.monolog.provider.api;

import java.util.HashMap;

/**
 * @author Jerome Offroy
 *
 */
public interface TopicProvider {
    /**
     * Method createTopic used to create a new topic with its description and its Class caller reference.
     * @param name of the new Topic
     * @param ref reference of the caller Class
     * @param description of the new Topic
     */
    public String createTopic(String name, Class ref, String description[]);
    
    /**
     * Method getTopic return the Topic associated to a Class reference.
     * @param ref Class 
     * @return String Topic
     */
    public String getTopic(Class ref);
    
    /**
     * Method getTopics lists all the created Topic.
     * @return String[] list of Topics
     */
    public String [] getTopics();
    
    /**
     * Method getTopicDescription gets description for a Topic.
     * @param name of the Topic
     * @return String[] description
     */
    public String[] getTopicDescription(String name);
    
    /**
     * get Topics used by reference class
     * @return HashMap
     */
    public HashMap getTopicsByRef() ;
    
}