/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/


package org.objectweb.ccm.descriptor;

import java.util.zip.*;

/**
 * The goal of thhis class is to provide facilities to
 * retrieve a file into a zip file.
 */
public class ZipEntryRetriever {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    //The main zip file
    private ZipFile zf;
    //The translation table to convert file path
    private java.util.Hashtable table;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Internal method to setup the convert table
     */
    private void 
    setIdentifierTable()
    {
      String entryName;
      
      for(java.util.Enumeration enum=zf.entries();
            enum.hasMoreElements();)
        {
            entryName=((java.util.zip.ZipEntry)enum.nextElement()).getName();               
            table.put(entryName.replace('\\','/').trim(),entryName);                
        }
                
    }    
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    /**
     * Main constructor taking in prameter the zip file
     * to wrap
     * @param zf The zip file to wrap
     */
    public ZipEntryRetriever(ZipFile zf){
     this.zf=zf;   
     table=new java.util.Hashtable();
     setIdentifierTable();
    }


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Return the ZipEntry denote by the given entryName. Directory delimiters
     * can indefferently be '\' or '/' or a mix of both. In all case the entryName
     * is translate in a internal representation to warrant the file resolution.
     * @param entryName the entry file name.
     */
    public ZipEntry
    retrieveZipEntryIgnoringDelimiterType(String entryName)
    throws java.util.zip.ZipException, java.io.IOException
    {
       if(!table.containsKey(entryName.replace('\\','/').trim()))
        throw new java.util.zip.ZipException("Cannot found zip entry:"+entryName) ;
             
       String originalName=(String)table.get(entryName.replace('\\','/'));       
       return zf.getEntry(originalName);
    }

    /**
     * Return an InputStream open on the file denoted by the entryName. 
     * retrieveZipEntryIgnoringDelimiterType(String entryName) is used 
     * to found the zip entry.
     * @param entryName the entry file name.
     */
    public java.io.InputStream
    getZipEntryISIgnoringDelimiterType(String entryName)
    throws java.util.zip.ZipException,java.io.IOException
    {
       return zf.getInputStream(retrieveZipEntryIgnoringDelimiterType(entryName));
    }
}