/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.ast.lib;

/** Used to access AST InterfaceDecl. */
import org.objectweb.openccm.ast.api.InterfaceDecl;

/** Used to access AST OperationDecl. */
import org.objectweb.openccm.ast.api.OperationDecl;

/**
 * InterfacePortDeclImpl is a wrapper class for
 * IDL interface port declarations.
 *
 * Inherits from:
 * - DeclarationImpl as provides are declarations,
 * - ClientOperationMapping
 *   to obtain the equivalent client operation mapping.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public abstract class InterfacePortDeclImpl
                extends DeclarationImpl
                implements org.objectweb.openccm.ast.api.InterfacePortDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The port interface type. */
    protected InterfaceDeclImpl interface_;

    /** Client operation mapping. */
    protected OperationDecl[] client_mapping_;

    /** The local interface mapping. **/
    private InterfaceDeclImpl local_mapping_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the provides declaration.
     */
    protected
    InterfacePortDeclImpl(Repository rep,
                          ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        interface_ = null;
        client_mapping_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.WithDependencies
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.Declaration
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.InterfacePortDecl
    //
    // ==================================================================

    /**
     * Set the associated InterfaceDecl.
     *
     * @param itf The associated InterfaceDecl.
     */
    public void
    setInterface(InterfaceDecl itf)
    {
        if(itf != null)
        {
            interface_ = (InterfaceDeclImpl)itf;
        }
    }

    /**
     * Obtain the associated InterfaceDecl.
     *
     * @return The associated InterfaceDecl.
     */
    public InterfaceDecl
    getInterface()
    {
        return interface_;
    }

    /**
     * Obtain the local mapping. (CCM_<name> interface)
     *
     * @return The local consumer mapping.
     */
    public InterfaceDecl
    getLocalMapping()
    {
        if (local_mapping_==null)
        {
            // load the local consumer mapping for this event.
            String parent_base_id =  getInterface().getParent().getId();
            int idx = parent_base_id.lastIndexOf(':');
            parent_base_id = parent_base_id.substring(0, idx);
            String id = parent_base_id + "/CCM_" + getInterface().getName() + ":" + getVersion();
            local_mapping_ = (InterfaceDeclImpl)getRepository().loadMapping(getInterface().getParent(), id);
        }
        return local_mapping_;
    }
}
