/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.openccm.generator.common.api;


/** To access all the AST Declaration. */
import org.objectweb.openccm.ast.api.Declaration;

/** To access all the AST Scope. */
import org.objectweb.openccm.ast.api.Scope;

/** To use List. */
import java.util.List;

/** To use GenerationException. */
import org.objectweb.openccm.generator.common.lib.GenerationException;


/**
 * This interface allows naviguation in the OpenCCM AST
 * and calls to map files.
 **/
public interface GeneratorBase
         extends org.objectweb.openccm.generator.common.api.Generator
{
    /**
     * Initialize the generator.
     **/
    public void
    initialize();

    /**
     * Obtains the OpenCCM's Abstract Syntax Tree.
     *
     * @return The OpenCCM's Abstract Syntax Tree.
     */
    public org.objectweb.openccm.ast.api.AST
    getAST();
    
    /**
     * Is the declaration a business (i.e. a non system) declaration ?
     *
     * @param decl - The declaration.
     *
     * @return True if the declaration is business declaration, false otherwise.
     **/
    public boolean
    isBusinessDeclaration(Declaration decl);

    /**
     * Visit a  declaration in the repository
     *
     * @param obj - The declaration to visit.
     **/
    public void
    visit(Declaration obj);

    /**
     * Visit dependencies tree.
     *
     * @param decl - The declaration to visit.
     **/
    public void
    visit_dep_tree(Declaration decl);

    /**
     * Visit dependencies tree.
     *
     * @param childs - Declarations to visit.
     **/
    public void
    visit_dep_tree(List childs);

    /**
     * Visit declarations.
     *
     * @param vect - A List containing Declarations to visit.
     **/
    public void
    visit(List vect);

    /**
     * Visit a single contained declaration
     *
     * @param contained - The declaration to visit.
     * @param extension - used to map id
     **/
    public void
    visitContained(Declaration contained, String extension);

    /**
     * Get a declaration.
     * name must be a valid declaration name.
     *
     * @param name - The name of the declaration to get.
     *
     * @return The matching declaration.
     **/
    public Declaration
    getDeclaration(String name)
    throws GenerationException;

    /**
     * Get a declaration from a scope.
     * name must be a valid declaration name.
     *
     * @param scope - The scope to visit.
     * @param name - The name of the declaration to get.
     *
     * @return The matching declaration.
     **/
    public Declaration
    getDeclaration(Scope scope, String name)
    throws GenerationException;

    /**
     * Get contents of the scope matching limited_types.
     *
     * @param scope - The scope to visit.
     * @param limited_types - A logical combination of DeclarationKind.
     *
     * @return The list of contained declarations.
     **/
    public List
    getDeclarations(Scope scope, long limited_types)
    throws GenerationException;

    /**
     * Get all declarations matching with limited_types contained in the file scope,
     * including sub-scopes.
     *
     * @param limited_types - A logical combination of DeclarationKind.
     *
     * @return The list of founded declarations.
     **/
    public List
    getAllDeclarations(Scope scope, long limited_types)
    throws GenerationException;
}
