/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.generator.java.ast.api;

import org.objectweb.openccm.generator.java.ast.lib.ModifierKindImpl;
import java.util.ArrayList;

/**
 * This is a base interface for java Objects.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public interface ObjectBase
{
    /**
     * Add a comment.
     *
     * @param comment - The comment to add.
     */
    public void
    addComment(String comment);

    /**
     * Get all comments.
     *
     * @return The list of comments.
     */
    public ArrayList
    getComments();

    /**
     * Set its name.
     *
     * @param name - The name to set.
     */
    public void
    setName(String name);

    /**
     * Obtain its name.
     *
     * @return Its name.
     */
    public String
    getName();

    /**
     * Set its modifier.
     *
     * @param mk - The modifier to set.
     */
    public void
    setModifier(ModifierKindImpl mk);

    /**
     * Obtain its modifier.
     *
     * @return Its modifier.
     */
    public ModifierKindImpl
    getModifier();

    /**
     * Set final this object.
     *
     * @param b - The value to set.
     */
    public void
    setFinal(boolean b);

    /**
     * Is this object final ?
     *
     * @return True if it is final, else false.
     */
    public boolean
    isFinal();
}
