/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.generator.psdl.lib;

// Package dependencies
import org.objectweb.openccm.ast.api.DeclarationKind;


public class PSDLGenerator
     extends org.objectweb.openccm.generator.idl.lib.IDL3Generator
  implements org.objectweb.openccm.generator.psdl.api.PSDLGenerator
{

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public PSDLGenerator(org.objectweb.openccm.ast.api.AST ast)
    {
        // Call the IDL3Generator constructor
        super(ast);

        // Init internal states
        translator_ = new org.objectweb.openccm.generator.translator.ast2idl.lib.AST_PSDLTranslator();
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Initialize the generator.
     **/
    public void
    initialize(String outputfile, String app_name)
    {
        java.util.List list = new java.util.ArrayList();

        /** Initialize environment **/

        // Templates to use.
        list.clear();
        list.add("org/objectweb/openccm/generator/common/common.vm");
        list.add("org/objectweb/openccm/generator/idl/idl3.vm");
        list.add("org/objectweb/openccm/generator/psdl/psdl.vm");
        setLibrary(list);

        // The output file
        open(outputfile, "out");

        // call the common generator init method
        super.init();

        /** Initialize velocity context **/
        put("which", app_name);
        put("gen", this);
        put("translator", translator_);
        put("declarator", "");
        put("limited_types" , new Long(org.objectweb.openccm.ast.api.DeclarationKind.dk_psdl));
    }

    // ===========================================================
    //
    // Useful methods for PSDL Mapping.
    //
    // ===========================================================

    /**
     * Utility method to check forward declarations.
     *
     * @return The associated search mask.
     **/
    public long
    forward_check()
    {
        return ( DeclarationKind.dk_abstract_storage_home +
                 DeclarationKind.dk_abstract_storage_type +
                 DeclarationKind.dk_storage_type );
    }

}
