/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet, Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.ir3;

//Package dependencies
import org.omg.CORBA.*;

/** 
 * Abstract base class for anonymous CORBA::IDLType objects.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.5
 */

abstract public class AnonymousIDLType
                extends IRObject_impl
                implements IDLTypeOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** TODO */
    public
    AnonymousIDLType(IFR ifr)
    {
        // Calls the IRObject_impl contructor.
        super(ifr);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * TODO
     */
    protected boolean
    isUnderDestruction()
    {
        // Returns true if this object is already in destruction.
        if(super.isUnderDestruction()) return true;

        // If dependencies to this object exist then
        if (getReferencedByCounter() != 0)
        {
            // Visits all IR objects referencing this object.
            org.objectweb.util.ref.api.Referencable[] referenced_by = getReferencedBy();
            for(int i=0; i<referenced_by.length; i++) {
                // Returns false if one of the referencing object is not
                // under destruction.
                if(!((IRObject_impl)(referenced_by[i])).isUnderDestruction())
                    return false;
            }
        }
        return true;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // For org.objectweb.util.ref.api.Referencable interface.
    //
    // ==================================================================

    /** 
     * Removes a reference.
     *
     * @param ref The Referencable object to remove.
     *
     * @postcondition if ref was previously added then
     *                getReferencedByCounter() == old.getReferencedByCounter() - 1
     */
    public void
    removeReferencedBy(org.objectweb.util.ref.api.Referencable ref)
    {
        super.removeReferencedBy(ref);

        // If there is no more dependency then
        // destroy this anonymous CORBA::IDLType.
        if (getReferencedByCounter() == 0)
            destroy();
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    // same as in the IRObject_impl class.

    // ==================================================================
    //
    // For CORBA::IDLType interface.
    //
    // ==================================================================

    // implemented into subclasses.
}
