/*====================================================================

ObjectWeb Util Package.
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

--------------------------------------------------------------------
$Id: StringBufferHelper.java,v 1.1 2004/02/13 17:46:07 rouvoy Exp $
====================================================================*/

package org.objectweb.util.misc.lib;

/**
 * Some helper functions for the java.lang.StringBuffer class.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public abstract class StringBufferHelper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /**
     * Appends a string to the given string buffer.
     *
     * @param sb The string buffer.
     * @param string The string.
     */
    public static void append(StringBuffer sb, String string) {
        sb.append('\"');
        sb.append(string);
        sb.append('\"');
    }

    /**
     * Appends a string array to the given string buffer.
     *
     * @param sb The string buffer.
     * @param strings The string array.
     */
    public static void append(StringBuffer sb, String[] strings) {
        sb.append('{');
        if(strings.length > 0) {
            append(sb, strings[0]);
            for(int i=1; i<strings.length; i++) {
                sb.append(',');
                append(sb, strings[i]);
            }
        }
        sb.append('}');
    }

    /**
     * Appends a <name,value> pair to the given string buffer.
     *
     * @param sb The string buffer.
     * @param name The name.
     * @param value The value.
     */
    public static void append(StringBuffer sb, String name, boolean value) {
        sb.append(name);
        sb.append('=');
        sb.append(value);
        sb.append(',');
    }

    /**
     * Appends a <name,value> pair to the given string buffer.
     *
     * @param sb The string buffer.
     * @param name The name.
     * @param value The value.
     */
    public static void append(StringBuffer sb, String name, int value) {
        sb.append(name);
        sb.append('=');
        sb.append(value);
        sb.append(',');
    }

    /**
     * Appends a <name,value> pair to the given string buffer.
     *
     * @param sb The string buffer.
     * @param name The name.
     * @param value The value.
     */
    public static void append(StringBuffer sb, String name, String value) {
        sb.append(name);
        sb.append('=');
        append(sb, value);
        sb.append(',');
    }

    /**
     * Appends a <name,value> pair to the given string buffer.
     *
     * @param sb The string buffer.
     * @param name The name.
     * @param value The value.
     */
    public static void append(StringBuffer sb, String name, String[] value) {
        sb.append(name);
        sb.append('=');
        append(sb, value);
        sb.append(',');
    }

    /**
     * Appends a given array of strings to a given string buffer
     * and separates items with a given separator.
     *
     * @param sb The given string buffer.
     * @param strings The given array of strings.
     * @param separator The given sperator.
     */
    public static void append(StringBuffer sb, String[] strings, String separator) {
        if(strings.length > 0) {
            sb.append(strings[0]);
            for(int i=1; i<strings.length; i++) {
                sb.append(separator);
                sb.append(strings[i]);
            }
        }
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
