/*====================================================================

ObjectWeb Util Ref Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

--------------------------------------------------------------------
$Id: ReferencableBase.java,v 1.1 2004/02/13 17:46:08 rouvoy Exp $
====================================================================*/

package org.objectweb.util.ref.lib;

import java.util.Vector;

import org.objectweb.util.ref.api.Referencable;

/**
 * Abstract base implementation of Referencable objects.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
abstract public class ReferencableBase
           implements Referencable
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** List of Referencable objects. */
    private Vector referenced_by_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public ReferencableBase() {
        // Inits internal state.
        referenced_by_ = new Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    /**
     * Obtains its identity.
     *
     * Must be defined in subclasses.
     *
     * @return Its identity.
     */
    public abstract String getIdentity();

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.ref.api.Referencable
    //
    // ==================================================================

    /**
     * Obtains the reference counter,
     * i.e. the number of Referencable objects that reference it.
     *
     * @return The number of references.
     */
    public int getReferencedByCounter() {
        return referenced_by_.size();
    }

    /**
     * Obtains all its added Referencable objects.
     *
     * @return All added Referencable objects.
     */
    public Referencable[] getReferencedBy() {
       return (Referencable[])referenced_by_.toArray(new Referencable[0]);
    }

    /**
     * Adds a reference.
     *
     * @param ref The Referencable object to add.
     *
     * @postcondition getReferencedByCounter() == old.getReferencedByCounter() + 1
     */
    public void addReferencedBy(Referencable ref) {
        referenced_by_.addElement(ref);
    }

    /**
     * Removes a reference.
     *
     * @param ref The Referencable object to remove.
     *
     * @postcondition if ref was previously added then
     *                getReferencedByCounter() == old.getReferencedByCounter() - 1
     */
    public void removeReferencedBy(Referencable ref) {
        referenced_by_.removeElement(ref);
    }
}
