/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.trace.PI;

/* MONOLOG import */
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
* The Client side interceptor used to log CORBA Client activity
 * @author <a href="mailto:offroy@lifl.fr">Jerome Offroy@lifl.fr</a>
 */
public class TraceCI extends org.omg.CORBA.LocalObject implements org.omg.PortableInterceptor.ClientRequestInterceptor{

    /**
      *  slot for trace 
      **/
    private int trace_slot;

    /** 
     * reference logger to use in this class
     */
	protected Logger logger_;

    /**
     * Level of logging for Client side Interceptor
     */
    private short logLevelClient;
    /* TODO change static def of ClientLevelLog */
    
    /**
     * used to get CORBA call information (client side)
     */
	protected ClientRequestInfoReader crir_;

    /**
     * Constructor
     */
    public TraceCI(int trace_slot, org.omg.PortableInterceptor.ORBInitInfo info) {
        /* MONOLOG logger initialization */
        logger_ = MonologLogger.loggerCI;    
        this.trace_slot = trace_slot;        
        logLevelClient = 1;
        this.crir_ = new ClientRequestInfoReader(logLevelClient);

        if (MonologLogger.enableTrace) 
            logger_.log(BasicLevel.DEBUG, "Init Trace Interceptor CORBA Client Side");
    }

    /**********************************************************************
     * Public Method 
     **********************************************************************/
    /**
     * Called when the client send a request to the server
     * @param ri the client request
     */        
    public void send_request(org.omg.PortableInterceptor.ClientRequestInfo ri){
        if (MonologLogger.enableTrace) 
            logger_.log(BasicLevel.INFO, crir_.displayClientRequestInfoMonolog(ri));
    }

    /**
     * used to query information during a Time Independent Invocation polling get reply sequence ??
     * @param ri the client request
     */
    public void send_poll(org.omg.PortableInterceptor.ClientRequestInfo ri) {
         if (MonologLogger.enableTrace) 
            logger_.log(BasicLevel.INFO, crir_.displayClientRequestInfoMonolog(ri));
    }

    /**
     * Called when the client receive a reply from the server
     * @param ri the client request
     */
    public void receive_reply(org.omg.PortableInterceptor.ClientRequestInfo ri) {
         if (MonologLogger.enableTrace) 
            logger_.log(BasicLevel.INFO, crir_.displayClientRequestInfoMonolog(ri));
    }

    /**
     *  Called when the client receive an exception from the server
     * @param ri the client request
     */
    public void receive_exception(org.omg.PortableInterceptor.ClientRequestInfo ri) {
         if (MonologLogger.enableTrace) 
            logger_.log(BasicLevel.ERROR, crir_.displayClientRequestInfoMonolog(ri,false));
    }

    /**
     *  Called when the client receive a message from the server 
     *  which is not a reply and not an exception
     * @param ri the client request
     */
    public void receive_other(org.omg.PortableInterceptor.ClientRequestInfo ri) {
         if (MonologLogger.enableTrace) 
            logger_.log(BasicLevel.INFO, crir_.displayClientRequestInfoMonolog(ri));
    }

    /**
     * return the name of the Logging Client Interceptor
     */
    public java.lang.String name() {
        return "LoggingClientInterceptor";
    }

    /**
     * Destroy the interceptor.
     */
    public void destroy() {
    }
}
